; =======================================================================
FUNCTION CCF_Read_One_Line, $
   lun         ; LUN of the CCF file
; Read one line from the given file

   s = ''

   if (NOT EOF(lun)) then begin

; read one line from file
      READF, lun, FORMAT='(A)', s
;;;      print, 's|'+s+'|'

; remove comments from the line
      pos = STRPOS(s, ';')
      if (pos GE 0) then s = STRMID(s, 0, pos)
      s = STRTRIM(s, 2)

   endif

   RETURN, s
END ; CCF_Read_One_Line

; =======================================================================
FUNCTION CCF_Get_Command, $
   s        ; a string from which a command is extracted
; get commands in the CCF file
; s: command line (start with '#')

   command = STRMID(s, 1, STRLEN(s)-1)
   command = STRUPCASE(STRTRIM(command, 2))

   return, command
END ; CCF_Get_Command

; =======================================================================
FUNCTION CCF_Type_Conversion, $
   s, $        ; input strings to be converted
   val_type    ; value type to be converted. Must be in upper case.
; convert string to specfied variable type
; error signal propergated by !ERROR_STATE structure

   ON_IOERROR, Bad_Exit
   MESSAGE, /RESET

   val = 0

   CASE val_type OF
      'BYTE': val = BYTE(FIX(s))
       'INT': val = FIX(s)
      'UINT': val = UINT(s)
      'LONG': val = LONG(s)
     'ULONG': val = ULONG(s)
     'FLOAT': val = FLOAT(s)
    'DOUBLE': val = DOUBLE(s)
    'STRING': begin
	s = repstr(s, '\\', string(10B))
	val = s
     end
    'LONG64': val = LONG64(s)
    'ULONG64': val = ULONG64(s)
       else : begin
         val = !FAIL
         lmessage, 1, 'Undefined type ('+val_type+').'
       end
   ENDCASE

Bad_Exit:

   ON_IOERROR, NULL

   return, val
END ; CCF_Type_Conversion

; =======================================================================
FUNCTION CCF_Read_Header, $
   lun, $   ; LUN of the CCF file
   error    ; 
            ; (returned) CCF Header structure

; reset error status
   error = !SUCCEED

; Header structure definition
   hdr = ccf_create_new_header()

; initialize local variables
   s = ''
   ss = ''
   com = ''
   WHILE (NOT EOF(lun)) do begin
      s = CCF_Read_One_Line(lun)
      if (STRLEN(s) LE 0) then continue

;;;      print, 'H|'+com+'|'+s+'|'
      if (STRPOS(s, '#') EQ 0) then begin
         com = CCF_Get_Command(s)
;;;         print, 'C|'+com+'|'

         if (com EQ 'ENDHEADER') then begin
            break
         endif else begin
            lmessage, 1, 'Unexpected '+com+' found. Ignored.'
         endelse
      endif

      ss = STRSPLIT(s, ' ', /EXTRACT)
      sk = STRUPCASE(ss[0])
      CASE sk OF
         'NAME' : hdr.NAME = STRUPCASE(ss[1])

       'SET_NO' : begin
            hdr.SET_NO = CCF_Type_Conversion(ss[1], 'INT')
            if (!ERROR_STATE.CODE NE 0) then begin
               lmessage, 1, STRING('Invalid SET_NO (', ss[1], ').', /PRINT)
               error = !FAIL
               return, error
            endif
         end

       'REV_NO' : begin
            hdr.REV_NO = CCF_Type_Conversion(ss[1], 'INT')
            if (!ERROR_STATE.CODE NE 0) then begin
               lmessage, 1, STRING('Invalid REV_NO (', ss[1], ').', /PRINT)
               error = !FAIL
               return, error
            endif
         end

       'STATUS' : hdr.STATUS = STRUPCASE(ss[1])

        'RANGE' : begin
            if (n_elements(ss) LT 3) then begin
               lmessage, 1, 'RANGE needs start and end time.'
               error = !FAIL
               return, error
            endif
            hdr.RANGE = CCF_Type_Conversion(ss[1:2], 'DOUBLE')
            if (!ERROR_STATE.CODE NE 0) then begin
               lmessage, 1, STRING('Invalid REV_NO (', ss[1:2], ').', /PRINT)
               error = !FAIL
               return, error
            endif
         end

      'CREATOR' : hdr.CREATOR = ss[1]

        'CDATE' : hdr.CDATE = ss[1]

       'FMTVER' : begin
            hdr.FMTVER = CCF_Type_Conversion(ss[1], 'INT')
            if (!ERROR_STATE.CODE NE 0) then begin
               lmessage, 1, STRING('Invalid FMTVER (', ss[1], ').', /PRINT)
               error = !FAIL
               return, error
            endif
         end

      'COMMENT' : begin
            pos = STRPOS(s, ' ')
            ss = STRTRIM(STRMID(s, pos), 2)
            if (hdr.COMMENT EQ '') then hdr.COMMENT = ss $
            else hdr.COMMENT = hdr.COMMENT + STRING(10B) + ss
         end
           else : begin
            lmessage, 1, 'Undefined Header Keyword ('+sk+') ignored.'
           end
      ENDCASE

   ENDWHILE

;;;   help, hdr, /struct

   return, hdr
END ; CCF_Read_Header

; =======================================================================
FUNCTION CCF_Allocate_Data_Const, $
   st, $    ; data type statement. Must be in upper case.
   error    ; returned error code

   error = !SUCCEED

   if (st[0] NE 'C') then begin
      lmessage, 1, 'Data type mismatch ('+st[0]+').'
      error = !FAIL
      return, error
   endif

   val_type = st[1]

   CASE val_type OF
      'BYTE': ccf = 0B
       'INT': ccf = 0
      'UINT': ccf = 0U
      'LONG': ccf = 0L
     'ULONG': ccf = 0UL
     'FLOAT': ccf = 0E
    'DOUBLE': ccf = 0D
    'STRING': ccf = ''
    'LONG64': ccf = 0L
    'ULONG64': ccf = 0UL
       else : begin
         lmessage, 1, 'Undefined type ('+val_type+').'
         ccf = !FAIL
         error = !FAIL
       end
   ENDCASE

;;; help, ccf
   return, ccf
end ; CCF_Allocate_Data_Const

; =======================================================================
FUNCTION CCF_Allocate_Data_Array, $
   st, $    ; data type statement. Must be in upper case.
   error    ; returned error code

; reset error code
   error = !SUCCEED

   if (st[0] NE 'A') then begin
      lmessage, 1, 'Data type mismatch ('+st[0]+').'
      error = !FAIL
      return, error
   endif

   ndim = n_elements(st)
   if (ndim LT 3) then begin
      lmessage, 1, 'Array dimmension needed ('+st+').'
      error = !FAIL
      return, error
   endif

   val_type = st[1]

   dim = CCF_Type_Conversion(st[2:(ndim-1)], 'LONG')
   if (!ERROR_STATE.CODE NE 0) then begin
      lmessage, 1, 'Array dimension error Error ('+st+').'
      error = !FAIL
      return, error
   endif

   CASE val_type OF
      'BYTE': ccf = MAKE_ARRAY(dim, /BYTE)
       'INT': ccf = MAKE_ARRAY(dim, /INTEGER)
      'UINT': ccf = MAKE_ARRAY(dim, /UINT)
      'LONG': ccf = MAKE_ARRAY(dim, /LONG)
     'ULONG': ccf = MAKE_ARRAY(dim, /ULONG)
     'FLOAT': ccf = MAKE_ARRAY(dim, /FLOAT)
    'DOUBLE': ccf = MAKE_ARRAY(dim, /DOUBLE)
    'STRING': ccf = MAKE_ARRAY(dim, /STRING)
    'LONG64': ccf = MAKE_ARRAY(dim, /L64)
    'ULONG64': ccf = MAKE_ARRAY(dim, /UL64)
       else : begin
         lmessage, 1, 'Undefined type ('+val_type+').'
         ccf = !FAIL
         error = !FAIL
       end
   ENDCASE

;;;   help, ccf
   return, ccf
end

; =======================================================================
FUNCTION CCF_Allocate_Data_Struct, $
   st, $    ; data type statement. Must be in upper case.
   lun, $   ; LUN of the input CCF file.
   error    ; returned error code

; reset error code
   if (st[0] NE 'S') then begin
      lmessage, 1, 'Data type mismatch ('+st[0]+').'
      error = !FAIL
      return, error
   endif

   if (n_elements(st) LT 2) then begin
      lmessage, 1, 'Number of structure element have to be given.'
      error = !FAIL
      return, error
   endif

   nstruct = CCF_Type_Conversion(st[1], 'UINT')
   if (!ERROR_STATE.CODE NE 0) then begin
      lmessage, 1, 'Data conversion failed ('+st[1]+').'
      error = !FAIL
      return, error
   endif

; initialize local variables
   s = ''
   ss = ''
   ccf = -1

   WHILE (NOT EOF(lun)) do begin
      s = CCF_Read_One_Line(lun)
      if (STRLEN(s) LE 0) then continue

;;;      print, 't|'+s+'|'
      if (STRPOS(s, '#') EQ 0) then begin
         com = CCF_Get_Command(s)
;;;         print, 'd|'+com+'|'

         if (com EQ 'ENDDEFINE') then begin
            break
         endif else begin
            lmessage, 1, 'Unexpected '+com+' found.'
            error = !FAIL
            return, error
         endelse
      endif

      ss = STRSPLIT(s, ' ', /EXTRACT)
      ss = STRUPCASE(ss)
      if (ss[0] EQ 'T') then begin
         tagname = ss[1]

         tagdata = -1
         CASE ss[2] OF
            'C'   :  tagdata = CCF_Allocate_Data_Const(ss[2:(n_elements(ss)-1)], error)
            'A'   :  tagdata = CCF_Allocate_Data_Array(ss[2:(n_elements(ss)-1)], error)
            else  :  lmessage, 1, 'Unknown tag data type'+ss[2]+'.'
         ENDCASE
         if (tagdata[0] EQ -1) then continue

         if (error EQ !FAIL) then return, error

         if (SIZE(ccf, /TNAME) EQ 'STRUCT') then begin
            ccf = CREATE_STRUCT(ccf, tagname, tagdata)
         endif else begin
            ccf = CREATE_STRUCT(tagname, tagdata)
         endelse
      endif

   ENDWHILE
   if (nstruct GT 1) then ccf = REPLICATE(ccf, nstruct)


;;;   help, ccf,/struct
   return, ccf
END ; CCF_Allocate_Data_Struct

; =======================================================================
FUNCTION CCF_Define_CCF_Type, $
   lun, $      ; LUN of the CCF file
   ccf_type, $ ; (returned) data type
   error       ; error code

; reset error code
   error = !SUCCEED


; intialize local variables
   s = ''
   ss = ''
   com = ''
   ccf_type = ''

   WHILE (NOT EOF(lun)) do begin
      s = CCF_Read_One_Line(lun)
      if (STRLEN(s) LE 0) then continue

;;;      print, 'T|'+s+'|'
      if (STRPOS(s, '#') EQ 0) then begin
         com = CCF_Get_Command(s)
;;;         print, 'ddd|'+com+'|'

         if (com EQ 'ENDDEFINE') then begin
            break
         endif else begin
            lmessage, 3, 'Unexpected '+com+' found. Ignored.', /CONTINUE
         endelse

      endif

      if (ccf_type EQ '') then begin
         ss = STRSPLIT(s, ' ', /EXTRACT)
         ss = STRUPCASE(ss)
         ccf_type = ss[0]

         if (ccf_type EQ 'S') then begin
            ccf = CCF_Allocate_Data_Struct(ss, lun)
            break
         endif else begin
            CASE ccf_type OF
               'C'   :  ccf = CCF_Allocate_Data_Const(ss, error)
               'A'   :  ccf = CCF_Allocate_Data_Array(ss, error)
               else  :  ccf_type = ''
            ENDCASE
            if (error EQ !FAIL) then begin
               ccf = !FAIL
               return, error
            endif
         endelse
      endif

   ENDWHILE

;;;   print, 'xxxxxxx', com, ccf_type
   if (ccf_type EQ '') then begin
      lmessage, 1, 'Failed to complete CCF data definition.'
      ccf = !FAIL
      error = !FAIL
   endif

   return, ccf
END ; CCF_Define_CCF_Type

; =======================================================================
FUNCTION CCF_Read_One_Data_Block, $
   lun, $      ; LUN of the CCF file
   ccf, $      ; (returned) ccf parameters
   error       ; error codes
; lun: 
; ccf: Data to be stored

; reset error codes.
   error = !SUCCEED

   if (SIZE(ccf, /TNAME) EQ 'UNDEFINED') then begin
      lmessage, 1, 'CCF data type undefined.'
      error = !FAIL
      return, error
   endif

   if ((SIZE(ccf))[0] EQ 0) then begin
      ccf_type = 'C'       ;  scholar
   endif else begin
      ccf_type = 'A'       ;  Array
   endelse

; check value type
   val_type = SIZE(ccf, /TNAME)

; initialize local variables
   s = ''
   ss = ''
   scat = ''
   com = ''

   WHILE (NOT EOF(lun)) do begin
      s = CCF_Read_One_Line(lun)
      if (STRLEN(s) LE 0) then continue

      if (STRPOS(s, '#') EQ 0) then begin
         com = CCF_Get_Command(s)
;;;         print, 'c|'+com+'|'

         if (com EQ 'ENDDATA') then begin
            break
         endif else begin
            lmessage, 3, 'Unexpected '+com+' found. Ignored.'
         endelse
      endif

      if (val_type EQ 'STRING') then scat = scat + STRING(10B) + s $
      else                           scat = scat + ' ' + s
   ENDWHILE
;;;   print, 'S|'+scat+'|'

   if (com NE 'ENDDATA') then begin
      lmessage, 1, 'Reached to EOF before #ENDDATA.'
      error = !FAIL
      return, error
   endif

   if (scat EQ '') then begin
      lmessage, 1, 'No data found before ENDDATA.'
      error = !FAIL
      return, error
   endif

   if (val_type EQ 'STRING') then ss = STRSPLIT(scat, STRING(10B), /EXTRACT) $
   else                           ss = STRSPLIT(scat, ' ', /EXTRACT)

   if (ccf_type EQ 'C') then begin
      val = CCF_Type_Conversion(ss[0], val_type)
      if (!ERROR_STATE.CODE NE 0) then begin
         lmessage, 1, 'Data conversion failed ('+ss[0]+').'
         error = !FAIL
         return, error
      endif
      ccf = val
   endif else if (ccf_type EQ 'A') then begin
      val = CCF_Type_Conversion(ss, val_type)
      if (!ERROR_STATE.CODE NE 0) then begin
         lmessage, 1, 'Data conversion failed ('+scat+').'
         error = !FAIL
         return, error
      endif
      nc = n_elements(ccf)
      nv = n_elements(val)
      if (nc NE nv) then begin
         msg = STRING(FORMAT='(A,I5,A,I5,A)', 'Array size inconsistent (', nc, ', ', nv, ').',/PRINT)
         lmessage, 1, msg
         error = !FAIL
         return, error
      endif
      ccf = REFORM(val, SIZE(ccf, /DIMENSION))
   endif

;;;   help, ccf

   return, ccf
END ; CCF_Read_One_Data_Block

; =======================================================================
FUNCTION CCF_Read_Structure_Data, $
   lun, $   ; LUN of the CCF file
   ccf, $   ; Data to be stored
   error    ; error status

; reset error codes.
   error = !SUCCEED

; check if the data type is structure
   if (SIZE(ccf, /TNAME) NE 'STRUCT') then begin
      lmessage, 1, 'CCF data type is not STRUCT.'
      error = !FAIL
      return, error
   endif

   nstruct = n_elements(ccf)

   tagname_list = TAG_NAMES(ccf)

; initialize local variables
   s = ''
   ss = ''
   com = ''

   WHILE (NOT EOF(lun)) do begin
      s = CCF_Read_One_Line(lun)
      if (STRLEN(s) LE 0) then continue

;;;      print, 'S|'+com+'|'+s+'|'
      if (STRPOS(s, '#') EQ 0) then begin
         com = CCF_Get_Command(s)
;;;         print, 'C|'+com+'|'

         if (com EQ 'ENDDATA') then begin
            break
         endif else begin
            lmessage, 3, 'Unexpected '+com+' found. Ignored.'
         endelse
      endif

      ss = STRSPLIT(s, ' ', /EXTRACT)
      ss = STRUPCASE(ss)
      if (ss[0] EQ 'T') then begin
         tagname = ss[1]

         tagidx = WHERE(tagname_list EQ tagname, cnt)
         if (cnt NE 1) then begin
            lmessage, 1, 'Invalid TAG name ('+tagname+').'
            lmessage, 1, 'Accepted TAG names are ('+STRING(tagname_list, /PRINT)+').'
            error = !FAIL
            return, error
         endif

         tagdata = ccf.(tagidx)
         tagdata = CCF_Read_One_Data_Block(lun, tagdata, error)
         if (error EQ !FAIL) then begin
            lmessage, 1, 'While reading '+tagname+'.'
            return, error
         endif
         ccf.(tagidx) = tagdata
         break
      endif

   ENDWHILE

;;;   help, ccf, /struct

   return, ccf

END; CCF_Read_Structure_Data

; =======================================================================
FUNCTION CCF_Read_CCF_File, $
   ccf_file, $       ; Name of CCF file to be read
   HEADER=header, $  ; (returned) CCF Header 
   ERROR=error       ; (returned) Error status (!SUCCEED: no error, !FAIL: Error)
                     ; (returned) CCF value (!FAIL on Error with Error status)

; reset error codes.
   error = !SUCCEED

; Starting message
   lmessage, 5, 'Read '+ccf_file

; open the CCF file
   OPENR, lun, ccf_file, /Get_LUN, ERROR=error
   if (error NE 0) then begin
      msg = STRING(FORMAT='(A, A, A, I, A, A)', $
         'Error to open CCF file (', ccf_file, '), error = ', error, !ERROR_STATE.msg,'.')
      lmessage, 1, !ERROR_STATE.msg
      error = !FAIL
      return, error
   endif

; initialize local variables
   s = ''
   ss = ''
   ccf_type = ''
   ccf = -1

   WHILE (NOT EOF(lun)) do begin
; read one line from the file
      s = CCF_Read_One_Line(lun)
      if (STRLEN(s) LE 0) then continue
;;;      print, '0|'+s+'|'

; search for command
      if (STRPOS(s, '#') EQ 0) then begin
         com = CCF_Get_Command(s)
;;;         print, 'c|'+com+'|'

; process according to the command keyword
         CASE com OF
            'HEADER'    :  begin
                  header = CCF_Read_Header(lun, error)
                  if (error EQ !FAIL) then break;
               end

            'DEFINE'    :  begin
                  ccf = CCF_Define_CCF_Type(lun, ccf_type, error)
                  if (error EQ !FAIL) then break;
               end

            'DATA'      :  begin
               error = !FAIL
               CASE ccf_type OF
                  'C'   :  ccf = CCF_Read_One_Data_Block(lun, ccf, error)
                  'A'   :  ccf = CCF_Read_One_Data_Block(lun, ccf, error)
                  'S'   :  ccf = CCF_Read_Structure_Data(lun, ccf, error)
                  else  :  begin
                     lmessage, 1, 'Data type undefined ('+ccf_type+').'
                     error = !FAIL
                  end
               ENDCASE
               if (error EQ !FAIL) then return, error;
            end

            'ENDHEADER' :  lmessage, 3, 'Unexpected '+com+' found.'
            'ENDDEFINE' :  lmessage, 3, 'Unexpected '+com+' found.'
            'ENDDATA'   :  lmessage, 3, 'Unexpected '+com+' found.'
            else        :  lmessage, 3, 'Unknown command '+com+' found.'
         ENDCASE
      endif

   ENDWHILE

   if (error EQ !FAIL) then return, error

   ;if (ccf_type EQ '') then begin
   ;   msg = STRING(FORMAT='(A, A, A)', 'No CCF data found in ', ccf_file, '.')
   ;   lmessage, 1
   ;endif
   
   FREE_LUN, lun
   
;;;   help, ccf
;;;   print, ccf

   RETURN, ccf

END ; CCF_Read_CCF_File
