; =======================================================================
FUNCTION CCF_Validate_Header, $
   header      ; CCF Header

; 06/05/03 I.Yamamura STATUS should be only either 'TEST' or 'SAFE'
; 08/02/20 NI moved from ccf_write_ccf_file.pro to ccf_validate_header.pro.

   h = header
   if (SIZE(h, /TNAME) NE 'STRUCT') then begin
      lmessage, 1, 'Header is not a STRUCT.'
      error = !FAIL
      return, error
   endif

   if (h.NAME EQ '') then begin
      lmessage, 1, 'NAME undefined.'
      return, !FAIL
   endif

   l = STRLEN(h.NAME)
   if (l LT 4) then begin
      lmessage, 1, 'NAME too short ('+h.NAME+').'
      return, !FAIL
   endif else if (l GT 5) then begin
      lmessage, 1, 'NAME too long ('+h.NAME+').'
      return, !FAIL
   endif else if (l EQ 5) then begin
      ss = STRMID(h.name, 4, 1)
      if ((ss NE 'S') AND (ss NE 'L') AND (ss NE '_')) then begin
         lmessage, 1, '5th char of NAME have to be either S or L or _ ('+h.NAME+').'
         return, !FAIL
      endif
   endif

   if (h.SET_NO LE 0 OR h.REV_NO LE 0) then begin
      lmessage, 1, STRING('SET_NO / REV_NO invalid (', h.SET_NO, ', ', h.REV_NO, ').', /PRINT)
      return, !FAIL
   endif
      
   if ((h.STATUS NE 'TEST') AND (h.STATUS NE 'SAFE')) then begin
      lmessage, 1, 'STATUS invalid ('+h.STATUS+').'
      return, !FAIL
   endif

   if (n_elements(h.RANGE) NE 2) then begin
      lmessage, 1, 'RANGE must have 2 elements.'
      return, !FAIL
   endif

   if (h.RANGE[0] LT 0 OR h.range[1] LT 0) then begin
      lmessage, 1, STRING('RANGE must be 0 or larger (', h.RANGE[0], ',', h.RANGE[1], ').', /PRINT)
      return, !FAIL
   endif

   if ((h.RANGE[1] GT 0) AND (h.RANGE[1] LE h.range[0])) then begin
      lmessage, 1, STRING('RANGE end time must be 0 or larger than start time (', h.RANGE, ').', /PRINT)
      return, !FAIL
   endif

   if (h.CREATOR EQ '') then begin
      lmessage, 1, 'CREATOR is undefined.'
      return, !FAIL
   endif

   if (h.COMMENT EQ '') then begin
      lmessage, 1, 'COMMENT is empty...'
   endif

   return, !SUCCEED
END ; CCF_Validate_Header
