; =======================================================================
;FUNCTION CCF_Validate_Header, $
;   header      ; CCF Header
; 08/02/20 NI moved to ccf_validate_header.pro


; =======================================================================
FUNCTION CCF_Write_Header, $
   lun, $   ; LUN of the CCF file
   header   ; CCF Header structure (assumed to be validated)

; 08/02/20 NI Modified output format (2F15.5 -> F15.5, x, F15.5)

   PRINTF, lun, '#HEADER'
   PRINTF, lun, FORMAT='(A-10, A)',      'NAME',    header.NAME
   PRINTF, lun, FORMAT='(A-10, I3.3)',   'SET_NO',  header.SET_NO
   PRINTF, lun, FORMAT='(A-10, I3.3)',   'REV_NO',  header.REV_NO
   PRINTF, lun, FORMAT='(A-10, A)',      'STATUS',  header.STATUS
   PRINTF, lun, FORMAT='(A-10, F15.5, x, F15.5)', 'RANGE',   header.RANGE
   PRINTF, lun, FORMAT='(A-10, A)',      'CREATOR', header.CREATOR
   PRINTF, lun, FORMAT='(A-10, A)',      'CDATE',   header.CDATE
   PRINTF, lun, FORMAT='(A-10, I3.3)',   'FMTVER',  header.FMTVER

   s = header.COMMENT
   WHILE (s NE '') do begin
      pos = STRPOS(s, STRING(10B))
      if (pos LT 0) then begin
         ss = s
         s = ''
      endif else begin
         ss = STRMID(s, 0, pos)
         l = STRLEN(s)
         if (l GT pos) then s = STRMID(s, pos+1, l-pos-1)
      endelse

      PRINTF, lun, FORMAT='(A-10, A)', 'COMMENT', ss
   ENDWHILE

   PRINTF, lun, '#ENDHEADER'

   return, !SUCCEED
END ; CCF_Write_Header


; =======================================================================
;FUNCTION CCF_Data_Definition_String, $
; 08/02/20 NI moved to ccf_data_defs.pro.

; =======================================================================
;FUNCTION CCF_Write_One_Data_Block, $
; 08/02/20 NI concatenated with CCF_Write_Structure and
;          divided into the definition (CCF_Write_Data_Definition)
;          and data writing (CCF_Write_Data_Body) parts.

; =======================================================================
;FUNCTION CCF_Write_Structure_Data, $
; 08/02/20 NI concatenated with CCF_Write_Structure and
;          divided into the definition (CCF_Write_Data_Definition)
;          and data (CCF_Write_Data_Body) writing parts.

; =======================================================================
FUNCTION CCF_Write_Data_Definition, lun, ccf, FMT = fmt, DIM = dim

; 08/02/20 NI Newly created.

defs = CCF_Data_Defs(ccf, FMT = fmt, DIM = dim, ERROR = error)

; Write definition
PRINTF, lun, STRING(10B)
PRINTF, lun, '#DEFINE'
PRINTF, lun, defs, format = '(A0)'
PRINTF, lun, '#ENDDEFINE'

return, !SUCCEED

END ; CCF_Write_Data_Definition


; =======================================================================
FUNCTION CCF_Write_Data_Body, lun, ccf, FMT = fmt_e, DIM = dim

; This function should be called after the execution of ccf_write_data_definition
; with FMT and DIM keywords.

; 08/02/20 NI Newly created.

if (SIZE(ccf, /TNAME) EQ 'STRUCT') then begin

	; Write data body for structures
	; ------------------------------
	nstruct = N_ELEMENTS(ccf)
	tagname_list = TAG_NAMES(ccf)
	ntag = n_elements(tagname_list)

	for i = 0L, ntag - 1 do begin
		tccf = ccf.(i)

		; For string data, substitute \\ for ^J
		; ------------------------------
		if (size(tccf, /tname) eq 'STRING') then begin
			tccf = repstr(tccf, string(10B), '\\')
		endif

		ndat = N_ELEMENTS(tccf)
		PRINTF, lun, STRING(10B)
		PRINTF, lun, '#DATA'
		PRINTF, lun, '   T ', tagname_list[i]

		if (ndat EQ 1) then begin
			; Constant
			PRINTF, lun, FORMAT='('+fmt_e[i]+')', tccf

		endif else begin

			if (nstruct EQ 1) then begin

				fmt = STRING('(', ndat, fmt_e[i], ')')
				PRINTF, lun, FORMAT=fmt, tccf

			endif else begin

				nel = ndat / nstruct
				fmt = STRING('(', nel, fmt_e[i], ')')
				tccf = reform(tccf, nel, nstruct, /OVERWRITE)

				for j = 0L, nstruct-1 do begin
					PRINTF, lun, FORMAT='("; ccf[",I,"]")', j
					PRINTF, lun, FORMAT=fmt, tccf[*, j]
				endfor

			endelse

		endelse

	PRINTF, lun, '#ENDDATA'

	endfor


endif else begin

	; Writing data body for variables
	; ------------------------------
	ndim = N_ELEMENTS(dim)
	ndat = N_ELEMENTS(ccf)

	PRINTF, lun, STRING(10B)
	PRINTF, lun, '#DATA'

	; For string data, substitute \\ for ^J
	; ------------------------------
	if (size(ccf, /tname) eq 'STRING') then begin
		ccf = repstr(ccf, string(10B), '\\')
	endif

	if (ndat EQ 1) then begin
		; Constant

		PRINTF, lun, FORMAT='(' + fmt_e + ')', ccf

	endif else begin
		; Array

		fmt = STRING('(', ndat, fmt_e, ')')

		PRINTF, lun, FORMAT=fmt, ccf

	endelse

	PRINTF, lun, '#ENDDATA'

endelse

return, !SUCCEED

END ; CCF_Write_Data_Body


; =======================================================================
FUNCTION CCF_Write_CCF_File, $
   ccf_file, $             ; CCF File Name
   ccf, $                  ; CCF Data
   header, $               ; CCF Header 
   OVERWRITE=overwrite     ; Allow to overwrite existing file.
                           ; (returned) Error status

; reset error codes.
   error = !SUCCEED

; Check Header
   error = CCF_Validate_Header(header)
   if (error EQ !FAIL) then begin
      lmessage, 1, 'Invalid Header.'
      error = !FAIL
      return, error
   endif

; Check if CCF data really exist
   if (SIZE(ccf, /TNAME) EQ 'UNDEFINED') then begin
      lmessage, 1, 'CCF is undefined.'
      error = !FAIL
      return, error
   endif

; Start Writing to file
; Check if there is a file with the same name
   if (NOT Keyword_Set(OVERWRITE) AND (FILE_TEST(ccf_file) EQ 1)) then begin
      lmessage, 1, 'File "'+ccf_file+'" exists.'
      lmessage, 1, 'Use /OVERWRITE option if you really want to overwrite.'
      error = !FAIL
      return, error
   endif

; open the CCF file in write mode
   OPENW, lun, ccf_file, /Get_LUN, ERROR=error
   if (error NE 0) then begin
      msg = STRING(FORMAT='(A, A, A, I, A, A)', $
         'Error to open CCF file (', ccf_file, '), error = ', error, !ERROR_STATE.msg,'.')
      lmessage, 1, !ERROR_STATE.msg
      error = !FAIL
      return, error
   endif

; Set Time Stamp
   CALDAT, SYSTIME(/JULIAN), mm, dd, yy, hh, ms, ss
   header.CDATE = STRING(FORMAT='(I4.4,"-",I2.2,"-",I2.2,"T",I2.2,":",I2.2,":",I2.2)', $
      yy, mm, dd, hh, ms, ss, /PRINT)

; Set FMTVER
   header.FMTVER = CCF_Current_FMTVER()

; Write Header
   error = CCF_Write_Header(lun, header)

; Write data definition and data body
	error = CCF_Write_Data_Definition(lun, ccf, FMT = fmt, DIM = dim)
	error = CCF_Write_Data_Body(lun, ccf, FMT = fmt, DIM = dim)

   FREE_LUN, lun
   
   RETURN, error

END ; CCF_Write_CCF_File
