;
; Swap_Fields.pro
; Feb. 6 2008
;
; Input :
;		str .. string variable, scalor or vector.
;		order .. integer or long vector variable.
;
; Optional input:
;		separator .. string used as a field separator.
;
; Example)
; 
; IDL> str = '0 1 2 3 4 5 6 7'
; IDL> print, swap_fields(str, [5, 2, 7])
; 5 2 7 0 1 3 4 6
; IDL>
; IDL> str = 'abc:def:ghi'
; IDL> print, swap_fields(str, [2, 1], separator = ':')
; ghi:def:abc
; IDL>
;


; Subroutine
; Move field "key" to be the first one.
; ------------------------------
function field_to_first, str, key, func = split_func

sortstr = strarr(n_elements(str))
for i = 0, n_elements(str)-1 do begin

	cmd = 'tmp = ' + split_func[0] + '(str[i], "' $
		+ split_func[1] + '", ' + split_func[2] + ')'
	result = execute(cmd)
	if (result ne 1) then begin
		message, 'Failed to execute ' + cmd, /cont
		return, -1
	endif

	case key of

		0 : begin
			; do nothing.
			sortstr[i] = str[i]
		end

		n_elements(tmp)-1 : begin
			sortstr[i] = tmp[key]
			for j = 0, n_elements(tmp)-2 do $
				sortstr[i] = sortstr[i] + split_func[1] + tmp[j]
		end

		else : begin
			sortstr[i] = tmp[key]
			for j = 0, key-1 do $
				sortstr[i] = sortstr[i] + split_func[1] + tmp[j]
			for j = key+1, n_elements(tmp)-1 do $
				sortstr[i] = sortstr[i] + split_func[1] + tmp[j]
		end

	endcase
endfor

return, sortstr

end

;
; Main procedure
; ------------------------------
function swap_fields, str, order, $
	separator = separator

if (not keyword_set(separator)) then separator = ' '

split_function = 'strsplit'
split_func_opt = '/extract'

split_func = [split_function,  separator, split_func_opt]

str_to_be_swapped = str

order = reverse(order)

for i = 0, n_elements(order)-1 do begin

	str_to_be_swapped = field_to_first(str_to_be_swapped, order[i], $
		func = split_func)

	; Shift the orders younger than the current sortkey.
	; ------------------------------
	index = where(order lt order[i], count)
	if (count ne 0) then order[index] ++

endfor

return, str_to_be_swapped

end
