pro fts_part1_output, savefilename,  addname = addname, psfile = psfile, fixmin = fixmin, fixmax = fixmax, wnmin = wnmin, wnmax = wnmax, txtfile = txtfile, txtch = txtch, defringe = defringe, mcal = mcal

;;
;; make output files from 'part.sav' file
;;

  if(n_params() ne 1) then begin
    print, '------------------------USAGE-----------------------------'
    print, 'fts_part1_output, savefilename [,/MCAL] [,ADDNAME = string] [,/PSFILE] [,FIXMIN = value] [,FIXMAX = value] [,WNMIN = value] [,WNMAX = value] [,/TXTFILE] [,TXTCH = integer vector] [,/DEFRINGE]'
    print, '----------------------------------------------------------'
    return
  endif

  cvsroot = GETENV('FISDR_ROOT')
  if(strmid(cvsroot, 0, 1, /reverse_offset) eq '/') then cvsroot = strmid(cvsroot, 0, strlen(cvsroot)-1L)

  nch_all = [60L, 45L]

;;;;;;;;;;;;;;;;;;;;;;;;
;; restore input file ;;
;;;;;;;;;;;;;;;;;;;;;;;;

  restore, savefilename

  if(keyword_set(addname) ne 1) then addname = ''
  length_filename = strlen(savefilename)
  filename_core = strmid(savefilename, 0, length_filename-4)+addname
 
  case datatype of
    'FIS_SW':idetect = 0L
    'FIS_LW':idetect = 1L
    else:begin
      print, 'datatype is wrong'
      stop
    endelse
  endcase

  nch = nch_all[idetect]
  nwn = n_elements(wn)

  case pktid_f of
    124L:imode = 0L   ;; SED mode
    14L:imode = 1L    ;; full-reso mode
    else:begin
      print, 'pktid_f is wrong'
      stop
    endelse
  endcase

  newspec = dblarr(nwn, 2, nch)

;;;;;;;;;;;;;;
;; defringe ;;
;;;;;;;;;;;;;;

  if(keyword_set(defringe)) then begin
    if(idetect eq 1L) then message, 'Defringe cannot apply for LW data'

    fringefile = cvsroot+'/fts/fts_part2/data/fringe_FRdivSED.dat'

    if(imode eq 0L) then $
      readcol, fringefile, format = '(d,x,x,d)', fringe_wn, fringe_calc, comment = '#' $
    else $
      readcol, fringefile, format = '(d,x,d)', fringe_wn, fringe_calc, comment = '#'

    interpol_fringe = interpol(fringe_calc, fringe_wn, wn)

    interpol_fringe_allch = interpol_fringe#replicate(1D, nch)

    newspec[*, 0, *] = spec[*, 0, 0, 0, *]/interpol_fringe_allch
    newspec[*, 1, *] = spec[*, 0, 0, 1, *]/interpol_fringe_allch

  endif else begin
    newspec = reform(spec[*, 0, 0, *, *])
  endelse

;;;;;;;;;;;;;;;;;;;
;; x (-1) or not ;;
;;;;;;;;;;;;;;;;;;;

  if((idetect eq 0L) and (keyword_set(mcal) eq 0)) then newspec = newspec*(-1.0)
  if((idetect eq 1L) and (keyword_set(mcal))) then newspec = newspec*(-1.0)

;;;;;;;;;;;;;
;; ps file ;;
;;;;;;;;;;;;;

  if(keyword_set(psfile)) then begin

    psfilename = filename_core+'.ps'

    ymin = dblarr(nch)
    ymax = dblarr(nch)
    if(keyword_set(fixmin)) then begin
      ymin[*] = fixmin
    endif else begin
      ymin[*] = 0.0
    endelse
    if(keyword_set(fixmax)) then begin
      ymax[*] = fixmax
    endif else begin
      for ich = 0L, nch-1L do begin
        ymax[ich] = abs(mean(newspec[*, *, ich], /nan)*5.0)
      endfor
    endelse

    if(idetect eq 0L) then ngroup = 4L else ngroup = 3L
    nrow = 5L
    ncolumn = 3L
    nrow_tot = nrow*ngroup

    plotminwn_default = [85.0D, 50.0D]
    plotmaxwn_default = [170.0D, 90.0D]

    set_plot, 'PS', /copy
    !P.MULTI = [0, nrow, ncolumn]
    device, filename = psfilename, /landscape, /color
    loadmycolors_all, names = colorname

    for igroup = 0L, ngroup-1L do begin
      for icolumn = 0L, ncolumn-1L do begin
        for ich = icolumn*nrow_tot+igroup*nrow, icolumn*nrow_tot+igroup*nrow+nrow-1L do begin

          titleword = 'ch '+strtrim(string(ich+1L), 2)

          if(keyword_set(wnmin)) then begin
            plotminwn = wnmin
          endif else begin
            plotminwn = plotminwn_default[idetect]
          endelse
          if(keyword_set(wnmax)) then begin
            plotmaxwn = wnmax
          endif else begin
            plotmaxwn = plotmaxwn_default[idetect]
          endelse

          plot, [plotminwn, plotmaxwn], [ymin[ich], ymax[ich]], /xstyle, /ystyle, title = titleword, /nodata

          oplot, wn, newspec[*, 0, ich], color = where(colorname eq 'red')
          oplot, wn, newspec[*, 1, ich], color = where(colorname eq 'orange')

        endfor
      endfor
    endfor

    device, /close
    set_plot, 'X'

  endif

;;;;;;;;;;;;;;;
;; text file ;;
;;;;;;;;;;;;;;;

  if(keyword_set(txtfile)) then begin

    txtfilename = filename_core+'.txt'

    if(keyword_set(txtch)) then begin
      printch = txtch-1L
      nprintch = n_elements(printch)
    endif else begin
      printch = indgen(nch)
      nprintch = nch
    endelse

    openw, 1, txtfilename

    tmp = 'ch'+strtrim(string(printch[0]+1L), 2)
    if(nprintch ge 2) then begin
      for ich = 1L, nprintch-1L do begin
        tmp = tmp+', ch'+strtrim(string(printch[ich]+1L), 2)
      endfor
    endif
    printf, 1, '# wavenumber[cm^-1], go and back of '+tmp

    for iwn = 0L, nwn-1L do begin
      tmp = ''
      for ich = 0L, nprintch-1L do begin
        tmp = tmp+string(newspec[iwn, 0, printch[ich]], format = '(e10.2)')+' ' $
          +string(newspec[iwn, 1, printch[ich]], format = '(e10.2)')+' '
      endfor

      printf, 1, format = '(f6.2,1x,a)', wn[iwn], tmp
    endfor

    close, 1
  endif else begin
    if(keyword_set(txtch)) then message, 'txtch should be used with txtfile option'
  endelse

  message, 'finish!', /continue

end
