; $Id: desmat_cs_sines.pro,v 1.1 2008/03/07 05:27:27 do Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	desmat_cs_sines.pro
; PURPOSE:
;	design matrix for smoothing cubic spline through data points
; CATEGORY:
;	Function.
; CALLING SEQUENCE:
;	design = desmat_cs_sines( x, knots, frequency )
;
; INPUTS:
;	x	    x data. independent axis
;	knots	    array of knots, arbitrary spaced and increasing
;	frequency   frequency
;
; OUTPUT:
;	dmcs	    cubic splines design matrix, for reuse
;
; RETURN
;	design	    design matrix
;
; RESTRICTIONS:
;	1. knots has to be of minimal length 2.
;	2. first one smaller than all x; last one larger than all x
;
; PROCEDURE:
;
;
;
;
;
; EXAMPLE:  x = 0.1 * findgen( 20 )
;	    y = exp( -3 * x ) * sin( 6 * x + 2 )
;	    knots = findgen( 5 ) * 0.4
;	    dm = desmat_cs_sines( x, knots, freq )
;	    yfit = fit_model( x, y, design=dm )
;	    fits a 5 knots cubic spline through these points
; REFERENCE:
;
; MODIFICATION HISTORY:
;	Nov. 2005		Written by Do Kester
;-

FUNCTION desmat_cs_sines, time, knots, frequency, dmcs=dm

if ( n_elements( dm ) EQ 0 ) then dm = desmat_cubic_splines( time, knots )
sz = size( dm )
if ( sz[1] NE n_elements( knots ) + 2 ) then begin
    dm = desmat_cubic_splines( time, knots )
    sz = size( dm )
endif

nm = sz[1]
np = nm * 2
npt= sz[2]

freq = 2 * !PI * frequency

desmat = [ dm * ( ( dblarr( nm ) + 1 ) # cos( freq * time ) ), $
	   dm * ( ( dblarr( nm ) + 1 ) # sin( freq * time ) ) ]

return, desmat

end

