;#> frin_amplitude.dc3
; Identifier	frin_amplitude
;
; Purpose	calculate the fringe amplitudes
;
; Synopsis	result = frin_amplitude( d, xw, x2, ampcos = ac, $
;						    ampsin = as, $
;						    status = status )
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		d	 I     struct	design matrix from frin_matrix
;		xw	 I     fltarr	data * weights
;		x2	 I     float	sum over data squared
;		ampcos	 O     fltarr	amplitudes of the cosines
;		ampsin	 O     fltarr	amplitudes of the sines
;		status	 O     int	nonzero status in case of problems
;
; Returns	result	 O     float	chi-squared
;
; Description
;
;
;
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_amplitude.pro
;
; Author	Do Kester
;
; Version	1.2
;
; History	1.0  15-03-2001 DK, first implementation of frin_amplitude.pro
;		1.1  05-04-2001 DK, remove compile_opt
;		1.2  24-10-2001 FL, catch for memory allocation problems
;#<
;===========================================================================

function fitline, yy, wt

compile_opt idl2, hidden

ndat = n_elements( yy )
des = dblarr( 2, ndat )
des[0,*] = 1
des[1,*] = dindgen( ndat )
dew = des * ( ( dblarr( 2 ) + 1 ) # wt )

mat = dew # transpose( des )
vec = dew # yy

ludc, mat, index, interchanges=parity
sol = lusol( mat, index, vec )

return, sol # des
end
