;#> frin_amplitude.dc3
; Identifier	frin_amplitude
;
; Purpose	calculate the fringe amplitudes
;
; Synopsis	result = frin_amplitude( d, xw, x2, ampcos = ac, $
;						    ampsin = as, $
;						    status = status )
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		d	 I     struct	design matrix from frin_matrix
;		xw	 I     fltarr	data * weights
;		x2	 I     float	sum over data squared
;		ampcos	 O     fltarr	amplitudes of the cosines
;		ampsin	 O     fltarr	amplitudes of the sines
;		status	 O     int	nonzero status in case of problems
;
; Returns	result	 O     float	chi-squared
;
; Description
;
;
;
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_amplitude.pro
;
; Author	Do Kester
;
; Version	1.2
;
; History	1.0  15-03-2001 DK, first implementation of frin_amplitude.pro
;		1.1  05-04-2001 DK, remove compile_opt
;		1.2  24-10-2001 FL, catch for memory allocation problems
;#<
;===========================================================================

function frin_amplitude, d, xw, x2, ampcos=ac, ampsin=as, status=status

compile_opt idl2, hidden

catch, error

if error ne 0 then begin
  catch, /Cancel
  help, /Last_Message, Output=theErrorMessage
  print,''
  error,'S','FRIN_AMPLITUDE: cannot allocate memory'
  error,'S','Try to reduce the size by e.g. increasing CYSTEP'
  print,''

  status = 1B
  return,0
endif else status = 0B

;print, size( xw ), size( c2 )

xx = make_array( size=size( d.c2 ), value=1.0 ) # xw
cx = total( d.cp * xx, 2 )
sx = total( d.sp * xx, 2 )
delta = d.c2 * d.s2 - d.cs * d.cs
ac = ( d.s2 * cx - d.cs * sx ) / delta		; amplitude of cosine
as = ( d.c2 * sx - d.cs * cx ) / delta		; amplitude of sine

return, x2 - ac * cx - as * sx			; chi_sq
end
