;#> frin_smooth.dc3
; Identifier	frin_smooth
;
; Purpose	make a smooth and clipped version of the flux
;
; Synopsis	result = frin_cs_smooth( flux, infl, detlis=detlist, wgt=wgt )
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		flux	 I     fltarr	flux array
;		infl	 I     fltarr	flag array
;		detlist  I     intarr	list of detectors to be smoothed
;		wgt	 O     fltarr	weights array
;
; Returns	result	 O     fltarr	smoothed background
;
; Description
;
;
;
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: fringes )
;
; Filename	frin_smooth.pro
;
; Author	Do Kester
;
; Version	2.0
;
; History	1.0  07-03-2001 DK, first implementation of frin_smooth.pro
;		2.0  06-03-2008 DK, Akari version
;#<
;===========================================================================

function frin_cs_smooth, flux, infl, detlist=detlist, wgt=wgt

; compile_opt idl2, hidden

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Make a smooth version of the flux, and calculate (Tukey) weights
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
smoo = make_array( size=size( flux ) )
wgt  = make_array( size=size( flux ) )
tukscale = 10.0

jd = detlist - 1
for k = 0, n_elements( jd ) - 1 do begin
   ff = flux[jd[k],*]
   b = where( infl[jd[k],*] )
   for ii = 0, 2, 2 do begin
      smf = ff
      smf[b] = !values.f_nan
      for i=1,3+ii do smf = smooth( smf, 5, /nan, /edge_truncate )
      tuk = median( abs( ff - smf ) ) * tukscale
      dist = abs( ff - smf ) / tuk ; scale to Tukey's scale
      b = where( infl[jd[k],*] OR dist GT 1 )
   endfor

   ww = ( ( dist < 1 )^2 - 1 )^2 ; make a Tukey weight
   b = where( finite( dist ) EQ 0, nb )
   if ( nb GT 0 ) then ww[b] = 0

   b = where( infl[jd[k],*] EQ 0, nb )
   if ( nb GT 0 ) then wgt[jd[k],b] = ww[b]
   smoo[jd[k],*] = smf
endfor
b = where ( finite( smoo ) EQ 0, nb )
if ( nb GT 0 ) then smoo[b] = 0 		    ; remove remaining NaNs

ool = 0.3
bool = where( abs( flux - smoo ) GT ool, nbool )
if ( nbool GT 0 ) then wgt[bool] = 0	    ; eliminate large outliers

return, smoo
end
