;#> plot_fft.dc1
; Identifier	plot_fft
;
; Purpose	plot a fft in continuous segments
;
; Synopsis	plot_fft, x, y, psym=psym, close=close, oplot=oplot, _extra=_e
;
; Arguments	Name	I/O    Type:	Description
;		------------------------------------------------------------
;		x	 I     fltarr	x coordinate of points to plot
;		y	 I     fltarr	y coordinate of points to plot
;		psym	 I     int(arr) plot symbols
;		close	 I     float	how close the points need to be
;					default : 6
;		oplot	 I     keyword	do overplotting
;		_extra	 I     struct	to be passedto theplotting routines
;
; Returns	--
;
; Description	Close adjacent points are connected by ffts.
;		When points are more apart than CLOSE * median of the absolute
;		distance between the points, the connecting is stopped.
;
;		All points are plotted, either connected or as individual
;		points.
;
; Comment
;
;
; Example	--
;
; Category	UTIL ( module: plot_fft )
;
; Filename	plot_fft.pro
;
; Author	Do Kester
;
; Version	1.1
;
; History	1.0  21-03-2001 DK, first implementation of plot_fft.pro
;		1.1  03-04-2001 DK, remove compile_opt
;		1.2  05-04-2001 DK, counting is difficult ...
;#<
;===========================================================================

pro plot_fft, y0, t, psym=psym, oplot=oplt, offset=offset, _extra=_extra

compile_opt idl2

if ( n_elements( psym ) EQ 0 ) then psym = 3
if ( n_elements( offset ) EQ 0 ) then offset = 0

sz = size( y0 )
dm = sz[0]
ny = sz[dm]

y = y0
if ( ( ny MOD 2 ) EQ 1 ) then begin
   y =[y,0]
   ny ++
endif
nh = ny / 2 + 1

if ( n_elements( t ) NE ny ) then begin
    f = findgen( ny )
    f[nh] = nh - ny + findgen( nh - 2 )
    f = f / ( ny * t )
endif else f = 1.0 / t


sf = shift( f, -nh )

if ( keyword_set( oplt ) ) $
then oplot, sf, shift( abs( fft( y, -1 )), -nh ) + offset, _extra=_extra $
else plot,  sf, shift( abs( fft( y, -1 )), -nh ) + offset, _extra=_extra

return
end

;===========================================================================

