;
; Supplies function for cal_fit3_lw
;
; Define a return function for LMFIT:
;   f(x) = a[0]*exp(-t/a[1]) + a[2]*(1-exp(-t/a[3])) + a[4]*exp(-t/a[5]) + a[6]
Function cal3f_lw_respons, t, a
  at = exp(-t/a[1])
  bt = exp((a[1]-t)/a[3])
  ct = exp(-t/a[5])
  f  = a[0]*at + a[2]/(bt+1)+a[4]*ct+a[6]

  Return, [f, at, a[0]*at*t/a[1]^2 - (a[2]/a[3])*bt/(bt+1)^2, 1/(bt+1), -a[2]*bt*t/a[3]^2/(bt+1)^2, ct, a[4]*ct*t/a[5]^2, 1.D]
End

;
; Performs curve fitting with the following function and returns y(t1)
; where t1 may be an array
;
;  ref. S.H.Oh et al. 2004
;
Function gb_cal_fit3f_lw, y, t, t1, YFIT=YFit, PARAMS=Params, TSTART=t0, CHISQ=chisq, FITA=FitA, ERR=err, ITMAX=itmax, ITMIN=itmin, ITER=ITER

  ;  initial guess for the coefficients
  ;
  ndata = N_Elements(y)
  a = findgen(7)
  if keyword_set(Params) then begin
 ;   print, '% params set as given value.'
    a = Params
  endif else begin
;     a[1] = 180.
;     a[3] = 70.
;     a[5] = 400.
;    a = [ 4.6e-4, 180d, 3.8d-4,  70d, 8d-4,  400d, -0.6d-4]
    a = [12.0d-4, 300d, 9.0d-4, 300d, 4d-4, 3500d,  2.5d-4] ; INIT GUESS 1
;    a = [12.0d-4, 300d, 9.0d-4, 300d, 4d-4, 2000d,  2.5d-4] ; INIT GUESS 2
  endelse
  if keyword_set(FitA) then begin
;    print, '% FITA set as given value.'
  endif else fita = [1,1,1,1,1,0,1]
;  print, a
  IF ~Keyword_Set(t0) THEN BEGIN
    t0 = t[0]
  ENDIF
  t_0 = t - t0
;  print,fita
  function_name = 'cal3f_lw_respons'
  c = lmfit(t_0, y, a,FITA=fita, FUNCTION_NAME=function_name, CHISQ=chisq,$
            /double, CONVERGENCE=convergence, ITMAX=300, ITMIN=itmin, MEASURE_ERROR=err, ITER=iter)
;  print,iter
;  stop
  case convergence of
    1  : convergence=1 ; be quiet ;-)
    0  : LMESSAGE, 7, "% the algorithm '"+function_name+"' did not converge."
    -1 : LMESSAGE, 7, "% the algorithm '"+function_name+"' encountered a singular matrix and did not converge."
  endcase

  at = exp(-(t1-t0)/a[1])
  bt = exp((a[1]-(t1-t0))/a[3])
  ct = exp(-(t1-t0)/a[5])
  b  = a[0]*at + a[2]/(bt+1)+a[4]*ct+a[6]

  if arg_present(YFit) then YFit = c   ; Fitted result value at each point
  if arg_present(Params) then Params = a ; Parameters of the fitting function
; print,a

  Return, b

End
