; $Id: gb_diff3pl.pro,v 1.2 2003/06/02 07:01:17 kawamura Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	gb_diff3pl.pro
; PURPOSE:
;	Input data is differentiated.
; CATEGORY:
;	Function. Green Box procedure in ASTRO-F pipeline.
; CALLING SEQUENCE:
;	Result = gb_diff3pl(vdet
;	[, CAL_CONST_FILE = cal_const_file, disp = disp])
; INPUTS:
;	vdet		array of detector value.
;	cal_const_file	Calibration constants file; 
;			format, usage are TBD (optional.)
;       rstcnt          array. values for reset counter.
;                       Shoult be changed to reset status later,
; OUTPUTS:
;	diff_vdet		array of differentiated detector value.
; RETURN:
;	!SUCCEED if function ended successfully.
;	!FAIL in failure.
; RESTRICTIONS:
;	
; PROCEDURE:
;
; EXAMPLE:
;       result = gb_diff3pl (vdet, diff_vdet, /disp, /flgrst)
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	May  2003		Written by A. Kawamura.
;-

FUNCTION gb_diff3pl, vdet, diff_vdet, Calib_Const_File = ccf, $
disp=disp, rstcnt = rstcnt
;ON_ERROR 2
IF N_PARAMS(0) EQ 0 THEN BEGIN
    MESSAGE,'FUNCTION gb_diff3pl, vdet, diff_vdet, $',/CONTINUE
    print,'      Calib_Const_File = ccf, rst = rst, /disp '
    print,'INPUTS:'
    print,'      vdet:        array of detector value.'
    print,'      diff_vdet:   array of differentiated detector value.'
    print,'      cal_const_file: Calibration constants file.'
    print,'      rstcnt:      array. value from resetcounter.'
    print,'                   This should be given for flagging 
    print,'                   before/after reset.'
    print,'      /disp:       for display.'
    print,'OUTPUT:'
    print,'      diff_vdet:   array of differentiated detector value.'
    RETURN, !FAIL
ENDIF

;;; Starting GB_Differentiation ;;;
   MESSAGE,'Called differentiation with 3-point Lag.',/CONTINUE
	
;;; number of channels ;;;
   si = SIZE(vdet)
   arr_dim = si[0]
   num_ch = si[1]
   num_data = si[2]

;;; Differentiate with 3-point Lag. ;;;;
   MESSAGE,'Executing differentiation...',/CONTINUE

IF KEYWORD_SET(disp) EQ 1 THEN BEGIN
   IF arr_dim EQ 1 THEN BEGIN
     MESSAGE,'Data is one dimentional array',/CONTINUE
     diff_vdet = deriv (vdet)
     plot,vdet[*],yrange = [-0.005,0.1]
     oplot,diff_vdet[*]
   ENDIF ELSE BEGIN
   

   diff_vdet = fltarr(num_ch, num_data)
   FOR I = 0, num_ch-1 DO BEGIN
     diff_vdet[I,*] = deriv (vdet[I, *])
     plot,vdet[I,*],yrange = [-0.005,0.1],title=I+1
     oplot,diff_vdet[I,*]
     wait,0.1	
   ENDFOR
   ENDELSE
ENDIF ELSE BEGIN
   IF arr_dim EQ 1 THEN BEGIN
     MESSAGE,'Data is one dimentional array',/CONTINUE
     diff_vdet = deriv (vdet)
   ENDIF ELSE BEGIN
print,num_ch, num_data
   diff_vdet = fltarr(num_ch, num_data)
   FOR I = 0, num_ch-1 DO BEGIN
     diff_vdet[I,*] = deriv (vdet[I, *])
   ENDFOR
   ENDELSE

ENDELSE

;;; Set flags for immediately before and/or after reset, "no_dif,"
;	print,'...setting flag no_dif...not yet done'
;	rstws=tsd->get_status_tsd('rstwidelon')
;	f_no_diff_value = rstws + SHIFT (rstws, 1) + SHIFT (rstws, -1)
;	dummy = tsd->set_flag_tsd('no_dif', f_no_diff_value)

IF KEYWORD_SET(rstcnt) EQ 1 THEN BEGIN
   rstsize = SIZE (rstcnt)
   IF rstsize[1] NE num_data THEN BEGIN
      MESSAGE,'parameter for reset is not valid',/CONTINUE
      RETURN, !FAIL
   ENDIF
        MESSAGE,'Setting NaN to where value of reset counter changes',/CONTINUE

        diff_rstcnt = rstcnt - SHIFT(rstcnt, +1)

        ind_rstcnt = WHERE(diff_rstcnt NE 0, count)
        cnt = count
        ind = WHERE(ind_rstcnt - SHIFT(ind_rstcnt, +1) EQ 0, count2)
        IF count2 NE 0 THEN $
        MESSAGE,'reset count increases beyond 1 pix.',/CONTINUE
       
        diff_vdet(*,ind_rstcnt) = !VALUES.F_NaN
        diff_vdet(*,ind_rstcnt-1) = !VALUES.F_NaN 
        diff_vdet(*,ind_rstcnt+1) = !VALUES.F_NaN      

ENDIF

	RETURN, !SUCCEED

END



