; $Id: gb_get_cds_mode_ind.pro,v 1.2 2006/01/16 00:01:23 baba Exp $
;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	gb_get_cds_mode_ind.pro
;
; PURPOSE:
;       This function returns the index values of the TSD data
;       taken with CDS mode.
;
; CATEGORY:
;	Function. Green Box library in ASTRO-F pipeline.
;
; CALLING SEQUENCE:
;	Result = gb_get_cds_mode_ind (packetid, count)
;
; INPUTS:
;
; OUTPUTS:
;
; RETURN:
;	This function returns the index values of the TSD data
;       taken with CDS mode.
;
; RESTRICTIONS:
;	Currently, packetid with 0x43 and 0x53 are considered to be
;	CDS mode.
;       This module uses IDL default library "WHERE".
; PROCEDURE:
;
; EXAMPLE:
;       The following process will returen the index values
;       of 'infile.fits' taken with CDS mode.
;
;       tsd=obj_new('tsd')
;       ret = tsd->read_tsd('infile.fits')
;       packetid = tsd->get_value_tsd('fis_obs','packetid')
;       index = gb_get_cds_mode_ind(packetid,count)
;       
; REFERENCE:
;	Baba et al. 2002 Nov. 21, ``ASTRO-F/FIS Survey Data Structure"
;
; MODIFICATION HISTORY:
;       Nov. 2004    IY Written.
;       Nov. 2004    AK Converted to GB library format
;-

FUNCTION gb_get_cds_mode_ind, packetid, count
; return index of the CDS sampling mode data
; input
;  PacketID: array of packet ID
; return
;  index of the CDS sampling mode. -1 if no index found
; output
;  count: number of appropriate data

   idx = where ( packetid EQ '43'X OR packetid EQ '53'X, count)

return, idx
end

