; $Id: gb_wr_start_msg.pro,v 1.4 2006/01/18 06:57:00 baba Exp $
;
; Copyright (c) 2004-2005, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;       gb_wr_start_msg.pro
; PURPOSE:
;       Write starting messages for each GB modules
; CATEGORY:
;   Function. Green Box library for ASTRO-F pipeline.
; CALLING SEQUENCE:
;   ret = gb_wr_start_msg(mod_name)
; INPUTS:
;   mod_name  name of the module
; OUTPUTS:
;
;
; RESTRICTIONS:
;   Only for GB modules
; PROCEDURE:
;
;
; EXAMPLE:
;       ret = gb_wr_start_msg('gb_flag_bad_data')
; REFERENCE:
;       Kawamura et al. 2003 Jan. "ASTRO-F data reduction pipeline -green box-"
;
; MODIFICATION HISTORY:
;   Dec. 2003     Written by A. Kawamura.
;   Jun. 2005     Last modified by Makiuti
;-

FUNCTION gb_wr_start_msg, mod_name
;ON_ERROR 2
IF N_PARAMS(0) NE 1 THEN BEGIN
MESSAGE, '  Calling Sequence: ret=gb_wr_start_msg(mod_name])',/CONTINUE
MESSAGE, '       mod_name = name of the module,e.g.gb_flag_bad_data',/CONTINUE
    return, !FAIL;
ENDIF

IF mod_name EQ 'gb_flag_bad_data' THEN BEGIN
   msg=strarr(1,13)
   msg[0,0]='% GB_FLAG_BAD_DATA:'
   msg[0,2]='  ;;; flags set                        ;;;'
   msg[0,3]='      dead, saturate, afterrst'
   msg[0,4]='  ;;; bad flags are set at pixels with ;;;'
   msg[0,5]='      dead, saturate, NaN, INFINITY'
   msg[0,6]='      blank, in_saa, near_moon'
   msg[0,7]='  ;;; bad flags for frames             ;;;'
   msg[0,8]='      blank, in_saa
   msg[0,10]='      # bad is set at pix. with bad_frame for TSD3'
   msg[0,11]='      # blank should be set before GB'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_conv_to_volt' THEN BEGIN
   msg=strarr(1,7)
   msg[0,0]='% GB_CONV_TO_VOLT:'
   msg[0,2]='  ;;; conversion eq. for V>-0.01V              ;;;'
   msg[0,3]='      dead, saturate, afterrst'
   msg[0,4]=''
   msg[0,5]=''
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_ramp_curve_corr' THEN BEGIN
   msg=strarr(1,4)
   msg[0,0]='% GB_RAMP_CURVE_CORR:'
   msg[0,2]='  ;;; correction of ramp curve               ;;;'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_diff2' THEN BEGIN
   msg=strarr(1,5)
   msg[0,0]='% GB_DIFFERENTIAION:'
   msg[0,2]='  ;;; flags set                        ;;;'
   msg[0,3]='      nodif(->bad)'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_diff3' THEN BEGIN
   msg=strarr(1,5)
   msg[0,0]='% GB_DIFFERENTIAION:'
   msg[0,2]='  ;;; flags set                        ;;;'
   msg[0,3]='      nodif(->bad)'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_transient_corr' THEN BEGIN
   msg=strarr(1,5)
   msg[0,0]='% GB_TRANSIENT_CORR:'
   msg[0,2]='  ;;; flags set                        ;;;'
   msg[0,3]='      bad'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_make_dark_table' THEN BEGIN
   msg=strarr(1,6)
   msg[0,0]='% GB_MAKE_DARK_TABLE:'
   msg[0,2]='  ;;; data taken when shutter is close is extracted'
   msg[0,3]='      output is written in dk_dt_table_lw/sw.save;;;'
   msg[0,4]='      # bad flags are chacked ;;;'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_make_dc_cal_lamp_table' THEN BEGIN
   msg=strarr(1,6)
   msg[0,0]='% GB_MAKE_DC_CAL_LAMP_TABLE:'
   msg[0,2]='  ;;; data taken when cal lamp is on with
   msg[0,3]='      shutter is close is extracted'
   msg[0,3]='      output is written in cal_dt_table_lw/sw.save;;;'
   msg[0,4]='      # bad flags are chacked ;;;'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_dark_subtraction' THEN BEGIN
   msg=strarr(1,4)
   msg[0,0]='% GB_DARK_SUBTRACTION:'
   msg[0,2]='  ;;; dark subtracted by using dk_dt_table.save;;;'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_responsivity_corr' THEN BEGIN
   msg=strarr(1,4)
   msg[0,0]='% GB_RESPONSIVITY_CORR:'
   msg[0,2]='  ;;; data is scaled by using dk_dt_table.save;;;'
   LMESSAGE, 7, msg, /PRINT

ENDIF ELSE IF mod_name EQ 'gb_flux_calib' THEN BEGIN
   msg=strarr(1,4)
   msg[0,0]='% GB_FLUX_CALIB:'
   msg[0,2]='  ;;; data is scaled by using calibration table;;;'
   LMESSAGE, 7, msg, /PRINT

ENDIF
    return, !SUCCEED
END
