; $$
;+
; NAME:
;	INTVAL
; PURPOSE:
;	Search blocks in the array containing subscripts and computes start and end index of the blocks.
;	
; EXPLANATION:
;	
; CALLING SEQUENCE:
;  	intval, index, start_index, end_index
;
; INPUTS:
;  	index			- A vector containing the one-dimentional subscripts.
;
; OUTPUTS:
;  	start_index		- A scalor or vector containing start position of each block. The dimension of this variable is equal to the number of block.
;  	end_index		- A scalor or vector containing end position of each block. The dimension of this variable is equal to the number of block.
;
; EXAMPLE:
;	IDL> a = [2, 3, 12,13,39,1,2,3,2,6,29,18,47,4,-2,0,4,8]
;	IDL> index = where(a ge 4)
;	IDL> intval, index, start_pos, end_pos
;	IDL> print, startpos, endpos
;           2           9          16
;           4          13          17
;
; PROCEDURES USED:
;  	
; REVISION HISTORY:
;  	Created by N. Ikeda, 4/16/07
;-

pro intval, index, startpos, endpos
;startpos = 0l
startpos = min(index)
endpos = 0l
for k = 0l, n_elements(index)-2 do begin
	if (index[k] ne index[k+1] - 1) then begin
		endpos = [endpos, index[k]]
		startpos = [startpos, index[k+1]]
;		endpos = [endpos, k]
;		startpos = [startpos, k+1l]
	endif
endfor
;endpos = [endpos, n_elements(index)-1l]
endpos = [endpos, max(index)]
endpos = endpos[1:*]
end
