;
; Copyright (c) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; NAME:
;	mmt_define_glitch_measure.pro
; PURPOSE:
;       Define structure which holds measured values for glitches
; CATEGORY:
;	Green Box procedure in library in ASTRO-F pipeline.
; CALLING SEQUENCE:
;
; INPUTS:
;       new          1 to create new glitch_measures
;                    0 to combine two glitch_measures
;	n_signal     Number of signals  
;
; OUTPUTS:
;       glitch_measure structure
;
; RESTRICTIONS:
;	
; PROCEDURE:
;
; EXAMPLE:
;
; REFERENCE:
;
; MODIFICATION HISTORY:
; 	May. 2005		Written by T.Takagi
;    
;-

FUNCTION mmt_define_glitch_measures, new, n_signal=n_signal, gm1=gm1, gm2=gm2

if (new eq 1) then begin 
    if (n_signal ge 1) then begin 
        glitch_measures = {isGlitch:intarr(n_signal),$
                           ch:intarr(n_signal),$
                           id_start:intarr(n_signal),$
                           max_id:intarr(n_signal), $
                           proj:intarr(n_signal),$
                           sigma:fltarr(n_signal),$
                           width1:intarr(n_signal),$
                           w_ratio:fltarr(n_signal),$
                           sn:fltarr(n_signal),$
                           duration:intarr(n_signal),$
                           size:intarr(n_signal),$
                           integral:fltarr(n_signal),$
                           int1:fltarr(n_signal),$
                           int2:fltarr(n_signal),$
                           glitch_frac:fltarr(n_signal),$
                           max_diff:fltarr(n_signal),$
                           fader:bytarr(n_signal),$
                           dipper:bytarr(n_signal) $
                          }    
    endif else if (n_signal eq 0) then begin 
        glitch_measures = 0
    endif
endif

if (new eq 0) then begin 
;; combine glitch_measures
    sz1 = size(gm1)
    sz2 = size(gm2)
    n_signal1=0
    n_signal2=0
    if (sz1[0] ne 0) then n_signal1 = n_elements(gm1.isGlitch)
    if (sz2[0] ne 0) then n_signal2 = n_elements(gm2.isGlitch)
    n_signal = n_signal1+n_signal2

    if (n_signal eq n_signal1) then begin 
        glitch_measures = gm1
    endif else if (n_signal eq n_signal2) then begin 
        glitch_measures = gm2 
    endif else begin 
        glitch_measures = mmt_define_glitch_measures(1,n_signal=n_signal)
        
        glitch_measures.isGlitch[0:n_signal1-1] = gm1.isGlitch
        glitch_measures.id_start[0:n_signal1-1] = gm1.id_start
        glitch_measures.max_id[0:n_signal1-1] = gm1.max_id
        glitch_measures.proj[0:n_signal1-1] = gm1.proj
        glitch_measures.sigma[0:n_signal1-1] = gm1.sigma
        glitch_measures.width1[0:n_signal1-1] = gm1.width1
        glitch_measures.w_ratio[0:n_signal1-1] = gm1.w_ratio
        glitch_measures.sn[0:n_signal1-1] = gm1.sn
        glitch_measures.duration[0:n_signal1-1] = gm1.duration
        glitch_measures.size[0:n_signal1-1] = gm1.size
        glitch_measures.integral[0:n_signal1-1] = gm1.integral
        glitch_measures.int1[0:n_signal1-1] = gm1.int1
        glitch_measures.int2[0:n_signal1-1] = gm1.int2
        glitch_measures.glitch_frac[0:n_signal1-1] = gm1.glitch_frac
        glitch_measures.max_diff[0:n_signal1-1] = gm1.max_diff
        glitch_measures.dipper[0:n_signal1-1] = gm1.dipper
        glitch_measures.fader[0:n_signal1-1] = gm1.fader

        glitch_measures.isGlitch[n_signal1:n_signal-1] = gm2.isGlitch
        glitch_measures.id_start[n_signal1:n_signal-1] = gm2.id_start
        glitch_measures.max_id[n_signal1:n_signal-1] = gm2.max_id
        glitch_measures.proj[n_signal1:n_signal-1] = gm2.proj
        glitch_measures.sigma[n_signal1:n_signal-1] = gm2.sigma
        glitch_measures.width1[n_signal1:n_signal-1] = gm2.width1
        glitch_measures.w_ratio[n_signal1:n_signal-1] = gm2.w_ratio
        glitch_measures.sn[n_signal1:n_signal-1] = gm2.sn
        glitch_measures.duration[n_signal1:n_signal-1] = gm2.duration
        glitch_measures.size[n_signal1:n_signal-1] = gm2.size
        glitch_measures.integral[n_signal1:n_signal-1] = gm2.integral
        glitch_measures.int1[n_signal1:n_signal-1] = gm2.int1
        glitch_measures.int2[n_signal1:n_signal-1] = gm2.int2
        glitch_measures.glitch_frac[n_signal1:n_signal-1] = gm2.glitch_frac
        glitch_measures.max_diff[n_signal1:n_signal-1] = gm2.max_diff
        glitch_measures.dipper[n_signal1:n_signal-1] = gm2.dipper
        glitch_measures.fader[n_signal1:n_signal-1] = gm2.fader
        
    endelse
endif    

RETURN, glitch_measures
END
