; $Id: tabledef.pro,v 1.14 2007/10/10 05:56:20 cyamauch Exp $
;
; Copyright(C) 2002, ASTRO-F Data Reduction Team.  All rights reserved.
;
;+
; CLASS_NAME:
;   TABLEDEF
;
; PURPOSE:
;   Define the object structure for a Time Series Data (TSD) of FIS Survey.
;
; CATEGORY:
;   Table Data definition subroutine
;
; CALLING SEQUENCE:
;   None
;
; INPUTS:
;   None
;
; OUTPUTS:
;   None
;
; RESTRICTIONS:
;   None
;
; COMMENTS:
;   Called from initializer method of TSD__DEFINE.
;
; DESCRIPTION:
;   See 'ASTRO-F FIS/Survey Data Structure' in detail.
;
; EXAMPLE:
;   None
;
; SIDE EFFECTS:
;   None
;
; MODIFICATION HISTORY:
;   Written by: Hajime Baba, February 2003.
;
;-

; generated by tsd_idl_code_generator.c

PRO tabledef_version4
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    null = PTR_NEW()

    COMMON fis_sw_block, fis_sw    ; Common within this module
    fis_ndet = '100'
    fis_sw_status_members = $
      [ 'CREON', 'SHTOP', 'FWPOSON', 'FWPOS_B1', $
        'FWPOS_B0', 'MPOSON', 'MPOS_B1', 'MPOS_B0', $
        'RSTWIDELON', 'RSTWIDESON', 'RSTN170ON', 'RSTN60ON', $
        'LWBOOSTON', 'SWBOOSTON', 'LWBIASON', 'SWBIASON', $
        'CALALON', 'CALASON', 'CALBON', 'SINALON', $
        'SINASON' ]
    fis_sw_flag_members = $
      [ 'BAD_FRAME', 'UNDEF_ANOM_FRAME', 'BLANK', 'IN_SAA', $
        'NEAR_MOON', 'UNTRUSTED_FRAME' ]
    fis_sw_pix_flag_members = $
      [ 'BAD', 'UNDEF_ANOM', 'ARITH_ERR', 'DEAD', $
        'SATURATE', 'RESET', 'RSTANOM', 'NO_DIFF', $
        'NO_RP_CORR', 'NO_DK_CORR', 'NO_DCCAL_CORR', 'NO_TR_CORR', $
        'NO_FLAT_FIELD', 'NO_GPGL', 'NO_MTGL', 'NO_ABSCAL', $
        'GPGL_TYPE1', 'GPGL_TYPE2', 'GPGL_TYPE3', 'GPGL_TYPE4', $
        'GPGL_TAIL', 'MTGL_TYPE1', 'MTGL_TYPE2', 'MTGL_TYPE3', $
        'MTGL_TYPE4', 'MTGL_TAIL', 'NO_PERI_CORR', 'SX_PEAK', $
        'SX_SOURCE', 'SX_OBJ', 'FLUTTER', '' ]
    fis_sw_quality_members = $
      [ 'QUAL_CV_PARAM', 'QUAL_CV_PARAM', 'QUAL_RC_PARAM', 'QUAL_RC_PARAM', $
        'QUAL_RC_CF', 'QUAL_RC_CF', 'QUAL_DF_EQ', 'QUAL_DF_EQ', $
        'QUAL_RP_DATA', 'QUAL_RP_DATA', 'QUAL_RP_PARAM', 'QUAL_RP_PARAM', $
        'QUAL_RP_TABLE', 'QUAL_RP_TABLE', 'QUAL_FF_PARAM', 'QUAL_FF_PARAM', $
        'QUAL_FF_CF', 'QUAL_FF_CF', 'QUAL_GPGL_CORR', 'QUAL_GPGL_CORR', $
        'QUAL_MTGL_CORR', 'QUAL_MTGL_CORR', 'QUAL_TR_HIST', 'QUAL_TR_HIST', $
        'QUAL_TR_PARAM', 'QUAL_TR_PARAM', 'QUAL_DK_DATA', 'QUAL_DK_DATA', $
        'QUAL_DK_PARAM', 'QUAL_DK_PARAM', 'QUAL_DK_TABLE', 'QUAL_DK_TABLE', $
        'QUAL_FX_CORR', 'QUAL_FX_CORR', 'QUAL_FX_PARAM', 'QUAL_FX_PARAM', $
        'QUAL_FLUTTER', 'QUAL_FLUTTER', '', '' ]
    fis_sw = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 39)
    fis_sw = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1K', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti', '1K', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti', '1K', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(fis_sw_status_members) }, $
         {s, 'packetid', '1B', '', '', 'PacketID', null }, $
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170', '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60', '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb', '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw', '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw', '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt', '1J', 'COUNTER', '', 'Mirror POS counter (assigned and interpolated)', null }, $
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60', '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170', '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb', '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         {s, 'tpbody', '1I', 'ADU', '', 'FIS body temperature', null }, $
         {s, 'tpsw', '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw', '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         {s, 'det', '100J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '100E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '100E', '', '', 'Error of flux', null }, $
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_sw_flag_members) }, $
         {s, 'pix_flag', '3200X', '', '(32,100)', 'Flag for each pixel condition', PTR_NEW(fis_sw_pix_flag_members) }, $
         {s, 'quality', '4000X', '', '(40,100)', 'Quality for each pixel condition', PTR_NEW(fis_sw_quality_members) }, $
         {s, 'cnt_saa', '100I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_gp', '100I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_mt', '100I', '', '', 'Counter after spike for each pixel', null } $
    ]

    COMMON fis_lw_block, fis_lw    ; Common within this module
    fis_ndet = '75'
    fis_lw_status_members = $
      [ 'CREON', 'SHTOP', 'FWPOSON', 'FWPOS_B1', $
        'FWPOS_B0', 'MPOSON', 'MPOS_B1', 'MPOS_B0', $
        'RSTWIDELON', 'RSTWIDESON', 'RSTN170ON', 'RSTN60ON', $
        'LWBOOSTON', 'SWBOOSTON', 'LWBIASON', 'SWBIASON', $
        'CALALON', 'CALASON', 'CALBON', 'SINALON', $
        'SINASON' ]
    fis_lw_flag_members = $
      [ 'BAD_FRAME', 'UNDEF_ANOM_FRAME', 'BLANK', 'IN_SAA', $
        'NEAR_MOON', 'UNTRUSTED_FRAME' ]
    fis_lw_pix_flag_members = $
      [ 'BAD', 'UNDEF_ANOM', 'ARITH_ERR', 'DEAD', $
        'SATURATE', 'RESET', 'RSTANOM', 'NO_DIFF', $
        'NO_RP_CORR', 'NO_DK_CORR', 'NO_DCCAL_CORR', 'NO_TR_CORR', $
        'NO_FLAT_FIELD', 'NO_GPGL', 'NO_MTGL', 'NO_ABSCAL', $
        'GPGL_TYPE1', 'GPGL_TYPE2', 'GPGL_TYPE3', 'GPGL_TYPE4', $
        'GPGL_TAIL', 'MTGL_TYPE1', 'MTGL_TYPE2', 'MTGL_TYPE3', $
        'MTGL_TYPE4', 'MTGL_TAIL', 'NO_PERI_CORR', 'SX_PEAK', $
        'SX_SOURCE', 'SX_OBJ', 'FLUTTER', '' ]
    fis_lw_quality_members = $
      [ 'QUAL_CV_PARAM', 'QUAL_CV_PARAM', 'QUAL_RC_PARAM', 'QUAL_RC_PARAM', $
        'QUAL_RC_CF', 'QUAL_RC_CF', 'QUAL_DF_EQ', 'QUAL_DF_EQ', $
        'QUAL_RP_DATA', 'QUAL_RP_DATA', 'QUAL_RP_PARAM', 'QUAL_RP_PARAM', $
        'QUAL_RP_TABLE', 'QUAL_RP_TABLE', 'QUAL_FF_PARAM', 'QUAL_FF_PARAM', $
        'QUAL_FF_CF', 'QUAL_FF_CF', 'QUAL_GPGL_CORR', 'QUAL_GPGL_CORR', $
        'QUAL_MTGL_CORR', 'QUAL_MTGL_CORR', 'QUAL_TR_HIST', 'QUAL_TR_HIST', $
        'QUAL_TR_PARAM', 'QUAL_TR_PARAM', 'QUAL_DK_DATA', 'QUAL_DK_DATA', $
        'QUAL_DK_PARAM', 'QUAL_DK_PARAM', 'QUAL_DK_TABLE', 'QUAL_DK_TABLE', $
        'QUAL_FX_CORR', 'QUAL_FX_CORR', 'QUAL_FX_PARAM', 'QUAL_FX_PARAM', $
        'QUAL_FLUTTER', 'QUAL_FLUTTER', '', '' ]
    fis_lw = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 39)
    fis_lw = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1K', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti', '1K', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti', '1K', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(fis_lw_status_members) }, $
         {s, 'packetid', '1B', '', '', 'PacketID', null }, $
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170', '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60', '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb', '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw', '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw', '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt', '1J', 'COUNTER', '', 'Mirror POS counter (assigned and interpolated)', null }, $
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60', '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170', '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb', '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         {s, 'tpbody', '1I', 'ADU', '', 'FIS body temperature', null }, $
         {s, 'tpsw', '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw', '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         {s, 'det', '75J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '75E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '75E', '', '', 'Error of flux', null }, $
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_lw_flag_members) }, $
         {s, 'pix_flag', '2400X', '', '(32,75)', 'Flag for each pixel condition', PTR_NEW(fis_lw_pix_flag_members) }, $
         {s, 'quality', '3000X', '', '(40,75)', 'Quality for each pixel condition', PTR_NEW(fis_lw_quality_members) }, $
         {s, 'cnt_saa', '75I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_gp', '75I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_mt', '75I', '', '', 'Counter after spike for each pixel', null } $
    ]

    COMMON iso_block, iso    ; Common within this module
    iso_ndet = '4'
    iso_status_members = $
      [ 'CREON', 'SHTOP', 'FWPOSON', 'FWPOS_B1', $
        'FWPOS_B0', 'MPOSON', 'MPOS_B1', 'MPOS_B0', $
        'RSTWIDELON', 'RSTWIDESON', 'RSTN170ON', 'RSTN60ON', $
        'LWBOOSTON', 'SWBOOSTON', 'LWBIASON', 'SWBIASON', $
        'CALALON', 'CALASON', 'CALBON', 'SINALON', $
        'SINASON' ]
    iso_flag_members = $
      [ 'BAD_FRAME', 'UNDEF_ANOM_FRAME', 'BLANK', 'IN_SAA', $
        'NEAR_MOON', 'UNTRUSTED_FRAME' ]
    iso_pix_flag_members = $
      [ 'BAD', 'UNDEF_ANOM', 'ARITH_ERR', 'DEAD', $
        'SATURATE', 'RESET', 'RSTANOM', 'NO_DIFF', $
        'NO_RP_CORR', 'NO_DK_CORR', 'NO_DCCAL_CORR', 'NO_TR_CORR', $
        'NO_FLAT_FIELD', 'NO_GPGL', 'NO_MTGL', 'NO_ABSCAL', $
        'GPGL_TYPE1', 'GPGL_TYPE2', 'GPGL_TYPE3', 'GPGL_TYPE4', $
        'GPGL_TAIL', 'MTGL_TYPE1', 'MTGL_TYPE2', 'MTGL_TYPE3', $
        'MTGL_TYPE4', 'MTGL_TAIL', 'NO_PERI_CORR', 'SX_PEAK', $
        'SX_SOURCE', 'SX_OBJ', 'FLUTTER', '' ]
    iso_quality_members = $
      [ 'QUAL_CV_PARAM', 'QUAL_CV_PARAM', 'QUAL_RC_PARAM', 'QUAL_RC_PARAM', $
        'QUAL_RC_CF', 'QUAL_RC_CF', 'QUAL_DF_EQ', 'QUAL_DF_EQ', $
        'QUAL_RP_DATA', 'QUAL_RP_DATA', 'QUAL_RP_PARAM', 'QUAL_RP_PARAM', $
        'QUAL_RP_TABLE', 'QUAL_RP_TABLE', 'QUAL_FF_PARAM', 'QUAL_FF_PARAM', $
        'QUAL_FF_CF', 'QUAL_FF_CF', 'QUAL_GPGL_CORR', 'QUAL_GPGL_CORR', $
        'QUAL_MTGL_CORR', 'QUAL_MTGL_CORR', 'QUAL_TR_HIST', 'QUAL_TR_HIST', $
        'QUAL_TR_PARAM', 'QUAL_TR_PARAM', 'QUAL_DK_DATA', 'QUAL_DK_DATA', $
        'QUAL_DK_PARAM', 'QUAL_DK_PARAM', 'QUAL_DK_TABLE', 'QUAL_DK_TABLE', $
        'QUAL_FX_CORR', 'QUAL_FX_CORR', 'QUAL_FX_PARAM', 'QUAL_FX_PARAM', $
        'QUAL_FLUTTER', 'QUAL_FLUTTER', '', '' ]
    iso = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 39)
    iso = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1K', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti', '1K', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti', '1K', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(iso_status_members) }, $
         {s, 'packetid', '1B', '', '', 'PacketID', null }, $
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170', '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60', '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb', '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw', '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw', '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt', '1J', 'COUNTER', '', 'Mirror POS counter (assigned and interpolated)', null }, $
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60', '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170', '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb', '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         {s, 'tpbody', '1I', 'ADU', '', 'FIS body temperature', null }, $
         {s, 'tpsw', '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw', '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         {s, 'det', '4J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '4E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '4E', '', '', 'Error of flux', null }, $
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(iso_flag_members) }, $
         {s, 'pix_flag', '128X', '', '(32,4)', 'Flag for each pixel condition', PTR_NEW(iso_pix_flag_members) }, $
         {s, 'quality', '160X', '', '(40,4)', 'Quality for each pixel condition', PTR_NEW(iso_quality_members) }, $
         {s, 'cnt_saa', '4I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_gp', '4I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_mt', '4I', '', '', 'Counter after spike for each pixel', null } $
    ]

    COMMON fis_hk_block, fis_hk    ; Common within this module
    fis_hk_status_members = $
      [ 'BLANK', 'BLANK_SUBCOM_DATA' ]
    fis_hk = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 11)
    fis_hk = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512Hz)', null }, $
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(fis_hk_status_members) }, $
         {s, 'fis_fw_f_pul', '1B', 'COUNTER', '', 'Filter wheel pulse counter (Forward)', null }, $
         {s, 'fis_fw_r_pul', '1B', 'COUNTER', '', 'Filter wheel pulse counter (Rewind)', null }, $
         {s, 'fis_lw_wire_light', '1B', '', '', 'LW wire light DAC set value', null }, $
         {s, 'fis_run_mode', '1B', '', '', 'RUN mode', null }, $
         {s, 'fis_obs_mode', '1B', '', '', 'Observation mode', null }, $
         {s, 'fis_asq_pattern', '1B', '', '', 'ASQ pattern', null }, $
         {s, 'fis_asq_step', '1J', '', '', 'ASQ step', null }, $
         {s, 'sw_heater_tmp', '1B', '', '', 'SW heater temperature', null } $
    ]

    COMMON irc_hk_block, irc_hk    ; Common within this module
    irc_hk_status_members = $
      [ 'BLANK' ]
    irc_hk = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 3)
    irc_hk = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512Hz)', null }, $
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(irc_hk_status_members) } $
    ]

    COMMON hk_2_block, hk_2    ; Common within this module
    hk_2_status_members = $
      [ 'BLANK', 'BLANK_CRYO_TEMP_DATA', '', '', $
        'CRYO_TEMP_DATA_EN_DS', '', 'FIS_CPU_RUN_RST', 'FIS_FIS_ON_OFF', $
        'FIS_TMP_POL_POS_NEG', 'FIS_OBS_SEQ', 'FIS_RSW_AUTO_CMND', 'FIS_FCAL_FRQ_B1', $
        'FIS_FCAL_FRQ_B0', '', 'IRC_CPU_RU_RS', 'IRC_IRC_ON_OF' ]
    hk_2 = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 13)
    hk_2 = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512Hz)', null }, $
         {s, 'status', '16X', '', '', 'Status', PTR_NEW(hk_2_status_members) }, $
         {s, 'cryo_he_tank_1', '1E', 'K', '', 'Temperature of He tank', null }, $
         {s, 'cryo_baffle_1', '1E', 'K', '', 'Temperature of baffle 1', null }, $
         {s, 'cryo_baffle_2', '1E', 'K', '', 'Temperature of baffle 2', null }, $
         {s, 'cryo_cold_hd_a_dr_i', '1E', 'A', '', 'Cold head A driving current', null }, $
         {s, 'cryo_cold_hd_b_dr_i', '1E', 'A', '', 'Cold head B driving current', null }, $
         {s, 'cryo_comp_a_dr_i', '1E', 'A', '', 'Compressor A driving current', null }, $
         {s, 'cryo_comp_b_dr_i', '1E', 'A', '', 'Compressor B driving current', null }, $
         {s, 'irc_mainref_tmp1', '1E', 'K', '', 'Temperature of prime mirror', null }, $
         {s, 'irc_subref_tmp', '1E', 'K', '', 'Temperature of sub mirror', null }, $
         {s, 'irc_fpip_tmp1', '1E', 'K', '', 'Temperature of FPI plate', null } $
    ]

    COMMON aocu_block, aocu    ; Common within this module
    aocu_status_members = $
      [ 'BLANK', 'AO_ROM_RAM', 'ADS_MODE_B1', 'ADS_MODE_B0', 'ADS_RATE_MODE_B1', 'ADS_RATE_MODE_B0' ]
    aocu_quality_members = $
      [ 'INTERPOLATED' ]
    aocu = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 17)
    aocu = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512Hz)', null }, $
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(aocu_status_members) }, $
         {s, 'quaternion', '4D', '', '', 'Quaternion at boresight', null }, $
         {s, 'body_rate', '3D', 'deg/s', '', 'Angular velocity at boresight', null }, $
         {s, 'aocu_saab_mode', '1B', '', '', 'STT-A operation mode', null }, $
         {s, 'aocu_sbab_mode', '1B', '', '', 'STT-B operation mode', null }, $
         {s, 'aocu_saab_win_num', '1B', '', '', 'Number of stars in STT-A', null }, $
         {s, 'aocu_sbab_win_num', '1B', '', '', 'number of stars in STT-B', null }, $
         {s, 'ra', '1D', '', '', 'J2000 R.A. at boresight', null }, $
         {s, 'dec', '1D', '', '', 'J2000 Dec. at boresight', null }, $
         {s, 'roll', '1D', '', '', 'Roll angle at boresight', null }, $
         {s, 'd_ra', '1D', '', '', 'Scan rate in J2000 R.A.', null }, $
         {s, 'd_dec', '1D', '', '', 'Scan rate in J2000 Dec.', null }, $
         {s, 'd_roll', '1D', '', '', 'Scan rate in roll angle', null }, $
         {s, 'crs_off', '1D', '', '', 'Offset from nominal scan path', null }, $
         {s, 'quality', '8X', '', '', 'Quality for condition', PTR_NEW(aocu_quality_members) } $
    ]

    COMMON gads_block, gads    ; Common within this module
    gads_status_members = $
      [ 'BLANK', 'ATT_CNT_MD_B1', 'ATT_CNT_MD_B0' ]
    gads_quality_members = $
      [ 'INTERPOLATED' ]
    gads = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 18)
    gads = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512Hz)', null }, $
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(gads_status_members) }, $
         {s, 'quaternion', '4D', '', '', 'Spacecraft Attitude (Quaternion)', null }, $
         {s, 'pos_err', '3D', 'rad', '', 'Uncertainty of the Attitude', null }, $
         {s, 'body_rate', '3D', 'rad/s', '', 'Angular velocity of the spacecraft (X,Y,X)', null }, $
         {s, 'ra', '1D', '', '', 'J2000 R.A. at boresight', null }, $
         {s, 'dec', '1D', '', '', 'J2000 Dec. at boresight', null }, $
         {s, 'roll', '1D', '', '', 'Roll angle at boresight', null }, $
         {s, 'd_ra', '1D', '', '', 'Scan rate in J2000 R.A.', null }, $
         {s, 'd_dec', '1D', '', '', 'Scan rate in J2000 Dec.', null }, $
         {s, 'd_roll', '1D', '', '', 'Scan rate in roll angle', null }, $
         {s, 'el_sol', '1D', '', '', 'Solar elongation', null }, $
         {s, 'aa_sol', '1D', '', '', 'Solar avoidance angle', null }, $
         {s, 'aa_ear', '1D', '', '', 'Earth avoidance angle', null }, $
         {s, 'aa_lun', '1D', '', '', 'Lunar avoidance angle', null }, $
         {s, 'crs_off', '1D', '', '', 'Offset from nominal scan path', null }, $
         {s, 'quality', '8X', '', '', 'Quality for condition', PTR_NEW(gads_quality_members) } $
    ]

    COMMON pr_block, pr    ; Common within this module
    pr_status_members = $
      [ 'BLANK' ]
    pr_quality_members = $
      [ 'INTERPOLATED' ]
    pr = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 18)
    pr = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512Hz)', null }, $
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(pr_status_members) }, $
         {s, 'quaternion', '4D', '', '', 'Quaternion at boresight', null }, $
         {s, 'pos_err', '3D', '', '', 'Estimated error of position', null }, $
         {s, 'ra', '1D', '', '', 'J2000 R.A. at boresight', null }, $
         {s, 'dec', '1D', '', '', 'J2000 Dec. at boresight', null }, $
         {s, 'roll', '1D', '', '', 'Roll angle at boresight', null }, $
         {s, 'd_ra', '1D', '', '', 'Scan rate in J2000 R.A.', null }, $
         {s, 'd_dec', '1D', '', '', 'Scan rate in J2000 Dec.', null }, $
         {s, 'd_roll', '1D', '', '', 'Scan rate in roll angle', null }, $
         {s, 'el_sol', '1D', '', '', 'Solar elongation', null }, $
         {s, 'aa_sol', '1D', '', '', 'Solar avoidance angle', null }, $
         {s, 'aa_ear', '1D', '', '', 'Earth avoidance angle', null }, $
         {s, 'aa_lun', '1D', '', '', 'Lunar avoidance angle', null }, $
         {s, 'crs_off', '1D', '', '', 'Offset from nominal scan path', null }, $
         {s, 'quality', '8X', '', '', 'Quality for condition', PTR_NEW(pr_quality_members) } $
    ]

    COMMON se_block, se    ; Common within this module
    se_status_members = $
      [ 'BLANK', 'TYPE', 'DAYNIGHT', 'IN_SAA', $
        'IN_POLAR' ]
    se = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 7)
    se = [ $
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT, TELEM
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1K', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512Hz)', null }, $
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(se_status_members) }, $
         {s, 'tm_saa', '1D', 's', '', 'Time after latest SAA', null }, $
         {s, 'sat_posx', '1D', '', '', '', null }, $
         {s, 'sat_posy', '1D', '', '', '', null }, $
         {s, 'sat_posz', '1D', '', '', '', null } $
    ]

END

;
; Period of validity: 2004 June.
;
PRO tabledef_version3
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    null = PTR_NEW()

    COMMON fis_sw_block, fis_sw    ; Common within this module
    fis_ndet = '100'

    fis_obs_status_members = $
      [ '', '', '', '', '', '', '', '', '', '', '', 'SINASON', $
        'SINALON', 'CALBON', 'CALASON', 'CALALON', $
        'LWBIASON', 'SWBIASON', 'LWBOOSTON', 'SWBOOSTON', $
        'RSTN60ON', 'RSTN170ON', 'RSTWIDESON', 'RSTWIDELON', $
        'MPOSB0', 'MPOSB1', 'MPOSON', 'FWPOSB0', $
        'FWPOSB1', 'FWPOSON', 'SHTOP', 'CREON' ]
    fis_obs_flag_members = $
      [ '', '', '', 'NEAR_MOON', $
        'IN_SAA', 'BLANK', 'UNDEF_ANOM_FRAME', 'BAD_FRAME' ]
    fis_obs_pix_flag_members = $
      [ '', '', '', 'SX_OBJ', $
        'SX_SOURCE', 'SX_PEAK', 'MTGL_TAIL', 'MTGL_TYPE4', $
        'MTGL_TYPE3', 'MTGL_TYPE2', 'MTGL_TYPE1', 'GPGL_TAIL', $
        'GPGL_TYPE4', 'GPGL_TYPE3', 'GPGL_TYPE2', 'GPGL_TYPE1', $
        'NO_ABSCAL', 'NO_MTGL', 'NO_GPGL', 'NO_FLAT_FIELD', $
        'NO_TR_CORR', 'NO_DCCAL_CORR', 'NO_DK_CORR', 'NO_RP_CORR', $
        'NO_DIFF', 'RSTANOM', 'RESET', 'SATURATE', $
        'DEAD', 'ARITH_ERR', 'UNDEF_ANOM', 'BAD' ]
    fis_obs_quality_members = $
      [ '', '', $
        '', '', $
        'QUAL_FX_PARAM', 'QUAL_FX_PARAM', $
        'QUAL_FX_CORR', 'QUAL_FX_CORR', $
        'QUAL_DK_TABLE', 'QUAL_DK_TABLE', $
        'QUAL_DK_PARAM', 'QUAL_DK_PARAM', $
        'QUAL_DK_DATA', 'QUAL_DK_DATA', $
        'QUAL_TR_PARAM', 'QUAL_TR_PARAM', $
        'QUAL_TR_HIST', 'QUAL_TR_HIST', $
        'QUAL_MTGL_CORR', 'QUAL_MTGL_CORR', $
        'QUAL_GPGL_CORR', 'QUAL_GPGL_CORR', $
        'QUAL_FF_CF', 'QUAL_FF_CF', $
        'QUAL_FF_PARAM', 'QUAL_FF_PARAM', $
        'QUAL_RP_TABLE', 'QUAL_RP_TABLE', $
        'QUAL_RP_PARAM', 'QUAL_RP_PARAM', $
        'QUAL_RP_DATA', 'QUAL_RP_DATA', $
        'QUAL_DF_EQ', 'QUAL_DF_EQ', $
        'QUAL_RC_CF', 'QUAL_RC_CF', $
        'QUAL_RC_PARAM', 'QUAL_RC_PARAM', $
        'QUAL_CV_PARAM', 'QUAL_CV_PARAM' ]

    fis_sw = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 39)
    fis_sw = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime',   '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1J', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti',    '1J', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti',    '1J', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         ; (2) status
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         {s, 'packetid',    '1B', '', '', 'PacketID', null }, $
         ; counter
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170',  '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60',   '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb',  '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw',  '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw',  '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt',    '11I', 'COUNTER', '', 'Mirror POS counter', null }, $
         ; level
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60',   '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170',  '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb',  '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'tpbody',  '1I', 'ADU', '', 'FIS body temperature', null }, $
         {s, 'tpsw',    '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw',    '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'det',  '100J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '100E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '100E', '', '', 'Error of flux', null }, $
         ; (5) Flags
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
;         {s, 'pix_flag', '100J', '', '', 'Flag for each pixel condition', null }, $
         {s, 'pix_flag', '3200X', '', '(32,100)', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         ; (6) Quality
         {s, 'quality', '4000X', '', '(40,100)', 'Quality for each pixel condition', PTR_NEW(fis_obs_quality_members) }, $
         ; (7) Counter
         {s, 'cnt_saa', '100I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_gp', '100I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_mt', '100I', '', '', 'Counter after spike for each pixel', null }]


    COMMON fis_lw_block, fis_lw    ; Common within this module
    fis_ndet = '75'
    fis_lw = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 39)
    fis_lw = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime',   '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1J', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti',    '1J', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti',    '1J', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         ; (2) status
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         {s, 'packetid',    '1B', '', '', 'PacketID', null }, $
         ; counter
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170',  '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60',   '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb',  '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw',  '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw',  '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt',    '11I', 'COUNTER', '', 'Mirror POS counter', null }, $
         ; level
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60',   '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170',  '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb',  '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'tpbody',  '1I', 'ADU', '', 'FIS body temperature', null }, $
         {s, 'tpsw',    '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw',    '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'det',  '75J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '75E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '75E', '', '', 'Error of flux', null }, $
         ; (5) Flags
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
;         {s, 'pix_flag', '75J', '', '', 'Flag for each pixel condition', null }, $
         {s, 'pix_flag', '2400X', '', '(32,75)', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         ; (6) Quality
         {s, 'quality', '3000X', '', '(40,75)', 'Quality for each pixel condition', PTR_NEW(fis_obs_quality_members) }, $
         ; (7) Counter
         {s, 'cnt_saa', '75I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_gp', '75I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_mt', '75I', '', '', 'Counter after spike for each pixel', null }]


    COMMON iso_block, iso    ; Common within this module
    iso_ndet = '4'
    iso = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 39)
    iso = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime',   '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1J', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti',    '1J', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti',    '1J', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         ; (2) status
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         {s, 'packetid',    '1B', '', '', 'PacketID', null }, $
         ; counter
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170',  '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60',   '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb',  '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw',  '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw',  '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt',    '11I', 'COUNTER', '', 'Mirror POS counter', null }, $
         ; level
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60',   '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170',  '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb',  '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'tpbody',  '1I', 'ADU', '', 'ISO body temperature', null }, $
         {s, 'tpsw',    '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw',    '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'det',  '4J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '4E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '4E', '', '', 'Error of flux', null }, $
         ; (5) Flags
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
;         {s, 'pix_flag', '4J', '', '', 'Flag for each pixel condition', null }, $
         {s, 'pix_flag', '128X', '', '(32,4)', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         ; (6) Quality
         {s, 'quality', '160X', '', '(32,4)', 'Quality for each pixel condition', PTR_NEW(fis_obs_quality_members) }, $
         ; (7) Counter
         {s, 'cnt_saa', '4I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_gp', '4I', '', '', 'Counter after spike for each pixel', null }, $
         {s, 'cnt_glitch_mt', '4I', '', '', 'Counter after spike for each pixel', null }]


    COMMON fis_hk_block, fis_hk    ; Common within this module
    fis_hk_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    fis_hk = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 6)
    fis_hk = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(fis_hk_status_members) }, $
         ; (3) analog values
         {s, 'fwfpul', '1B', 'COUNTER', '', 'Filter wheel pulse counter (Forward)', null }, $
         {s, 'fwrpul', '1B', 'COUNTER', '', 'Filter wheel pulse counter (Rewind)', null }, $
         {s, 'tmppol', '1B', '', '', 'Thermometer polarity', null }]


    COMMON irc_hk_block, irc_hk    ; Common within this module
    irc_hk_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    irc_hk = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 6)
    irc_hk = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(irc_hk_status_members) }, $
         ; (3) analog values
         {s, 'tp_pmrr', '1E', '', '', 'Temperature of prime mirror', null }, $
         {s, 'tp_smrr', '1E', '', '', 'Temperature of sub mirror', null }, $
         {s, 'tp_fpip', '1E', '', '', 'Temperature of FPI plate', null }]


    COMMON hk_2_block, hk_2    ; Common within this module
    hk_2_status_members = $
      [ '', '', '', '', '', '', 'INVALID', 'BLANK' ]
    hk_2 = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 5)
    hk_2 = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(hk_2_status_members) }, $
         ; (3) temperature
         {s, 'tp_he_tank', '1E', '', '', 'Temperature of He tank', null }, $
         {s, 'tp_baffle',  '1E', '', '', 'Temperature of baffle', null }]


    COMMON aocu_block, aocu    ; Common within this module
    aocu_status_members = $
      [ '', '', '', '', 'MODE', 'STT', 'TFSS', 'BLANK' ]
    aocu = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 14)
    aocu = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(aocu_status_members) }, $
         ; (3) analog values
         {s, 'quaternion', '4D', '', '', 'Quaternion at boresight', null }, $
         {s, 'pos_err',    '3D', '', '', 'Estimated error of position', null }, $
         {s, 'body_rate',  '3D', '', '', 'Angular velocity at boresight', null }, $
         {s, 'ra',      '1D', '', '', 'J2000 R.A. at boresight', null }, $
         {s, 'dec',     '1D', '', '', 'J2000 Dec. at boresight', null }, $
         {s, 'rang',    '1D', '', '', 'Roll angle at boresight', null }, $
         {s, 'dra',     '1D', '', '', 'Scan rate in J2000 R.A.', null }, $
         {s, 'ddec',    '1D', '', '', 'Scan rate in J2000 Dec.', null }, $
         {s, 'drang',   '1D', '', '', 'Scan rate in roll angle', null }, $
         {s, 'crs_off', '1D', '', '', 'Offset from nominal scan path', null }, $
         ; (4) Quality
         {s, 'quality', '8X', '', '', 'Quality for condition', null }]


    COMMON gads_block, gads    ; Common within this module
    gads_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ] ; Details will be discussed with ESA.
    gads = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 18)
    gads = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(gads_status_members) }, $
         ; (3) analog values
         {s, 'quaternion', '4D', '', '', 'Quaternion at boresight', null }, $
         {s, 'pos_err',    '3D', '', '', 'Estimated error of position', null }, $
         {s, 'body_rate',  '3D', '', '', 'Angular velocity at boresight', null }, $
         {s, 'ra',      '1D', '', '', 'J2000 R.A. at boresight', null }, $
         {s, 'dec',     '1D', '', '', 'J2000 Dec. at boresight', null }, $
         {s, 'rang',    '1D', '', '', 'Roll angle at boresight', null }, $
         {s, 'dra',     '1D', '', '', 'Scan rate in J2000 R.A.', null }, $
         {s, 'ddec',    '1D', '', '', 'Scan rate in J2000 Dec.', null }, $
         {s, 'drang',   '1D', '', '', 'Scan rate in roll angle', null }, $
         {s, 'el_sol',  '1D', '', '', 'Solar elongation', null }, $
         {s, 'aa_sol',  '1D', '', '', 'Solar avoidance angle', null }, $
         {s, 'aa_ear',  '1D', '', '', 'Earth avoidance angle', null }, $
         {s, 'aa_lun',  '1D', '', '', 'Lunar avoidance angle', null }, $
         {s, 'crs_off', '1D', '', '', 'Offset from nominal scan path', null }, $
         ; (4) Quality
         {s, 'quality', '8X', '', '', 'Quality for condition', null }]


    COMMON pr_block, pr    ; Common within this module
    pr_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    pr = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 18)
    pr = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(pr_status_members) }, $
         ; (3) analog values
         {s, 'quaternion', '4D', '', '', 'Quaternion at boresight', null }, $
         {s, 'pos_err',    '3D', '', '', 'Estimated error of position', null }, $
         {s, 'body_rate',  '3D', '', '', 'Angular velocity at boresight', null }, $
         {s, 'ra',      '1D', '', '', 'J2000 R.A. at boresight', null }, $
         {s, 'dec',     '1D', '', '', 'J2000 Dec. at boresight', null }, $
         {s, 'rang',    '1D', '', '', 'Roll angle at boresight', null }, $
         {s, 'dra',     '1D', '', '', 'Scan rate in J2000 R.A.', null }, $
         {s, 'ddec',    '1D', '', '', 'Scan rate in J2000 Dec.', null }, $
         {s, 'drang',   '1D', '', '', 'Scan rate in roll angle', null }, $
         {s, 'el_sol',  '1D', '', '', 'Solar elongation', null }, $
         {s, 'aa_sol',  '1D', '', '', 'Solar avoidance angle', null }, $
         {s, 'aa_ear',  '1D', '', '', 'Earth avoidance angle', null }, $
         {s, 'aa_lun',  '1D', '', '', 'Lunar avoidance angle', null }, $
         {s, 'crs_off', '1D', '', '', 'Offset from nominal scan path', null }, $
         ; (4) Quality
         {s, 'quality', '8X', '', '', 'Quality for condition', null }]


    COMMON se_block, se    ; Common within this module
    se_status_members = $
      [ '', '', '', 'IN_POLAR', 'IN_SAA', 'DAYNIGHT', 'TYPE', 'BLANK' ]
    se = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 7)
    se = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(se_status_members) }, $
         ; (3) analog values
         {s, 'tm_saa', '1D', 's', '', 'Time after latest SAA', null }, $
         {s, 'sat_x', '1D', '', '', '', null }, $
         {s, 'sat_y', '1D', '', '', '', null }, $
         {s, 'sat_z', '1D', '', '', '', null }]

END

;
; Period of validity: 2003 November -- 2004 June.
;
PRO tabledef_version2
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    null = PTR_NEW()

    COMMON fis_sw_block, fis_sw    ; Common within this module
    fis_ndet = '100'

    fis_obs_status_members = $
      [ '', '', '', '', '', '', '', '', '', '', '', 'SINASON', $
        'SINALON', 'CALBON', 'CALASON', 'CALALON', $
        'LWBIASON', 'SWBIASON', 'LWBOOSTON', 'SWBOOSTON', $
        'RSTN60ON', 'RSTN170ON', 'RSTWIDESON', 'RSTWIDELON', $
        'MPOSB0', 'MPOSB1', 'MPOSON', 'FWPOSB0', $
        'FWPOSB1', 'FWPOSON', 'SHTOP', 'CREON' ]
    fis_obs_flag_members = $
      [ '', '', '', 'BADRCDW', 'BADRCDN', 'NEAR_MOON', 'IN_SAA', 'BLANK' ]
    fis_obs_pix_flag_members = $
      [ '', '', '', '', $
        '', '', '', 'MTGL_TAIL', $
        'MTGL_TYPE4', 'MTGL_TYPE3', 'MTGL_TYPE2', 'MTGL_TYPE1', $
        '', '', '', 'GPGL_TAIL', $
        'GPGL_TYPE4', 'GPGL_TYPE3', 'GPGL_TYPE2', 'GPGL_TYPE1', $
        'TRSNTCRR_QL0', 'TRSNTCRR_QL1', 'SPIKE', 'SX_SOURCE', $
        '', 'BADDARK', 'NOSTIM', 'NODIFF', $
        'AFTERRST', 'SATURATE', 'DEAD', 'BAD' ]

    fis_sw = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 37)
    fis_sw = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime',   '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1J', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti',    '1J', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti',    '1J', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         ; (2) status
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         {s, 'packetid',    '1B', '', '', 'PacketID', null }, $
         ; counter
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170',  '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60',   '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb',  '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw',  '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw',  '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt',    '11I', 'COUNTER', '', 'Mirror POS counter', null }, $
         ; level
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60',   '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170',  '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb',  '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'tpbody',  '1I', 'ADU', '', 'FIS body temperature', null }, $
         {s, 'tpsw',    '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw',    '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'det',  '100J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '100E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '100E', '', '', 'Error of flux', null }, $
         ; (5) Flags
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
;         {s, 'pix_flag', '100J', '', '', 'Flag for each pixel condition', null }, $
         {s, 'pix_flag', '3200X', '', '(32,100)', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         {s, 'cnt_spk', '100I', '', '', 'Counter after spike for each pixel', null }]


    COMMON fis_lw_block, fis_lw    ; Common within this module
    fis_ndet = '75'
    fis_lw = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 37)
    fis_lw = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime',   '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1J', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti',    '1J', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti',    '1J', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         ; (2) status
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         {s, 'packetid',    '1B', '', '', 'PacketID', null }, $
         ; counter
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170',  '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60',   '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb',  '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw',  '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw',  '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt',    '11I', 'COUNTER', '', 'Mirror POS counter', null }, $
         ; level
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60',   '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170',  '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb',  '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'tpbody',  '1I', 'ADU', '', 'FIS body temperature', null }, $
         {s, 'tpsw',    '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw',    '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'det',  '75J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '75E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '75E', '', '', 'Error of flux', null }, $
         ; (5) Flags
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
;         {s, 'pix_flag', '75J', '', '', 'Flag for each pixel condition', null }, $
         {s, 'pix_flag', '2400X', '', '(32,75)', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         {s, 'cnt_spk', '75I', '', '', 'Counter after spike for each pixel', null }]


    COMMON iso_block, iso    ; Common within this module
    iso_ndet = '4'
    iso = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 37)
    iso = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime',   '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original), LSB=1.9ms(512.0Hz)', null }, $
         {s, 'fistiorg', '1J', 'COUNTER', '', 'FISTI (original), LSB=1.4ms(682.688Hz)', null }, $
         {s, 'pimti',    '1J', 'COUNTER', '', 'PIMTI (interpolated)', null }, $
         {s, 'fisti',    '1J', 'COUNTER', '', 'FISTI (interpolated)', null }, $
         ; (2) status
         {s, 'status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         {s, 'packetid',    '1B', '', '', 'PacketID', null }, $
         ; counter
         {s, 'rstcntwidel', '1B', 'COUNTER', '', 'Reset counter WIDE-L', null }, $
         {s, 'rstcntwides', '1B', 'COUNTER', '', 'Reset counter WIDE-S', null }, $
         {s, 'rstcntn170',  '1B', 'COUNTER', '', 'Reset counter N170', null }, $
         {s, 'rstcntn60',   '1B', 'COUNTER', '', 'Reset counter N60', null }, $
         {s, 'calplscntal', '1B', 'COUNTER', '', 'CAL pulse counter AL', null }, $
         {s, 'calplscntas', '1B', 'COUNTER', '', 'CAL pulse counter AS', null }, $
         {s, 'calplscntb',  '1B', 'COUNTER', '', 'CAL pulse counter B', null }, $
         {s, 'aderrcntsw',  '1B', 'COUNTER', '', 'SW A/D Error Counter', null }, $
         {s, 'aderrcntlw',  '1B', 'COUNTER', '', 'LW A/D Error Counter', null }, $
         {s, 'mposcnt',    '11I', 'COUNTER', '', 'Mirror POS counter', null }, $
         ; level
         {s, 'dacbiaswides', '1B', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'dacbiasn60',   '1B', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'dacbiaswidel', '1B', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'dacbiasn170',  '1B', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'daccalas', '1B', '', '', 'DAC CAL AS level', null }, $
         {s, 'daccalal', '1B', '', '', 'DAC CAL AL level', null }, $
         {s, 'daccalb',  '1B', '', '', 'DAC CAL B level', null }, $
         {s, 'dacsinas', '1B', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'dacsinal', '1B', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'tpbody',  '1I', 'ADU', '', 'ISO body temperature', null }, $
         {s, 'tpsw',    '1I', 'ADU', '', 'SW detector temperature', null }, $
         {s, 'tplw',    '1I', 'ADU', '', 'LW detector temperature', null }, $
         {s, 'tpcalft', '1I', 'ADU', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'det',  '4J', 'ADU', '', 'Detector output', null }, $
         {s, 'flux', '4E', '', '', 'Calibrated flux', null }, $
         {s, 'ferr', '4E', '', '', 'Error of flux', null }, $
         ; (5) Flags
         {s, 'flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
;         {s, 'pix_flag', '4J', '', '', 'Flag for each pixel condition', null }, $
         {s, 'pix_flag', '128X', '', '(32,4)', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         {s, 'cnt_spk', '4I', '', '', 'Counter after spike for each pixel', null }]


    COMMON fis_hk_block, fis_hk    ; Common within this module
    fis_hk = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 5)
    fis_hk = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) analog values
         {s, 'fwfpul', '1B', 'COUNTER', '', 'Filter wheel pulse counter (Forward)', null }, $
         {s, 'fwrpul', '1B', 'COUNTER', '', 'Filter wheel pulse counter (Rewind)', null }, $
         {s, 'tmppol', '1B', '', '', 'Thermometer polarity', null }]


    COMMON irc_hk_block, irc_hk    ; Common within this module
    irc_hk_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    irc_hk = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 6)
    irc_hk = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(irc_hk_status_members) }, $
         ; (3) analog values
         {s, 'tp_pmrr', '1E', '', '', 'Temperature of prime mirror', null }, $
         {s, 'tp_smrr', '1E', '', '', 'Temperature of sub mirror', null }, $
         {s, 'tp_fpip', '1E', '', '', 'Temperature of FPI plate', null }]


    COMMON hk_2_block, hk_2    ; Common within this module
    hk_2 = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 5)
    hk_2_status_members = $
      [ '', '', '', '', '', '', 'INVALID', 'BLANK' ]
    hk_2 = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(hk_2_status_members) }, $
         ; (3) temperature
         {s, 'tp_he_tank', '1E', '', '', 'Temperature of He tank', null }, $
         {s, 'tp_baffle',  '1E', '', '', 'Temperature of baffle', null }]


    COMMON aocu_block, aocu    ; Common within this module
    aocu_status_members = $
      [ '', '', '', '', 'MODE', 'STT', 'TFSS', 'BLANK' ]
    aocu = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 10)
    aocu = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(aocu_status_members) }, $
         ; (3) analog values
         {s, 'bra',     '1D', '', '', 'J2000 R.A. at boresite', null }, $
         {s, 'bdec',    '1D', '', '', 'J2000 Dec. at boresite', null }, $
         {s, 'ang',     '1D', '', '', 'Angle between scan dir. and the north', null }, $
         {s, 'dbra',    '1D', '', '', 'Scan rate in J2000 R.A.', null }, $
         {s, 'dbdec',   '1D', '', '', 'Scan rate in J2000 Dec.', null }, $
         {s, 'dang',    '1D', '', '', 'Scan rate in roll angle', null }, $
         {s, 'crs_off', '1D', '', '', 'Offset from nominal scan path', null }]


    COMMON ads_block, ads    ; Common within this module
    ads_status_members = $
      [ '', '', '', '', '', 'QUALITY', 'ADS_TYPE', 'BLANK' ]
    ads = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 13)
    ads = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(ads_status_members) }, $
         ; (3) analog values
         {s, 'bra',    '1D', '', '', 'J2000 R.A. at boresite', null }, $
         {s, 'bdec',   '1D', '', '', 'J2000 Dec. at boresite', null }, $
         {s, 'ang',    '1D', '', '', 'Angle between scan dir. and the north', null }, $
         {s, 'bra_err' , '1D', '', '', 'Error of bra', null }, $
         {s, 'bdec_err', '1D', '', '', 'Error of bdec', null }, $
         {s, 'ang_err',  '1D', '', '', 'Error of position angle', null }, $
         {s, 'el_sol',   '1D', '', '', 'Solar elongation', null }, $
         {s, 'aa_sol',   '1D', '', '', 'Solar avoidance angle', null }, $
         {s, 'aa_ear',   '1D', '', '', 'Earth avoidance angle', null }, $
         {s, 'aa_lun',   '1D', '', '', 'Lunar avoidance angle', null }]


    COMMON se_block, se    ; Common within this module
    se_status_members = $
      [ '', '', '', 'IN_POLAR', 'IN_SAA', 'DAYNIGHT', 'TYPE', 'BLANK' ]
    se = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 7)
    se = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, TDIM, COMMENT
         ; (1) time
         {s, 'aftime', '1D', 's', '', 'ASTRO-F satellite time (calibrated)', null }, $
         {s, 'pimtiorg', '1J', 'COUNTER', '', 'PIMTI (original)', null }, $
         ; (2) status
         {s, 'status', '8X', '', '', 'Status', PTR_NEW(se_status_members) }, $
         ; (3) analog values
         {s, 'tm_saa', '1D', 's', '', 'Time after latest SAA', null }, $
         {s, 'sat_x', '1D', '', '', '', null }, $
         {s, 'sat_y', '1D', '', '', '', null }, $
         {s, 'sat_z', '1D', '', '', '', null }]

END

;
; Period of validity: 2002 June -- 2003 November.
;
PRO tabledef_version1
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL

    null = PTR_NEW()

    COMMON fis_sw_block, fis_sw    ; Common within this module
    fis_ndet = '100'

    fis_obs_status_members = $
      [ '', '', '', '', '', '', '', '', '', '', '', '', $
        'SINASON', 'SINALON', 'CALBON', 'CALASON', $
        'CALALON', 'LWBIASON', 'SWBIASON', 'LWBOOSTON', $
        'SWBOOSTON', 'RSTN60ON', 'RSTN170ON', 'RSTWIDESON', $
        'RSTWIDELON', 'MPOSON', 'FWPOSON', 'SHTOP', $
        'CREON', 'OBSMODE', 'DETECTOR', 'BLANK' ]
    fis_obs_flag_members = $
      [ '', '', '', 'BADRCDW', 'BADRCDN', 'NEAR_MOON', 'IN_SAA', 'BLANK' ]
    fis_obs_pix_flag_members = $
      [ '', '', '', '', $
        'TRSNT_CRR_QUALITY', 'NO_DIF', 'AFT_RESET', 'CRSTLK_CRR', $
        'DARK_BAD', 'NO_STIM', 'SOURCE', 'SPIKE',  $
        'TRSNT_CRR', 'SATURATE', 'BAD', 'DEAD' ]
    fis_sw = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 32)
    fis_sw = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         {s, 'cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (1) time
         {s, 'fis_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'fis_status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         ; counter
         {s, 'fis_rstcntwidel', '1I', '', '', 'Reset counter WIDE-L', null }, $
         {s, 'fis_rstcntwides', '1I', '', '', 'Reset counter WIDE-S', null }, $
         {s, 'fis_rstcntn170',  '1I', '', '', 'Reset counter N170', null }, $
         {s, 'fis_rstcntn60',   '1I', '', '', 'Reset counter N60', null }, $
         {s, 'fis_calplscntal', '1I', '', '', 'CAL pulse counter AL', null }, $
         {s, 'fis_calplscntas', '1I', '', '', 'CAL pulse counter AS', null }, $
         {s, 'fis_calplscntb',  '1I', '', '', 'CAL pulse counter B', null }, $
         ; level
         {s, 'fis_dacbiaswides', '1I', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'fis_dacbiasn60',   '1I', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'fis_dacbiaswidel', '1I', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'fis_dacbiasn170',  '1I', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'fis_daccalas', '1I', '', '', 'DAC CAL AS level', null }, $
         {s, 'fis_daccalal', '1I', '', '', 'DAC CAL AL level', null }, $
         {s, 'fis_daccalb',  '1I', '', '', 'DAC CAL B level', null }, $
         {s, 'fis_dacsinas', '1I', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'fis_dacsinal', '1I', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'fis_tpbody',  '1E', '', '', 'FIS body temperature', null }, $
         {s, 'fis_tpsw',    '1E', '', '', 'SW detector temperature', null }, $
         {s, 'fis_tplw',    '1E', '', '', 'LW detector temperature', null }, $
         {s, 'fis_tpcalft', '1E', '', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'fis_vdet', fis_ndet+'D', 'mV', '', 'Detector output', null }, $
         {s, 'fis_flux', fis_ndet+'D', 'Jy', '', 'Calibrated flux', null }, $
         {s, 'fis_ferr', fis_ndet+'D', 'Jy', '', 'Error of flux', null }, $
         {s, 'fis_dark', fis_ndet+'D', '', '', 'Estimated dark current', null }, $
         {s, 'fis_abserr', '1D', '', '', 'Absolute calibration error', null }, $
         ; (5) Flags
         {s, 'fis_flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
         {s, 'fis_pix_flag', fis_ndet+'I', '', '', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         {s, 'fis_cnt_spk', fis_ndet+'J', '', '', 'Counter after spike for each pixel', null }, $
         ; (6) version
         {s, 'fis_version', '1B', '', '', 'Version', null }]


    COMMON fis_lw_block, fis_lw    ; Common within this module
    fis_ndet = '75'
    fis_lw = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 32)
    fis_lw = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         {s, 'cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (1) time
         {s, 'fis_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'fis_status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         ; counter
         {s, 'fis_rstcntwidel', '1I', '', '', 'Reset counter WIDE-L', null }, $
         {s, 'fis_rstcntwides', '1I', '', '', 'Reset counter WIDE-S', null }, $
         {s, 'fis_rstcntn170',  '1I', '', '', 'Reset counter N170', null }, $
         {s, 'fis_rstcntn60',   '1I', '', '', 'Reset counter N60', null }, $
         {s, 'fis_calplscntal', '1I', '', '', 'CAL pulse counter AL', null }, $
         {s, 'fis_calplscntas', '1I', '', '', 'CAL pulse counter AS', null }, $
         {s, 'fis_calplscntb',  '1I', '', '', 'CAL pulse counter B', null }, $
         ; level
         {s, 'fis_dacbiaswides', '1I', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'fis_dacbiasn60',   '1I', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'fis_dacbiaswidel', '1I', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'fis_dacbiasn170',  '1I', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'fis_daccalas', '1I', '', '', 'DAC CAL AS level', null }, $
         {s, 'fis_daccalal', '1I', '', '', 'DAC CAL AL level', null }, $
         {s, 'fis_daccalb',  '1I', '', '', 'DAC CAL B level', null }, $
         {s, 'fis_dacsinas', '1I', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'fis_dacsinal', '1I', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'fis_tpbody',  '1E', '', '', 'FIS body temperature', null }, $
         {s, 'fis_tpsw',    '1E', '', '', 'SW detector temperature', null }, $
         {s, 'fis_tplw',    '1E', '', '', 'LW detector temperature', null }, $
         {s, 'fis_tpcalft', '1E', '', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'fis_vdet', fis_ndet+'D', 'mV', '', 'Detector output', null }, $
         {s, 'fis_flux', fis_ndet+'D', 'Jy', '', 'Calibrated flux', null }, $
         {s, 'fis_ferr', fis_ndet+'D', 'Jy', '', 'Error of flux', null }, $
         {s, 'fis_dark', fis_ndet+'D', '', '', 'Estimated dark current', null }, $
         {s, 'fis_abserr', '1D', '', '', 'Absolute calibration error', null }, $
         ; (5) Flags
         {s, 'fis_flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
         {s, 'fis_pix_flag', fis_ndet+'I', '', '', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         {s, 'fis_cnt_spk', fis_ndet+'J', '', '', 'Counter after spike for each pixel', null }, $
         ; (6) version
         {s, 'fis_version', '1B', '', '', 'Version', null }]


    COMMON iso_block, iso    ; Common within this module
    iso_ndet = '4'
    iso = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 32)
    iso = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         {s, 'cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (1) time
         {s, 'iso_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'iso_status', '32X', '', '', 'Status', PTR_NEW(fis_obs_status_members) }, $
         ; (3) analog values
         ; counter
         {s, 'iso_rstcntwidel', '1I', '', '', 'Reset counter WIDE-L', null }, $
         {s, 'iso_rstcntwides', '1I', '', '', 'Reset counter WIDE-S', null }, $
         {s, 'iso_rstcntn170',  '1I', '', '', 'Reset counter N170', null }, $
         {s, 'iso_rstcntn60',   '1I', '', '', 'Reset counter N60', null }, $
         {s, 'iso_calplscntal', '1I', '', '', 'CAL pulse counter AL', null }, $
         {s, 'iso_calplscntas', '1I', '', '', 'CAL pulse counter AS', null }, $
         {s, 'iso_calplscntb',  '1I', '', '', 'CAL pulse counter B', null }, $
         ; level
         {s, 'iso_dacbiaswides', '1I', '', '', 'DAC BIAS Wide-S level', null }, $
         {s, 'iso_dacbiasn60',   '1I', '', '', 'DAC BIAS N60 level', null }, $
         {s, 'iso_dacbiaswidel', '1I', '', '', 'DAC BIAS Wide-L level', null }, $
         {s, 'iso_dacbiasn170',  '1I', '', '', 'DAC BIAS N170 level', null }, $
         {s, 'iso_daccalas', '1I', '', '', 'DAC CAL AS level', null }, $
         {s, 'iso_daccalal', '1I', '', '', 'DAC CAL AL level', null }, $
         {s, 'iso_daccalb',  '1I', '', '', 'DAC CAL B level', null }, $
         {s, 'iso_dacsinas', '1I', '', '', 'DAC CAL sin-conv. AS level', null }, $
         {s, 'iso_dacsinal', '1I', '', '', 'DAC CAL sin-conv. AL level', null }, $
         ; temperature
         {s, 'iso_tpbody',  '1E', '', '', 'ISO body temperature', null }, $
         {s, 'iso_tpsw',    '1E', '', '', 'SW detector temperature', null }, $
         {s, 'iso_tplw',    '1E', '', '', 'LW detector temperature', null }, $
         {s, 'iso_tpcalft', '1E', '', '', 'CAL FT temperature', null }, $
         ; (4) detector data
         {s, 'iso_vdet', iso_ndet+'D', 'mV', '', 'Detector output', null }, $
         {s, 'iso_flux', iso_ndet+'D', 'Jy', '', 'Calibrated flux', null }, $
         {s, 'iso_ferr', iso_ndet+'D', 'Jy', '', 'Error of flux', null }, $
         {s, 'iso_dark', iso_ndet+'D', 'Jy', '', 'Estimated dark current', null }, $
         {s, 'iso_abserr', '1D', '', '', 'Absolute calibration error', null }, $
         ; (5) Flags
         {s, 'iso_flag', '8X', '', '', 'Flag for detector condition', PTR_NEW(fis_obs_flag_members) }, $
         {s, 'iso_pix_flag', iso_ndet+'I', '', '', 'Flag for each pixel condition', PTR_NEW(fis_obs_pix_flag_members) }, $
         {s, 'iso_cnt_spk', iso_ndet+'J', '', '', 'Counter after spike for each pixel', null }, $
         ; (6) version
         {s, 'iso_version', '1B', '', '', 'Version', null }]


    COMMON irc_block, irc    ; Common within this module
    irc_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    irc = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 6)
    irc = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         ; (1) time
         {s, 'irc_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'irc_status', '8X', '', '', 'Status', PTR_NEW(irc_status_members) }, $
         ; (3) analog values
         {s, 'irc_tp_pmrr', '1D', '', '', 'Temperature of prime mirror', null }, $
         {s, 'irc_tp_smrr', '1D', '', '', 'Temperature of sub mirror', null }, $
         {s, 'irc_tp_fpip', '1D', '', '', 'Temperature of FPI plate', null }, $
         ; (4) version
         {s, 'irc_version', '1B', '', '', 'Version', null }]


    COMMON cryo_block, cryo    ; Common within this module
    cryo_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    cryo = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 5)
    cryo = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         ; (1) time
         {s, 'cryo_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'cryo_status', '8X', '', '', 'Status', PTR_NEW(cryo_status_members) }, $
         ; (3) temperature
         {s, 'cryo_tp_he_tank', '1D', '', '', 'Temperature of He tank', null }, $
         {s, 'cryo_tp_baffle',  '1D', '', '', 'Temperature of baffle', null }, $
         ; (4) version
         {s, 'cryo_version', '1B', '', '', 'Version', null }]


    COMMON aocs_block, aocs    ; Common within this module
    aocu_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    aocu = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 10)
    aocs = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         ; (1) time
         {s, 'aocs_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'aocs_status', '8X', '', '', 'Status', PTR_NEW(aocu_status_members) }, $
         ; (3) analog values
         {s, 'aocs_bra',     '1D', '', '', 'J2000 R.A. at boresite', null }, $
         {s, 'aocs_bdec',    '1D', '', '', 'J2000 Dec. at boresite', null }, $
         {s, 'aocs_ang',     '1D', '', '', 'Angle between scan dir. and the north', null }, $
         {s, 'aocs_dbra',    '1D', '', '', 'Scan rate in J2000 R.A.', null }, $
         {s, 'aocs_dbdec',   '1D', '', '', 'Scan rate in J2000 Dec.', null }, $
         {s, 'aocs_dang',    '1D', '', '', 'Scan rate in roll angle', null }, $
         {s, 'aocs_crs_off', '1D', '', '', 'Offset from nominal scan path', null }, $
         ; (4) version
         {s, 'aocs_version', '1B', '', '', 'Version', null }]

    COMMON se_block, se    ; Common within this module
    se_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    se = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 7)
    se = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         ; (1) time
         {s, 'se_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'se_status', '8X', '', '', 'Status', PTR_NEW(se_status_members) }, $
         ; (3) analog values
         {s, 'se_tm_saa', '1D', 's', '', 'Time after latest SAA', null }, $
         {s, 'se_sat_x', '1D', '', '', '', null }, $
         {s, 'se_sat_y', '1D', '', '', '', null }, $
         {s, 'se_sat_z', '1D', '', '', '', null }, $
         ; (4) version
         {s, 'se_version', '1B', '', '', 'Version', null }]


    COMMON point_block, point    ; Common within this module
    point_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    point = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 13)
    point = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         ; (1) time
         {s, 'point_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'point_status', '8X', '', '', 'Status', PTR_NEW(point_status_members) }, $
         ; (3) analog values
         {s, 'point_bra',    '1D', '', '', 'J2000 R.A. at boresite', null }, $
         {s, 'point_bdec',   '1D', '', '', 'J2000 Dec. at boresite', null }, $
         {s, 'point_ang',    '1D', '', '', 'Angle between scan dir. and the north', null }, $
         {s, 'point_bra_err' , '1D', '', '', 'Error of bra', null }, $
         {s, 'point_bdec_err', '1D', '', '', 'Error of bdec', null }, $
         {s, 'point_ang_err',  '1D', '', '', 'Error of position angle', null }, $
         {s, 'point_el_sol',   '1D', '', '', 'Solar elongation', null }, $
         {s, 'point_aa_sol',   '1D', '', '', 'Solar avoidance angle', null }, $
         {s, 'point_aa_ear',   '1D', '', '', 'Earth avoidance angle', null }, $
         {s, 'point_aa_lun',   '1D', '', '', 'Lunar avoidance angle', null }, $
         ; (4) version
         {s, 'point_version', '1B', '', '', 'Version', null }]


    COMMON hk_block, hk    ; Common within this module
    hk_status_members = $
      [ '', '', '', '', '', '', '', 'BLANK' ]
    hk = REPLICATE({s, ttype:'', tform:'', tunit:'', tdim:'', comment:'', telem:null}, 3)
    hk = [$
         ; TAG_NAME(=TTYPE), TFORM, TUNIT, COMMENT
         ; (1) time
         {s, 'hk_cti', '1J', '', '', 'Calibrated PIM TI', null }, $
         ; (2) status
         {s, 'hk_status', '8X', '', '', 'Status', PTR_NEW(hk_status_members) }, $
         ; (3) version
         {s, 'hk_version', '1B', '', '', 'Version', null }]

END

; Note that the method whose name is same as filename should be
; located at the end of the file.
; This seems to be a specification of RESOLVE_ROUTINE, 'func'.
PRO tabledef, version
    COMPILE_OPT hidden
    IF !DEBUG THEN MESSAGE, 'Called.', /INFORMATIONAL
;    CASE version OF
;        2: tabledef_version2
;        1: tabledef_version1
;        ELSE: MESSAGE, 'Unknown TSD version: ' + version
;    ENDCASE
END

