FUNCTION y_average_ramps,  $
   xin,                    $  ; input X data
   yin,                    $  ; input Y data
   XREF=xref,              $  ; X-grid corresponding to returned ramp (regrided)
   NOREGRID=norgrid,       $  ; do not regird (ignore xin)
   YRESD=yresd,            $  ; (returned) Residial from averaged ramp
   SIGMA=sigma,            $  ; (returned) Stdev
   TSIGMA=tsigma,          $  ; (returned) Average Stdev in whole array
   CNT=cnt                    ; (returned) number of data used for calculation

; average on a regrided space
; 04/05/25 bug fix on averaging
; 05/01/01 renamed as 'y_average_ramps'
; 05/01/17 check dimension of input data  before processing
;          rewrite carefully to behave gently
;          regird part is exported to y_regird_ramps.pro
; 05/02/06 bug fix at calculating sigma and tsigma
; 05/05/07 return XREF when regrided
; 06/03/19 YRESD added
; 06/08/18 calculation of tsigma improved (not include BAD data)
; 06/10/09 return proper XREF (had dimension of original data)

BAD = y_bad_val_def()

; dimension check
   x_dim = size(xin, /DIMENSIONS)
   if (n_elements(x_dim) NE 3) then begin
      MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(x_dim,/PRINT), /CONTINUE
      return, 0
   endif

   y_dim = size(yin, /DIMENSIONS)
   if (n_elements(y_dim) NE 3) then begin
      MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(y_dim,/PRINT), /CONTINUE
      return, 0
   endif

   if (Total((x_dim-y_dim)^2 NE 0)) then begin
      MESSAGE, "X, Y dimension inconsistent!", /CONTINUE
      return, 0
   endif

   nsmpl = x_dim[0]
   nramp = x_dim[1]
   ndet  = x_dim[2]

; regird ramp data
   if (NOT Keyword_set(NOREGRID)) then begin
      yr = y_regrid_ramps(xin, yin, xref=xr, /AUTOGRID)
   endif else begin
      yr = yin
      xr = y_xidx(yr)
   endelse

; do average
   c = replicate(0D, nsmpl, nramp, ndet)
   idx = where ( xr NE BAD AND yr NE BAD, cnt, COMPLEMENT=jdx, NCOMPLEMENT=jcnt )

   if (cnt GT 0) then begin
      c[idx] = 1D
      ct = Total (c, 2)
      idxc = where (ct EQ 0, cntc)
      if (cntc GT 0) then ct[idxc] = 1D
      avg = Total(yr * c, 2) / ct

      aa = reform(avg, nsmpl * ndet)
      aa = aa # replicate(1D, nramp)
      aa = reform(aa, nsmpl, ndet, nramp, /OVERWRITE)
      aa = Transpose(aa, [0, 2, 1])
      yd = yr - aa
      if (jcnt GT 0) then yd[jdx] = BAD

      std = Total(((yd) * c)^2, 2) / ct
      std = sqrt(std)

      if (cntc GT 0) then begin
         avg[idxc] = BAD
         std[idxc] = BAD
         ct[idxc] = 0D
      endif
   endif else begin
      avg = replicate(BAD, nsmpl, ndet)
      std = replicate(BAD, nsmpl, ndet)
      ct  = replicate(BAD, nsmpl, ndet)
   endelse

   avg = reform(avg, nsmpl, 1, ndet, /OVERWRITE)
   sigma=reform(std, nsmpl, 1, ndet)
   cnt=reform(ct, nsmpl, 1, ndet)
   x1=fix(nsmpl*0.2)
   x2=fix(nsmpl*0.5)
   tstd = std[x1:x2,*]
   nstd = replicate(1D, (x2-x1+1), ndet)
   idx = where(tstd EQ BAD, cnt)
   if (cnt GT 0) then begin
      tstd[idx] = 0D
      nstd[idx] = 0D
   endif
   ttstd = Total(tstd, 1)
   nnstd = Total(nstd, 1)
   idx = where(nnstd EQ 0D, cnt)
   if (cnt GT 0) then nnstd[idx] = 1D
   tsigma = ttstd / nnstd
   if (cnt GT 0) then tsigma[idx] = BAD

   xref=y_xidx(avg)
   yresd=yd

   aa = 0B & stv = 0B & c = 0B
   tstd = 0B & nstd = 0B & ttstd = 0B & nnstd = 0B

   return,avg
end
