FUNCTION y_Flag_AbsSaturation, v, datatype, LIMIT=limit
; a very simple tool to flag out absolutely (physically) saturated data points
; 06/03/20 I.Yamamura
; 06/03/30 I.Yamamura datatype for switching process for SW and LW
; 06/08/18 I.Yamamura subtract ramp start data
; 06/10/10 I.Yamamura bug fix. subtract min instead of start point
; 07/06/07 I.Yamamura LIMIT implemented. Flag data > physical saturation level * limit
; 07/09/30 I.Yamamura bug fix (avoid crash when cnt EQ 0)

; set default limit
if (N_ELEMENTS(limit) NE 1) then limit = 0.9D

; define constant
   BAD = y_bad_val_def()

   dim = LONG(SIZE(v, /DIMENSION))

; backup BAD data position
   idx = WHERE(v EQ BAD, cnt)

   d = y_Diff_Ramp(v)

; subtract minimum value in each ramp
   vv = v
   if (cnt GT 0) then vv[idx] = 1D8        ; any large number
   vm0 = MIN(vv, DIMENSION=1)
   vm = replicate(1D, dim[0]) # reform(vm0, dim[1] * dim[2])
   vm = reform(vm, dim[0], dim[1], dim[2], /OVERWRITE)
   vv = v - vm
   if (cnt GT 0) then vv[idx] = BAD
   vm = 0B & vm0 = 0B

; condition to detect saturation
   if (datatype EQ 'FIS_SW') then begin
      idx = where( (vv GT 0.09D) AND (d^2 LT 3E-7), cnt)
   endif else if (datatype EQ 'FIS_LW') then begin
      idx = where( (vv GT 0.15D) AND (d^2 LT 1E-6), cnt)
   endif else begin
      lmessage, 0, 'Invalid DATATYPE='+STRING(datatype, /PRINT)
   endelse
   
   if (cnt GT 0) then begin

; calculate saturation level
      nd = REPLICATE(0D, dim[0] * dim[1], dim[2])
      isat = ARRAY_INDICES(nd, idx)
      nd[idx] = 1D

      vt = REFORM(v, dim[0] * dim[1], dim[2]) * nd
      vt = TOTAL(vt, 1)
      nt = TOTAL(nd, 1)
      idx = WHERE (nt EQ 0D, cnt)
      if (cnt GT 0) then nt[idx] = 1D

      vsat = (vt / nt) * limit
      idet = WHERE (vsat GT 0D, cdet)
      if (cdet GT 0) then begin
         for i = 0, cdet-1 do begin
            vi = v[*, *, idet[i]]
            idx = WHERE(vi GT vsat[idet[i]], cnt)
            if (cnt GT 0) then vi[idx] = BAD
            v[*, *, idet[i]] = vi
         endfor
      endif

;     if (cnt GT 0) then begin
;        v[idx] = BAD
;     endif

   endif

   lmessage, 5, STRING(cnt, /PRINT)+'  data points are saturated.'

   return, v
END
