FUNCTION y_Flag_Glitched_Ramps_int, ramp, LIMIT=limit, PLOTGR=plotgr
; 07/06/03 I.Yamamura based on y_Flag_Glictched_ramps.pro
; flagging glitched ramps. Detection on the integrated ramp.
;
; ramp data: ramp[nsmpl, nramps, ndets]
; limit: threshould in sigma
; /PLOT: plot glitched ramps

if (n_elements(LIMIT) LE 0) then limit = 3D

BAD = y_bad_val_def()

r_dim = size(ramp, /DIMENSIONS)
if (n_elements(r_dim) NE 3) then begin
   MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(r_dim,/PRINT)
   return, ramp
endif
nsmpl = r_dim[0]
nramp = r_dim[1]
ndet  = r_dim[2]
ndata = nsmpl * nramp

total_idx = -1L
total_cnt = 0L
r = ramp
LOOP_MAX = 20
i = 0

for loop = 0, LOOP_MAX-1 do begin
   idx  = WHERE(r EQ BAD, cnt)
   if (cnt GT 0) then r[idx] = !VALUES.F_NAN

   rmax = MAX(r, DIMENSION=1, /NAN)
   rmed = MEDIAN(rmax, DIMENSION=1, /DOUBLE)

   rmax = rmax -  REPLICATE(1D, nramp) # rmed

   ; calculate standard deviation
   nmax = REPLICATE(1D, nramp, ndet)
   idx = WHERE(rmax EQ !VALUES.F_NAN, cnt)
   if (cnt GT 0) then nmax[idx] = 0D
   tmax = TOTAL(nmax, 1)
   idx = WHERE(tmax EQ !VALUES.F_NAN, cnt)
   if (cnt GT 0) then tmax[idx] = 1D
   rstd = SQRT(TOTAL(rmax^2, 1, /NAN) / DOUBLE(tmax))

   rmax = rmax -  REPLICATE(1D, nramp) # (rstd * limit)

   idx  = WHERE (rmax GT 0D, cnt)

   mes = STRING(FORMAT='(I, A, I, A)', cnt, ' ramps are flagged at loop ', loop, '.')
   MESSAGE, mes, /CONTINUE

   if (cnt EQ 0) then break

   idx_arr = ARRAY_INDICES(rmax, idx)
   for i = 0, cnt-1 do r[*, idx_arr[0, i], idx_arr[1, i]] = BAD

   if (total_cnt EQ 0) then total_idx = idx else total_idx = [total_idx, idx]
   total_cnt += cnt

endfor

if (i GE LOOP_MAX) then begin
   MESSAGE, 'Maximum iteration reached.', /CONTINUE
endif

mes = STRING(FORMAT='(I, A)', total_cnt, ' ramps are flagged in total.')
MESSAGE, mes, /CONTINUE

if (total_cnt GT 0) then begin

   r = ramp
   rmax = MAX(r, DIMENSION=1, /NAN)
   rmed = MEDIAN(rmax, DIMENSION=1, /DOUBLE)

   rmax = rmax -  REPLICATE(1D, nramp) # rmed

   idx = total_idx[SORT(total_idx)]
   idx_arr = ARRAY_INDICES(INTARR(nramp, ndet), idx)

   print, "Flagged ramps are:"
   print, FORMAT='(A5,A5,A7,A8)', '#Det', '#Ramp', '#idx', 'Value'
   idet_b = -1
   for i = 0, total_cnt-1 do begin
      r[*, idx_arr[0, i], idx_arr[1, i]] = BAD

      print, FORMAT='(I5, I5, E12.4)', idx_arr[1, i]+1, idx_arr[0, i], $
         rmax[idx_arr[0, i], idx_arr[1, i]]
      if (keyword_set(PLOTGR)) then begin
         if (idx_arr[1, i] NE idet_b) then begin
            stop
            y_plot_ramp, ramp[*, idx_arr[0, i], idx_arr[1, i]], psym=3, /NODATA, $
               TITLE='Det #'+STRTRIM(STRING(idx_arr[1, i]+1),2)
         endif
         y_plot_ramp, ramp[*, idx_arr[0, i], idx_arr[1, i]], /OVER
         idet_b = idx_arr[1, i]
      endif

   endfor
   ramp_x = reform(r, nsmpl, nramp, ndet, /OVERWRITE)
   xx = y_xidx(ramp_x)

   mes = STRING(FORMAT='(I, A)', total_cnt, ' ramps are flagged as glitched.')
   MESSAGE, mes, /CONTINUE

endif else begin
   ramp_x = ramp
   MESSAGE, "No glitched ramp found.", /CONTINUE
endelse

return, ramp_x
end
