PRO y_Plot_cmp_One_Ramp, $
   ramp_x0,          $ ; X-axis data (sould be consistent with ramp_x)
   ramp_y0,          $ ; y-axis data (ramp_y0[nsamples, nramps, ndets])
   idet,             $ ; pixel index to plot
   REF=ref,          $ ; plot differential from this reference ramp
                       ; not given or < 0 -> search for a first unflagged ramp
   AREF_Y=aref_y,    $ ; external reference ramp [nsamples,1,ndets]
   AREF_X=aref_x,    $ ; corresponding X-dimension
   ORDER=order,      $ ; plot order
   ALLDATA=alldata,  $ ; plot all data, default is average deviation and stdev
   DIFF=diff,        $ ; plot residual from the reference
   TITLE=title,      $ ; additional title
   YRMS=yrms,        $ ; (returned) StDev from smoothed ramp curve
   TAVG=tavg,        $ ; (returned) Deviation averaged over all available ramps
   TSIG=tsig,        $ ; (returned) StDev of the deviation over all ramps
   ERRCODE=errcode,  $ ; returned error code
   _EXTRA=extra        ; anything else

; 05/05/07 I.Yamamura Extracted from y_Plot_cmp_Ramps
;          plot one detector data
; 06/03/18 I.Yamamura add YRMS option to return noise level
; 06/03/18 I.Yamamura add AREF_X, AREF -> AREF_Y
; 06/03/21 I.Yamamura clean up whole code
; 06/10/11 I.Yamamura use INTERPOL instead of SPLINE

; set CONSTANT
   BAD = y_bad_val_def()

; prepare input data array
   if ( n_params() EQ 2 ) then begin
      ramp_y = ramp_x0
      ramp_x = y_xidx(ramp_y)
   endif else begin
      ramp_x = ramp_x0
      ramp_y = ramp_y0
   endelse

; check input data size
   r_dim = size(ramp_y, /DIMENSIONS)
   if (n_elements(r_dim) NE 3) then begin
      MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(r_dim,/PRINT)
      return
   endif
   nsmpl = LONG(r_dim[0])
   nramp = LONG(r_dim[1])
   ndet  = LONG(r_dim[2])

; set plot order and coloring
   if (NOT Keyword_Set(ORDER)) then order = indgen(nramp)
   norder = n_elements(order)
   if (norder LE 1) then begin
      Message, "Give at least two ramps.", /continue
      errcode = 2
      return
   endif

   if (nramp GT 1) then pcol = fix(60 + (240-60) / double(nramp-1) * order)

; Initialize returned value
   tavg = BAD
   tsig = BAD
   yrms = BAD

; Initialize error code
   errcode = 0

; Plot title
   sdet = 'Det.# = '+STRTRIM(STRING(idet+1), 2)
   if (Keyword_set(TITLE)) then sdet = title + ' ' + sdet

; Plot mode setup.
   if (n_elements(DIFF) EQ 0) then begin
      dodiff = 0        ; Raw data by default
      if ((n_elements(REF) GT 0) OR (n_elements(AREF_Y) GT 0)) then dodiff = 1
   endif else begin
      if (diff EQ 0) then dodiff = 0 else dodiff = 1
   endelse

   if (n_elements(ALLDATA) EQ 0) then begin
      plotall = 1       ; Plot all data by default
   endif else begin
      if (alldata EQ 0) then plotall = 0 else plotall = 1
   endelse

; set up reference ramp
   iref = -1

   if (n_elements(AREF_Y) GT 0) then begin
; if reference ramp is given externally

      rdim = SIZE(aref_y, /DIMENSION)
      if (rdim[0] NE nsmpl OR rdim[2] NE ndet) then begin
         MESSAGE, "AREF dimension inconsitent. ignored.", /CONTINUE
         errcode = 1
         return
      endif

      ramp_y_ref = aref_y[*, 0, idet]
      if (n_elements(aref_x) EQ n_elements(aref_y)) then begin
         ramp_x_ref = aref_x[*, 0, idet]
      endif else begin
         ramp_x_ref = dindgen(nsmpl, 1, 1)
      endelse

   endif else begin
; if reference is withing the array

; check if REF option is given
      if (n_elements(REF) LE 0) then ref = -1

; if reference ramp is within the array
      if ((ref GE 0) AND (ref LT nramp)) then begin
         iref = ref

      endif else begin

; auto search
         idx = where ((ramp_x[*, *, idet] NE BAD) $
                  AND (ramp_y[*, *, idet] NE BAD), cnt)
         if (cnt EQ 0) then begin
            lmessage, 5, 'No valid data for pixel #'+STRING(idet,/PRINT)
            return
         endif
         adx = array_indices(ramp_x[*, *, idet], idx)
         iref = adx[1, 0]
         lmessage, 7, 'Select ramp #'+STRING(iref,/PRINT)+' as the reference.'

      endelse

      ramp_x_ref = ramp_x[*, iref, idet]
      ramp_y_ref = ramp_y[*, iref, idet]

   endelse

; make smoothed reference ramp and calculate rms noise level
; this is done regardless the plot mode

; select healty data points
   idx = where ((ramp_x_ref NE BAD) AND (ramp_y_ref NE BAD), cnt)

; we need at least two data points
   if (cnt LT 2) then begin
      errcode = 2
      return
   endif

   xr = ramp_x_ref[idx]
   yr = ramp_y_ref[idx]

; make sure that data is ascending order
   idx = sort(xr)
   xr = xr[idx]
   yr = yr[idx]

; set smoothing width
   width = n_elements(yr) / 16
   if (width GT  7) then width = 7 $
   else if (width LT 3) then width = 3
; smoothing
   if (n_elements(yr) GT width) then yt = smooth(yr, width) else yt = yr

; here estimating the noise level
   ys = yr - yt
   yrms = Sqrt(Total(ys^2) / double(n_elements(yr)-1))

; calculate difference from the reference ramp for each ramp
   if (dodiff EQ 1) then begin
      for iz = 0L, nramp-1 do begin
; take one ramp
         xx = ramp_x[*, iz, idet]
         yy = ramp_y[*, iz, idet]

; select healty data point
         idx = where (xx NE BAD AND yy NE BAD, cnt)
         if (cnt GT 0) then begin
            xx = xx[idx]
            yy = yy[idx]
         endif else continue

; calculate difference. reference data is regridded by spline
         if (iz EQ iref) then begin
           yy = ys
         endif else begin
            xmin = min(xr, max=xmax)
            ixx = where (xx GE xmin AND xx LE xmax, cntxx, $
               COMPLEMENT=ixxc, NCOMPLEMENT=cntxxc)
            if (cntxx GT 0) then begin
               ixx2 = sort(xx[ixx])
               ixx = ixx[ixx2]
;               yy[ixx] = yy[ixx] - SPLINE(xr, yt, xx[ixx], /DOUBLE)
               yy[ixx] = yy[ixx] - INTERPOL(yt, xr, xx[ixx], /QUADRATIC)
            endif
            if (cntxxc GT 0) then yy[ixxc] = BAD
         endelse

; Input data array is replaced with difference data
         ramp_y[*,   iz, idet] = replicate(BAD, nsmpl)
         ramp_y[idx, iz, idet] = yy
      endfor
   endif

; calculate avarage and stdev
   if (plotall EQ 0) then begin
      if (nramp LT 3) then begin
; need nramp >= 3 to store the results
         ramp_y_new = replicate(BAD, nsmpl, 3, ndet)
         ramp_x_new = ramp_y_new
         ramp_y_new[*, 0:(nramp-1), *] = ramp_y
         ramp_x_new[*, 0:(nramp-1), *] = ramp_x
         ramp_y = ramp_y_new
         ramp_x = ramp_x_new
         ramp_y_new = 0B & ramp_x_new = 0B
         nramp = 3
      endif
          
; make averaged map
      ramp_x_tmp = reform(ramp_x[*, *, idet], nsmpl, nramp, 1)
      ramp_y_tmp = reform(ramp_y[*, *, idet], nsmpl, nramp, 1)
      ramp_y_avg = y_average_ramps(ramp_x_tmp, ramp_y_tmp, XREF=ramp_x_avg, SIG=ramp_y_sig)

; store results
      ramp_y[*, *, idet] = replicate(BAD, nsmpl, nramp)
      ramp_y[*, 0, idet] = ramp_y_avg
      ramp_y[*, 1, idet] = ramp_y_sig
      ramp_y[*, 2, idet] = ramp_y_ref
      ramp_x[*, 0, idet] = ramp_x_avg[*,0,0]
      ramp_x[*, 1, idet] = ramp_x_avg[*,0,0]
      ramp_x[*, 2, idet] = ramp_x_ref

; calculate averaged average and StDev
      idx = where(ramp_y_avg NE BAD AND ramp_y_sig NE BAD, cnt)
      if (cnt GT 0) then begin
         tavg = Total(ramp_y_avg[idx]) / double(cnt)
         tsig = Total(ramp_y_sig[idx]) / double(cnt)
      end

      ramp_x_tmp = 0B & ramp_y_tmp = 0B
      ramp_y_avg = 0B & ramp_y_sig = 0B
   endif

; set plot range
   ramp_x_tmp = ramp_x[*, *, idet]
   ramp_y_tmp = ramp_y[*, *, idet]
   idx = where(ramp_x_tmp NE BAD AND ramp_y_tmp NE BAD, cnt)
   if (NOT Keyword_Set(XRANGE)) then begin
      if (cnt GT 0) then begin
         xmin = min(ramp_x_tmp[idx], max=xmax)
         xrng = [xmin, xmax]
      endif else xrng = [0, 0]
   endif else xrng = xrange

   if (NOT Keyword_Set(YRANGE)) then begin
      if (cnt GT 0) then begin
         ymin = min(ramp_y_tmp[idx], max=ymax)
         yrng = [ymin, ymax]
      endif else yrng = [0, 0]
   endif else xrng = xrange

; do plot

   if (plotall EQ 0) then begin
; plot avarage + StdDv
      y_plot_ramp, ramp_x[*, 0, idet], ramp_y[*, 0, idet], $
         ERROR=ramp_y[*, 1, idet], $
         XRANGE=xrng, YRANGE=yrng, $
         COLOR=220, psym=3, TITLE=sdet, _EXTRA = extra
      ox = !P.CLIP[0] * 0.9 + !P.CLIP[2] * 0.1
      oy = !P.CLIP[1] * 0.9 + !P.CLIP[3] * 0.1
      msg = 'RMS='+STRING(tsig*1e6, FORMAT='(F6.1)')+'E-06'
      xyouts, ox, oy, msg, /DEVICE, COLOR=220, CHARSIZE=0.5

      if ((dodiff NE 0) AND (iref GE 0)) then $
         y_plot_ramp, ramp_x[*,2,idet], ramp_y[*,2,idet], $
            COLOR=60, psym=3, TITLE=sdet, _EXTRA = extra, /OVER
   endif else begin
; plot all data
      first = 0
      for io = 0, norder-1 do begin
         iz = order[io]
         if (iz GE nramp) then continue
         pr_x = ramp_x[*,iz,idet]
         pr_y = ramp_y[*,iz,idet]
         idx = where (pr_x NE BAD and pr_y NE BAD, cnt)

         if (first EQ 0) then begin
            if (cnt EQ 0) then continue
            y_plot_ramp, pr_x, pr_y, $
               XRANGE=xrng, YRANGE=yrng, $
               COLOR=pcol[io], psym=3, TITLE=sdet, _EXTRA = extra
            ox = !P.CLIP[0] * 0.9 + !P.CLIP[2] * 0.1
            oy = !P.CLIP[1] * 0.9 + !P.CLIP[3] * 0.1
            msg = 'RMS='+STRING(yrms*1e6, FORMAT='(F6.1)')+'E-06'
            xyouts, ox, oy, msg, /DEVICE, COLOR=pcol[io], CHARSIZE=0.5
            first = 1
         endif else begin
            if (cnt GT 0) then y_plot_ramp, pr_x, pr_y, $
               COLOR=pcol[io], psym=3, TITLE=sdet, _EXTRA = extra, /OVER
         endelse
      endfor
   endelse

END
