PRO y_Plot_cmp_Ramps, $
   ramp_x0,          $ ; X-axis data (sould be consistent with ramp_x)
   ramp_y0,          $ ; y-axis data (ramp_y0[nsamples, nramps, ndets])
   REF=ref,          $ ; plot differential from this reference ramp
   AREF_Y=aref_y,    $ ; external reference ramp [nsamples,1,ndets]
   AREF_X=aref_x,    $ ; corresponding X-dimension
   ORDER=order,      $ ; plot order
   XRANGE=xrange,    $ ; plot order
   ALLDATA=alldata,  $ ; plot all data, default is average deviation and stdev
   DIFF=diff,        $ ; plot residual from the reference
   TITLE=title,      $ ; additional title
   MEMO=memo,        $ ; message on the top
   YRMS=yrms,        $ ; (returned) StDev from the smoothed curve
   TAVG=tavg,        $ ; Deviation averaged over all available ramps
   TSIG=tsig,        $ ; StDev of the deviation over all available ramps
   _EXTRA=extra        ; anything else
;
; 04/07/12 I.Yamamura from plot_7_ramps.pro
; 04/12/10 REF compares ratio -> difference wrt average differenciated value
; 04/12/12 REF plots difference of absolute signal + RMS of the reference
; 04/12/14 do not use /EDGE_TRUNCATE option at smooth()
; 04/12/20 AREF implemented
; 05/01/01 renamed with 'y_'
; 05/01/17 default plot is average and stdev of deviation for differential plot
;          /ALLDATA plots all data as previously
; 05/01/20 add /DOUBLE in SPLINE()
; 05/01/20 proper handling of bad data to avoid floating error in SPLINE()
; 05/01/23 work properly when compare only two ramps
; 05/01/31 plot range automatically estimated if not given.
; 05/02/01 give smaller smooth() width for small ramps
; 05/02/05 give X data explicitly when average is calculated to avoid
;          unexpected 'No valid data' when the 1st ramp is flagged.
; 05/02/06 color allocation slightly modified
; 05/02/06 return tavg and tsig, deviation from the reference ramp.
; 05/02/07 bug fix not to plot when the first ramp is flagged.
; 05/02/14 xidx() -> y_xidx()
; 05/02/26 inherit _EXTRA to y_plot_ramps.pro
; 05/05/07 default Smooth() width 11->7 (residual ~max, curveture small)
; 05/05/07 bug fix. X-index of the averaged ramp was not consistent with Y-data.
; 05/05/30 add XRANGE to specify the data plot range
; 06/03/18 add YRMS option to return RMS noise level.
; 06/03/18 clean up some part of codes
; 06/10/09 set automatic X-range properly

   BAD = y_bad_val_def()
   loadct, 39

   if ( n_params() EQ 1 ) then begin
      ramp_y = ramp_x0
      ramp_x = y_xidx(ramp_y)
   endif else begin
      ramp_x = ramp_x0
      ramp_y = ramp_y0
   endelse

   r_dim = size(ramp_y, /DIMENSIONS)
   if (n_elements(r_dim) NE 3) then begin
      MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(r_dim,/PRINT)
      return
   endif
   nsmpl = LONG(r_dim[0])
   nramp = LONG(r_dim[1])
   ndet  = LONG(r_dim[2])

   if (ndet EQ 100) then begin
      np = 4
      nx = 20
   endif else if (ndet EQ 75) then begin
      np = 3
      nx = 15
   endif else begin
      MESSAGE, 'Invalid number of detectors ', ndet
   endelse

if (n_elements(XRANGE) LT 2) then begin
   idx = WHERE (ramp_x NE BAD, cnt)
   if (cnt GT 0) then xmin = min(ramp_x[idx], max=xmax)
   dx = xmax - xmin
   xrange=[xmin - dx * 0.05, xmax + dx * 0.05]
endif

tavg = replicate(BAD, ndet) & tavgi = BAD
tsig = replicate(BAD, ndet) & tsigi = BAD
yrms = replicate(BAD, ndet) & yrmsi = BAD

; plot data
   for ip = 0, np-1 do begin
      erase
      for iy = 0, 4 do begin
         for ix = 0, 4 do begin
            !P.MULTI = [25 - iy*5 - ix, 5, 5]
            idet = ip * 5 + iy * nx + ix

            y_Plot_cmp_One_Ramp, ramp_x, ramp_y, idet, REF=ref, $
               AREF_X=aref_x, AREF_Y=aref_y, DIFF=diff, $
               XRANGE=xrange, ORDER=order, ALLDATA=alldata, TITLE=title, $
               YRMS=yrmsi, TAVG=tavgi, TSIG=tsigi, ERRCODE=errcode, _EXTRA=extra
            tavg[idet] = tavgi
            tsig[idet] = tsigi
            yrms[idet] = yrmsi
            if (errcode EQ 1) then break else if (errcode EQ 2) then continue

         endfor
      endfor
      if (keyword_set(MEMO)) then xyouts, 0.02, 0.98, /NORMAL, memo
      timestamp
   endfor

   !P.MULTI=[0,1,1]

end
