PRO y_Plot_Corr_Tbl, $
   ctbl,          $  ; correction table structure (defined in y_make_corr_tbl.pro)
   det,           $  ; scalar or array of detector index [0..99/74]
   XRANGE=xrange, $  ; plot range (sampling index)
   VRANGE=vrange, $  ; plot range (detector signal)
   CRANGE=crange, $  ; plot range (correction factor)
   _EXTRA=_extra
;
; 07/01/05 I.Yamamura from y_make_corr_tbl.pro

BAD = y_bad_val_def()

; prepare for plot
   pm_b = !P.MULTI
   !P.MULTI=[2, 2, 1]
   erase

; table size
   nv = N_ELEMENTS(ctbl.v)
   nd = N_ELEMENTS(det)
   nt = N_ELEMENTS(ctbl)
   x = indgen(nv)

; plot table
   for id = 0, nd-1 do begin
      idet = det[id]
      if (idet LT 0 OR idet GE nt) then begin
         LMESSAGE, 5, 'det out of range ('+idet+'). Skip.', /CONTINUE
         continue
      endif

      idx = WHERE ((ctbl[idet].v NE BAD) AND (ctbl[idet].c NE BAD), cnt)
      if (cnt LE 0) then begin
         LMESSAGE, 5, 'No valid data ('+idet+'). Skip.', /CONTINUE
         continue
      endif

      xx = x[idx]
      vv = ctbl[idet].v[idx]
      cc = ctbl[idet].c[idx]

      xmin = MIN (xx, max=xmax) & dx = xmax - xmin
      vmin = MIN (vv, max=vmax) & dv = vmax - vmin
      cmin = MIN (cc, max=cmax) & dc = cmax - cmin

      if (NOT KEYWORD_SET(XRANGE)) then xrange = [xmin - dx*0.05, xmax + dx*0.05]
      if (NOT KEYWORD_SET(VRANGE)) then vrange = [vmin - dv*0.05, vmax + dv*0.05]
      if (NOT KEYWORD_SET(CRANGE)) then crange = [cmin - dc*0.05, cmax + dc*0.05]

      plot, xx, vv, xstyle=1, ystyle=1, PSYM=1, TITLE='Original ramp.', $
         XRANGE=xrange, YRANGE=vrange, _EXTRA=_extra
      oplot, xx, vv + cc, color=240
      plot, vv, cc, xstyle=1, ystyle=1, PSYM=-1, TITLE='Correction Factor', $
         XRANGE=vrange, YRANGE=crange, _EXTRA=_extra

      souts = STRING(FORMAT='(A, I4)', 'Det = ', idet+1)
      xyouts, 0.5, 0.98, souts, ALIGNMENT=0.5, /NORMAL
   end

; post plot
   !P.MULTI=pm_b

return
end
