PRO y_Plot_Ramp, $
   ramp_x0,          $ ; X-axis data
   ramp_y0,          $ ; y-axis data (sould be consistent with ramp_x)
   BADPIX=badpix,    $ ; do not show BAD data if specified.
   ERROR =ramp_e0,   $ ; error
   XRANGE=xrange,    $ ; X-range
   YRANGE=yrange,    $ ; Y-range
   PSYM=psym,        $ ; symbol
   COLOR=color,      $ ; color
   OVERLAY=overlay,  $ ; overlay if specified
   FOLDING=folding,  $ ; self folding plot
   TITLE=title,      $ ; Title
   XTITLE=xtitle,    $ ; X-title
   YTITLE=ytitle,    $ ; Y-title
   _EXTRA=extra        ; anything else

; 04/03/03 add BADPIX option
;          add FOLDING option
; 04/12/14 data clip within the plot range to avoid PS error
; 05/01/01 add 'y_' to function name
; 05/01/17 use y_bad_val_def()
; 05/01/17 ERROR plot properly implemented
; 05/01/20 check BAD in ramp_x0
; 05/02/26 inherit _EXTRA to plot.pro

; local variables
   minx = 0.0 & maxx = 0.0 & dx = 0.0
   miny = 0.0 & maxy = 0.0 & dy = 0.0

; define bad value
   BAD = y_bad_val_def()

; setup arrays and remove bad data
   if ( n_params() EQ 1 ) then begin
      idx = where ( ramp_x0 NE BAD, count)
      if ( count EQ 0 ) then begin
         MESSAGE, "No valid data.", /CONTINUE
         return
      endif
      ramp_x = double(idx)
      ramp_y = ramp_x0[idx]
      if (n_elements(ramp_e0) GT 0) then ramp_e = ramp_e0[idx]
      if (keyword_set(FOLDING)) then begin
         dim = size(ramp_x0, /dimension)
         ramp_x = ramp_x MOD dim[0]
      endif
   endif else begin
      idx = where ( ramp_x0 NE BAD AND ramp_y0 NE BAD, count)
      if ( count EQ 0 ) then begin
         MESSAGE, "No valid data.", /CONTINUE
         return
      endif
      ramp_x = ramp_x0[idx]
      ramp_y = ramp_y0[idx]
      if (n_elements(ramp_e0) GT 0) then ramp_e = ramp_e0[idx]
      if (keyword_set(FOLDING)) then begin
         dim = size(ramp_x0, /dimension)
         ramp_x = ramp_x MOD dim[0]
      endif
   endelse

   if ( n_elements(badpix) EQ n_elements(ramp_x0)) then begin
      bp = badpix[idx]
      idx = where ( bp EQ !FALSE, count)
      if ( count GT 0 ) then begin
         ramp_x = ramp_x[idx]
         ramp_y = ramp_y[idx]
         if (n_elements(ramp_e0) GT 0) then ramp_e = ramp_e0[idx]
      endif
   endif

; set plot range
   if (n_elements(XRANGE) EQ 2) then begin
      minx = xrange[0]
      maxx = xrange[1]
   endif else begin
      minx = min(ramp_x, MAX=maxx)
      dx = maxx - minx
      minx = minx - dx / 10.0
      maxx = maxx + dx / 10.0
   endelse

   if (n_elements(YRANGE) EQ 2) then begin
      miny = yrange[0]
      maxy = yrange[1]
   endif else begin
      miny = min(ramp_y, MAX=maxy)
      dy = maxy - miny
      miny = miny - dy / 10.0
      maxy = maxy + dy / 10.0
   endelse

;print, minx, maxx, miny, maxy

; data select only inside the plot range
   idx = where ( ramp_x GE minx AND ramp_x LE maxx AND $
         ramp_y GE miny AND ramp_y LE maxy, cnt)

   if (cnt GT 0) then begin
      ramp_x = ramp_x[idx]      
      ramp_y = ramp_y[idx]      
   endif else begin
      ramp_x = [minx, maxx]
      ramp_y = [miny, maxy]
   endelse

   if ( n_elements (PSYM) EQ 0 ) then psym = 1

   if (NOT keyword_set(OVERLAY)) then $
      plot, ramp_x, ramp_y, PSYM = psym, $
         XRANGE=[minx, maxx], YRANGE=[miny, maxy], $
         TITLE=title, XTITLE=xtitle, YTITLE=ytitle, $
         XSTYLE=1, YSTYLE=1, $
         /NOCLIP, /YNOZERO, /NODATA, $
         _EXTRA = extra

   oplot, ramp_x, ramp_y, PSYM = psym, COLOR=color, _EXTRA = extra
   if (n_elements(ramp_e) GT 0) then $
      errplot, ramp_x, ramp_y-ramp_e, ramp_y+ramp_e, COLOR=color, _EXTRA = extra

   if (keyword_set(CURV_X) AND keyword_set(CURV_Y)) then $
      oplot, curv_x, curv_y, color=128, _EXTRA = extra

   ramp_x = 0B & ramp_y = 0B

end
