FUNCTION y_Poly_Smooth, xi, yi, xo, w, DEGREE=degree, ERROR=error, BLANK=blank, DEBUG=debug
; data smoothing with polinomial fit
; 06/10/06 I.Yamamura
; 06/10/19 Loop counter int -> long
; 07/01/12 debug I/F updated
; 07/02/02 renamed as y_poly_smooth to avoid conflict with astron/pro/poly_smooth.pro

if (N_ELEMENTS(debug) EQ 0) then deb = -1 else deb = debug

if (N_ELEMENTS(degree) EQ 1) then deg = degree else deg = 1

if (N_ELEMENTS(blank) NE 1) then blank = 0D


xdim = size(xi, /DIMENSIONS)
ydim = size(yi, /DIMENSIONS)
if (n_elements(xdim) NE 1) then begin
   MESSAGE, "Input X and Y should be vectors.!", /CONTINUE
   PRINT, xdim, ydim
   return, -1
endif
if (xdim NE ydim) then begin
   MESSAGE, "Input X and Y have inconsistent dimension!", /CONTINUE
   return, -1
endif

no = N_Elements(xo)
yo = replicate(blank, no)
eo = replicate(blank, no)

for i = 0L, no-1 do begin
   xx = xi - xo[i]
   idx = WHERE (abs(xx) LE w/2D, cnt)
   if (cnt LT deg * 2 + 2) then continue

   xx = xx[idx]
   yy = yi[idx]

   nu = N_ELEMENTS(UNIQ(xx[SORT(xx)]))
   if (nu LT deg * 2 + 2) then continue

   xmax = MAX(xx, min=xmin)
   if (xmax LT -w/4D OR xmin GT w/4D) then continue

   a = POLY_FIT (xx, yy, deg, SIGMA=sig, /DOUBLE)
   yo[i] = a[0]
   eo[i] = sig[0]

   if (N_ELEMENTS(deb) EQ 2) then begin
      if (i GE deb[0] AND i LE deb[1]) then begin
         xw = dindgen(w) - w/2D
         plot, xx, yy, psym=1
         oplot, xw, a[0] + a[1] * xw + a[2] * xw^2, color=240
         print, i, nu, a[0], a[1], a[2]
         stop
      endif
   endif

endfor

if (N_ELEMENTS(ERROR) GT 0) then ERROR = eo
return, yo
end
