FUNCTION y_Poly_Smooth_Ramp, xi, yi, w, XREF=xref, YRESD=yresd, ERROR=error, SIGMA=sigma, TSIGMA=tsigma, DEBUG=debug, DEBRANGE=debrange, EXTRA_X=extra_x
; polynomial fit smoothing for ramp curves
; 06/10/07 I.Yamamura
; 07/01/12 I.Yamamura add DEBRANGE. Change debug mode I/F
; 07/02/02 I.Yamamura poly_smooth renamed as y_poly_smooth
; 07/02/02 I.Yamamura rescale x to cover whole data range.
; 07/02/06 I.Yamamura extra_x inplemented to enable extrapolation of the ramps.

BAD = y_bad_val_def()

if (N_ELEMENTS(DEBUG) EQ 0) then deb = -1 else deb = debug

xdim = size(xi, /DIMENSIONS)
ydim = size(yi, /DIMENSIONS)
if ((n_elements(xdim) NE n_elements(ydim)) OR Total(xdim - ydim) NE 0) then begin
   MESSAGE, "Input X and Y have inconsistent dimension!"
endif
nsmpl = ydim[0]
nramp = ydim[1]
ndet  = ydim[2]

ya = REPLICATE(BAD, nsmpl, 1, ndet)
ye = REPLICATE(BAD, nsmpl, 1, ndet)
yd = REPLICATE(BAD, nsmpl, nramp, ndet)
xo = dindgen(nsmpl)
eo = REPLICATE(0D, nsmpl)
so = REPLICATE(0D, nsmpl)

for idet = 0,  ndet - 1 do begin
   xx = xi[*, *, idet]
   yy = yi[*, *, idet]
   idx = WHERE ((xx NE BAD) AND (yy NE BAD), cnt)
   if (cnt GT 0) then begin
      xx = xx[idx]
      yy = yy[idx]

      ; 07/02/02 rescale xx in order to cover the minimum and maximum
      ;          y value within regridded range
      xmin = MIN(xx, max=xmax)
      if (N_ELEMENTS(extra_x) EQ 0) then extra_x = [0, 0]
      if (N_ELEMENTS(extra_x) EQ 1) then extra_x = REPLICATE(extra_x, 2)
      if (xmin LT xmax) then xx = (xx - xmin) / (xmax - xmin) * DOUBLE(nsmpl-1 - extra_x[0] - extra_x[1]) + extra_x[0]

      if (idet EQ deb) then $
         yo = y_Poly_Smooth(xx, yy, xo, w, DEGREE=2, ERROR=eo, BLANK=BAD, DEBUG=debrange) $
      else $
         yo = y_Poly_Smooth(xx, yy, xo, w, DEGREE=2, ERROR=eo, BLANK=BAD)

      yyd = yi[*, *, idet]
      yyd[idx] = yy - INTERPOL(yo, xo, xx)

      ya[*, 0, idet] = yo
      ye[*, 0, idet] = eo
      yd[*, *, idet] = yyd
   endif
endfor

if (N_ELEMENTS(ERROR) GT 0) then error = ye
if (N_ELEMENTS(YRESD) GT 0) then yresd = yd

if (N_ELEMENTS(SIGMA) GT 0) then begin
   nn = REPLICATE(1D, nsmpl, nramp, ndet)
   idx = WHERE (yd EQ BAD, cnt)
   if (cnt GT 0) then nn[idx] = 0D
   ss = TOTAL ( (yd * nn)^2, 2 )
   nn = TOTAL ( nn, 2 ) - 1D
   idx = WHERE ( nn LE 0D, cnt )
   if (cnt GT 0) then nn[idx] = 1D
   sigma = sqrt(ss / nn)
   if (cnt GT 0) then sigma[idx] = BAD
endif

if (N_ELEMENTS(TSIGMA) GT 0) then begin
; calculate tsigma
   x1 = fix(nsmpl*0.2)
   x2 = fix(nsmpl*0.5)
   ss = sigma[x1:x2, *]
   nn = replicate(1D, (x2-x1+1), ndet)
   idx = where(ss EQ BAD, cnt)
   if (cnt GT 0) then nn[idx] = 0D
   ss = Total(ss * nn, 1)
   nn = Total(nn, 1)
   idx = where(nn EQ 0D, cnt)
   if (cnt GT 0) then nn[idx] = 1D
   tsigma = ss / nn
   if (cnt GT 0) then tsigma[idx] = BAD
endif

xr = REFORM(xo # REPLICATE(1D, ndet), nsmpl, 1, ndet, /OVER)
if (N_ELEMENTS(XREF) GT 0) then xref = xr

return, ya
end
