FUNCTION y_rconv, $
   tsd,                 $  ; TSD data
   pixbad,              $  ; (returned) PIX_BAD flag in the same dimension
   RANGE=range,         $  ; range in unit of sampling
   READRANGE=readrange, $  ; position of the last sampling returned
   DET=det,             $  ; read DED field instead of FLUX field
   FLAGGED=flagged,     $  ; bad data flagged as BAD
   WIDE=wide,           $  ; extract wide array
   NARROW=narrow,       $  ; extract narrow array
   SHIFT=shift,         $  ; WIDE-S may have shifted reset position
   GLITCH=glitch,       $  ; set bad flag on glitched ramps
   MYRESET=myreset,     $  ; reset position searched by my self (no shift)
   UNIQRST=uniqrst         ; only data with a uniq reset interval
;  (returned)              ; ramp[nsmpl, nramp, ndet]

; 03/03/24 return unique reset interval data set first found in the range
; 03/03/25 reconfiguration. add /UNIQRST option.
; 03/03/26 LAST option implemented
; 04/03/02 WIDES option
; 04/05/18 WIDES -> SHIFT, add WIDE/NARROW option
;          by default all pixel data is returned if reset timings of the
;          WIDE and NARROW channels are synchronized. Otherwise, only
;          WIDE channel is returned. /WIDE and /NARROW option can
;          specify the channel to retrieve.
; 04/05/20 since early May the 'after_reset' flag is set at actual
;          reset positions.
; 04/07/16 print allocated array size propery
; 04/07/30 add MYRESET. Search for reset position from reset conter
;          status within this program, instead of refering to afterrst flag. 
;          No shift for WIDE-S is applied.
; 04/10/29 moved to TSD3
; 04/12/24 /DED implemented
; 05/01/01 renamed as y_rconv.pro
; 05/01/20 /FLAGGED returned already BAD flagged data
; 05/01/20 BAD_FRAME is read and copied to pixbad.
; 05/01/23 minor bug fix on indexing.
; 05/01/24 initialize returned array ramp[] with BAD.
;          error code !FAILED -> 0
; 06/08/20 buf fix. /FLAGGED did not flag ramp data -> corrected.
; 06/08/25 /GLITCH flag added
; 06/09/29 Bug fix on /GLITCH flag treatment
; 06/10/04 use SATURATE or RESET or RSTANON or NO_DIFF instead of Pixel BAD

BAD = y_bad_val_def()

; Inquire data type
datatype = tsd->Get_DataType()
;     if (datatype EQ 'FIS_SW') then ndet = 100 $
;else if (datatype EQ 'FIS_LW') then ndet =  75 $
;else begin
;   msg = 'Invalid Data Type ('+STRTRIM(datatype)+')'
;   Message, msg
;endelse

; Read data from TSD
if (keyword_set(DET)) then $
   flux = Transpose(tsd->Get_Value_Tsd('FIS_OBS', 'DET')) $
else $
   flux = Transpose(tsd->Get_Value_Tsd('FIS_OBS', 'FLUX'))

fbad = tsd->Get_Flag_Tsd('FIS_OBS', 'BAD_FRAME')

;pbad = Transpose(tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'BAD'))
psat = tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'SATURATE')
prst = tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'RESET')
pran = tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'RSTANOM')
pndf = tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'NO_DIFF')
pbad = Transpose(psat OR prst OR pran OR pndf)


gp1 = Transpose(tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'GPGL_TYPE1'))
gp2 = Transpose(tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'GPGL_TYPE2'))
gp3 = Transpose(tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'GPGL_TYPE3'))
gp4 = Transpose(tsd->Get_Pix_Flag_Tsd('FIS_OBS', 'GPGL_TYPE4'))
gpa = gp1 OR gp2 OR gp3 OR gp4
gp1 = 0B & gp2 = 0B & gp3 = 0B & gp4 = 0B

; Read reset data from TSD

if (Keyword_Set(MYRESET)) then begin
   rst = pbad        ; same dimmension / type
   rst[*,*] = 0B

   if (datatype EQ 'FIS_SW') then begin
      rstc = FIX(tsd->get_value_tsd('FIS_OBS', 'RSTCNTWIDES'))
      rstd = rstc - shift(rstc,1)
      idx = where (rstd EQ 1 OR rstd EQ -15, nrst)
      if (nrst GT 0) then rst[idx, 0:59] = 1

      rstc = FIX(tsd->get_value_tsd('FIS_OBS', 'RSTCNTN60'))
      rstd = rstc - shift(rstc,1)
      idx = where (rstd EQ 1 OR rstd EQ -15, nrst)
      if (nrst GT 0) then rst[idx, 60:99] = 1
   endif else if (datatype EQ 'FIS_LW') then begin
      rstc = FIX(tsd->get_value_tsd('FIS_OBS', 'RSTCNTWIDEL'))
      rstd = rstc - shift(rstc,1)
      idx = where (rstd EQ 1 OR rstd EQ -15, nrst)
      if (nrst GT 0) then rst[idx, 0:44] = 1

      rstc = FIX(tsd->get_value_tsd('FIS_OBS', 'RSTCNTN170'))
      rstd = rstc - shift(rstc,1)
      idx = where (rstd EQ 1 OR rstd EQ -15, nrst)
      if (nrst GT 0) then rst[idx, 45:74] = 1
   endif
   rstc = 0B & rstd = 0B & idx = 0B
endif else begin
   rst = Transpose(tsd->Get_Pix_Flag_TSD('FIS_OBS', 'RESET'))
endelse

r_dim = size(flux, /DIMENSIONS)
if (N_Elements(r_dim) NE 2) then $
   MESSAGE, "Incorrect data dimension (expect [nsmpl, ndet])."+STRING(r_dim,/PRINT)
ndet = r_dim[1]

; Check if the reset timing is consistent within the arrays
     if (ndet EQ 100) then nx = 20 $
else if (ndet EQ 75)  then nx = 15

; WIDE array
rst0 = rst[*,0] # Replicate(1, nx*3)
idx = where (rst[*,0:(nx*3-1)] NE rst0, cnt)
if (cnt GT 0) then Message, "Reset timing inconsistent within WIDE detectors."

; NARROW array
rst0 = rst[*,nx*3] # Replicate(1, nx*2)
idx = where (rst[*,(nx*3):(ndet-1)] NE rst0, cnt)
if (cnt GT 0) then Message, "Reset timing inconsistent within NARROW detectors."
rst0 = 0B

; Set the reference reset timing
rstw = Where(rst[*,0]    EQ !TRUE, cntw)
rstn = Where(rst[*,nx*3] EQ !TRUE, cntn)

; Check the data range
if ( N_Elements(range) NE 2 ) then range = [0L, 0L]
if ( range[0] LE 0L ) then range[0] = 0L
if ( range[1] LE 0L OR range[1] GE r_dim[0] ) then range[1] = r_dim[0]-1L

if (NOT Keyword_Set(NARROW)) then begin

   idxx = where ((rstw GE range[0] AND rstw LE range[1]), cnt)
   if (cnt EQ 0) then begin
      Message, 'No reset in WIDE array in the given range. ['+STRTRIM(STRING(range[0]))+ ':'+STRTRIM(STRING(range[1]))+']', /CONTINUE
      readrange = [BAD, BAD]
;      return, !FAILED
      return, 0
   endif
   rstw = rstw[idxx]
   if (Keyword_Set(SHIFT)) then rstw = rstw + shift

   nsmpw = shift(rstw, -1) - rstw

endif

if (NOT Keyword_Set(WIDE)) then begin

   idxx = where ((rstn GE range[0] AND rstn LE range[1]), cnt)
   if (cnt EQ 0) then begin
      Message, 'No reset in NARROW array in the given range. ['+STRTRIM(STRING(range[0]))+ ':'+STRTRIM(STRING(range[1]))+']', /CONTINUE
      readrange = [BAD, BAD]
;      return, !FAILED
      return, 0
   endif
   rstn = rstn[idxx]
   if (Keyword_Set(SHIFT)) then rstn = rstn + shift

   nsmpn = shift(rstn, -1) - rstn

endif

if (Keyword_Set(NARROW)) then begin

   MESSAGE, 'Only NARROW pixel data are returned ', /CONTINUE
   rdet = indgen(nx * 2) + nx * 3
   rst0 = rstn
   nsmpl = nsmpn

endif else if (Keyword_Set(WIDE)) then begin

   MESSAGE, 'Only WIDE pixel data are returned ', /CONTINUE
   rdet = indgen(nx * 3)
   rst0 = rstw
   nsmpl = nsmpw

endif else begin

   idx = WHERE (rstw NE rstn, cnt)
   if (cnt GT 0) then begin
      MESSAGE, 'Reset timing inconsistent between WIDE and NARROW.', /CONTINUE
      MESSAGE, 'Only WIDE pixel data are returned ', /CONTINUE

      rdet = indgen(nx * 3)
      rst0 = rstw
      nsmpl = nsmpw

   endif else begin
      MESSAGE, 'All pixel data are returned ', /CONTINUE

      rdet = indgen(nx * 5)
      rst0 = rstw
      nsmpl = nsmpw

   endelse

endelse

nsmpl_max = max(nsmpl)

if (Keyword_Set(UNIQRST)) then begin
   idx = WHERE(nsmpl EQ nsmpl[0], nramp)
   idr = INDGEN(nramp) - idx
   idx = idx[WHERE (idr EQ 0, nramp)]

   ramp   = replicate(BAD, nsmpl[0], nramp, ndet)
   pixbad = replicate(1B, nsmpl[0], nramp, ndet)
   gltbad = replicate(1B, nsmpl[0], nramp, ndet)
   frmbad = replicate(1B, nsmpl[0], nramp)
endif else begin
   nramp = n_elements(rst0) - 1
   ramp   = replicate(BAD, nsmpl_max, nramp, ndet)
   pixbad = replicate(1B, nsmpl_max, nramp, ndet)
   gltbad = replicate(1B, nsmpl_max, nramp, ndet)
   frmbad = replicate(1B, nsmpl_max, nramp)
endelse

for j = 0, nramp - 1 do begin
   ramp[0:(nsmpl[j]-1), j, rdet] = flux[(rst0[j]):(rst0[j+1]-1), rdet]
   pixbad[0:(nsmpl[j]-1), j, rdet] = pbad[(rst0[j]):(rst0[j+1]-1), rdet]
   gltbad[0:(nsmpl[j]-1), j, rdet] =  gpa[(rst0[j]):(rst0[j+1]-1), rdet]
   frmbad[0:(nsmpl[j]-1), j] = fbad[(rst0[j]):(rst0[j+1]-1)]
endfor

if (Keyword_Set(GLITCH)) then begin
   tgltbad = Total(gltbad, 1)
   idx = where(tgltbad NE 0, cnt)
   if (cnt GT 0) then begin
      print, cnt, "ramps are flagged as glitched."
      pixbad_t = reform(pixbad, nsmpl_max, nramp * ndet)
      pixbad_t[*, idx] = !TRUE
      pixbad = reform(pixbad_t, nsmpl_max, nramp, ndet)
   endif
endif

if (Keyword_Set(FLAGGED)) then begin
   ibad = where (frmbad GT 0, cbad)
   if (cbad GT 0) then begin
      is = ibad MOD nsmpl
      ir = ibad / nsmpl
      pixbad[is, ir, *] = 1
   endif
   ibad = where (pixbad GT 0, cbad)
   if (cbad GT 0) then ramp[ibad] = BAD
endif

readrange = [rst0[0], rst0[j]-1]

if (Keyword_Set(UNIQRST)) then nsmpl = nsmpl[0] else nsmpl = max(nsmpl)
print, 'ramp[',nsmpl, ',', nramp, ',', ndet,'] created.'
print, "Read Range = ", readrange

return, ramp
end
