FUNCTION y_Regrid_Ramps,   $
   xin,                    $ ; input x data
   yin,                    $ ; input y data
   XREF=xref,              $ ; (in/out) reference grid.
   AUTOGRID=autogrid         ; automatic grid calculation regardless XREF

; regrid ramps in a given or automatic calculated equally space grid X-grid
; 05/01/17 separated from y_avarage_ramps.pro
; 05/01/20 add /DOUBLE in SPLINE()
; 05/01/20 use SPLINE only within the input X range to avoid numerical errors.
; 05/08/18 reflect BAD from original data
; 06/10/06 use INTERPOL rather than SPLINE

   BAD = y_bad_val_def()

; dimension check
   x_dim = size(xin, /DIMENSIONS)
   if (n_elements(x_dim) NE 3) then begin
      MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(x_dim,/PRINT), /CONTINUE
      return, 0
   endif

   y_dim = size(yin, /DIMENSIONS)
   if (n_elements(y_dim) NE 3) then begin
      MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(y_dim,/PRINT), /CONTINUE
      return, 0
   endif

   if (Total((x_dim-y_dim)^2 NE 0)) then begin
      MESSAGE, "X, Y dimension inconsistent!", /CONTINUE
      return, 0
   endif

   nsmpl = x_dim[0]
   nramp = x_dim[1]
   ndet  = x_dim[2]

   ngrid = x_dim[0]

; check dimension of xref
   
   ext_xref = 0
   if ((NOT keyword_set(AUTOGRID)) AND (n_elements(xref) GT 0)) then begin
      r_dim = size(xref, /DIMENSIONS)
      if (n_elements(r_dim) NE 3) then begin
         MESSAGE, "xref dimension (expect [ngrid, nramp, ndet])."+STRING(r_dim,/PRINT), /CONTINUE
         return, 0
      endif
      
      if (Total((x_dim[1:2]-r_dim[1:2])^2 NE 0)) then begin
         MESSAGE, "nramp or ndet of xref inconsistent with xin!", /CONTINUE
         return, 0
      endif
      ngrid = r_dim[0]
      ext_xref = 1
   endif else begin
      xref = replicate(BAD, ngrid, nramp, ndet)
   endelse

; allocate array for output
   ynew = replicate(BAD, ngrid, nramp, ndet)

   for idet = 0, ndet-1 do begin
;     extract data of the pixel and reform in one dimension
      x0 = reform (xin[*, *, idet], nsmpl, nramp)
      y0 = reform (yin[*, *, idet], nsmpl, nramp)

;     if reference grid is not given, calculate here
      if (ext_xref EQ 1) then begin
         xr = xref[*, 0, idet]
      endif else begin

;     select healthy data
         idx = where ( x0 NE BAD AND y0 NE BAD, cnt )
;        if data is too few avarage is not possible, go to next pixel
         if (cnt LT 2) then continue

;     define grid for the averaged data
         xmin = min(x0[idx], max=xmax)
         xr = dindgen(ngrid) / double(ngrid-1) * (xmax - xmin) + xmin
         xref[*, *, idet] = xr # replicate(1D, nramp)
      endelse

; regrid every ramps
      for iramp = 0, nramp-1 do begin
         x = x0[*, iramp]
         y = y0[*, iramp]
         idx = where (x NE BAD AND y NE BAD, cnt)
;         if (cnt LE 2) then continue
         if (cnt LE 3) then continue

         x = x[idx]
         y = y[idx]

         idx = sort(x)
         x = x[idx]
         y = y[idx]

         xmin = min(x, max=xmax)
         idx = where (xr NE BAD AND xr GE xmin AND xr LE xmax, cnt)

         if (cnt GT 0) then begin
            idx2 = sort(xr[idx])
            idx = idx[idx2]

;            ynew[idx, iramp, idet] = SPLINE(x, y, xr[idx], /DOUBLE)
            ynew[idx, iramp, idet] = INTERPOL(y, x, xr[idx], /QUADRATIC)
         endif

         dx = shift(x, -1) - x
         idx = where (dx GT median(dx) * 1.5, cnt)
         if (cnt GT 0) then begin
            for i = 0, cnt-1 do begin
               xb = x[idx[i]]
               xe = x[idx[i]+1]
               jdx = where (xr GT xb AND xr LT xe, jcnt)
               if (jcnt GT 0) then ynew[jdx, iramp, idet] = BAD
            endfor
         endif

      endfor
   endfor
   x = 0B & y = 0B & x0 = 0B & y0 = 0B & xr = 0B & yr = 0B

   xref = reform(xref, ngrid, nramp, ndet, /OVERWRITE)
   ynew = reform(ynew, ngrid, nramp, ndet, /OVERWRITE)

   return, ynew
end
