PRO y_Stat_Ramp,    $  ; Statistics of Ramp data
   ramp,             $  ; ramp[nsmpl, nramps, ndets]
   average,          $  ; average value: average[ndets]
   stdev,            $  ; Std. Dev value: stdev[ndets]
   DIFF=diff            ; do differentiation if /DIFF option is given

; 04/12/20 I.Yamamura
; calculate average and standard deviation of ramp data
; data is assumed to be differentiated, othewise /DIFF is given
; 05/01/01 renamed with 'y_'

if (n_params() LT 3) then begin
   MESSAGE, "Usage: y_Stat_Ramp, ramp, average, stdev[, /DIFF]", /CONTINUE
   return
endif

BAD = y_bad_val_def()

r_dim = size(ramp, /DIMENSIONS)
if (n_elements(r_dim) NE 3) then begin
   MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(r_dim,/PRINT)
   return
endif
nsmpl = r_dim[0]
nramp = r_dim[1]
ndet  = r_dim[2]

if (keyword_set(DIFF)) then dramp = y_diff_ramp(ramp) $
                       else dramp = ramp

idx = where (dramp EQ BAD, cnt)

nd = replicate(1D, nsmpl, nramp, ndet)

if (cnt GT 0) then begin
   nd[idx] = 0D
   dramp[idx] = 0D
endif

tdr = Total(reform(dramp, nsmpl*nramp, ndet), 1)
tnd = Total(reform(nd, nsmpl*nramp, ndet), 1)

idz = where(tdr EQ 0D, cntz)
if (cntz GT 0) then tnd[idz] = 1D

average = tdr / tnd

adr = replicate(1D, nsmpl*nramp) # average
adr = reform(adr, nsmpl, nramp, ndet, /OVERWRITE)

sdr = dramp - adr
if (cntz GT 0) then sdr[idz] = 0D

sdr = Total(reform(sdr, nsmpl*nramp, ndet)^2, 1)
stdev = Sqrt(sdr / tnd)

return
end
