PRO y_regridx
; 03/05/20 range and SCALE are set relative to the ramp length
;          and avoiding the first sampling point.
; 04/07/20 this automatically provide data within the fitting range given
;          in the main routine.
; 05/01/17 use y_bad_val_def()
; 05/01/20 add /DOUBLE in SPILINE()
; 05/01/20 properly ignore BAD data to avoid floating errors
; 05/01/20 xf and xr are guaranteed to be increasing order and not BAD
;          use yf0 directly
; 05/02/06 interploate reference ramp instead of fitted ramp for speed up.
;          FUNCTION -> PRO using common variables
; 06/10/09 automatic reference ramp selection takes highest one rather than
;          lowest one
; 06/10/09 xout addted
; 06/10/11 give up for very low level ramps (mostly dead pixels)
; 06/10/17 skip if unique X or Y component are less than 3.
; 06/10/19 unique data check also for reference data
; 07/01/22 automatic reference ramp selection considers ramps with
;          smallest inclination
; 07/06/03 upper limit for pf_r[1] 30000 -> 100000 to accept dark ramp.

common y_acomx, xr, yr, yr2, xrmin, xrmax, xf, yf, xf0, yf0, nf, c2, BAD

ix = where (xf GE xrmin AND xf LE xrmax, cnt, comp=ixc, ncomp=cntc)
;if (cnt GT 0) then yf[ix] = SPL_INTERP(xr, yr, yr2, xf[ix], /DOUBLE)
;if (cnt GT 0) then yf[ix] = INTERPOL(yr, xr, xf[ix], /LSQUADRATIC)
if (cnt GT 0) then yf[ix] = INTERPOL(yr, xr, xf[ix], /QUADRATIC)
if (cntc GT 0) then yf[ixc] = BAD

return
end

FUNCTION y_afitxf, d
; 05/01/20 yr is guarantteed not to be BAD in the main program.

common y_acomx, xr, yr, yr2, xrmin, xrmax, xf, yf, xf0, yf0, nf, c2, BAD

; d[1] should be > 0
   if (d[1] LE 0D) then return, abs(BAD)

; print, d

   xf = xf0 * d[1] + d[0]
   y_regridx               ; result is in yf

jdx = where (yf0 EQ BAD, jnt)
if (jnt GT 0) then print, yf0, 'yf0 is bad'

   idx = where (yf NE BAD, cnt)
   if (cnt GT 1) then $
      c2 = Total((yf[idx] - yf0[idx])^2 / double(cnt-1)) $
   else $
      c2 = abs(BAD)
   return, c2
end

; ==========================================================================
FUNCTION y_xsftscl, $
   xrmp, $       ; x-axis index (double)
   ramp, $       ; array of ramp curve. ramp[samplings, ramp_id, det_id]
   dx,   $       ; x-axis index (double)
   fx,   $       ; x-axis scale factor (double)
   PLOTRES=plotres,  $; plot fitting result
   EXTREFX=extrefx,  $; external reference ramp (X; same dimension as xrmp)
   EXTREFY=extrefy,  $; external reference ramp (Y; same dimension as yrmp)
   REFRAMP=reframp,  $; reference ramp #
   NOSCALE=noscale,  $; do not scale
   VERBOSE=verbose,  $; print fitting parameters
   NOAMOEBA=noamoeba,$; no use of amoeba fitting
   RANGE=range,      $; x-axis range to be used for fitting
   HIGHREF=highref,  $; take the highest ramp as the reference (default lowest)
   YFITRANGE=yfitrange, $; y-axis range to be used for fitting
   XOUT=xout,        $; x-axis index for output data = x * fx + dx
   SIG=sig,          $; standard deviation double[ramp_id, det_id]
   _EXTRA=_extra
; (returned)     ; !SUCEED if suceeded
;
; 04/05/** extended from itrshiftx.pro
; 04/07/08 automatic rougn scaling as amoeba initial parameter
; 04/07/08 add NOAMOEBA option
; 04/07/09 do fitting only when more than 3 data points are valid
; 04/07/15 short cut if target = reference
; 04/07/20 fitting range is now for after shift & scale
; 04/11/25 skip when all y data are the same
; 04/12/31 add /NOSCALE option
; 05/01/01 renamed with 'y_'
; 05/01/17 use y_bad_val_def()
; 05/01/20 properly skip BAD ramps to avoid floating errors
; 05/01/20 tolerance changed 1e-8 => 1e-5.
;          The results (number of itteration) does not change so much.
; 05/01/20 bug fix to determine referrence ramp properly
; 05/01/28 give smaller Amoeba scale parameter to improve conversion.
; 05/01/29 return xf = 0 <- BAD when error to make x*fx+dx = BAD
; 05/02/05 further optimization of Amoeba scale. Better messaging.
; 05/02/06 further speed up by improving SPLINE interpolation.
;          tolerance changed 1e-5 => 1e-4.
;          Total speed up by factor of ~3.
;          Note initial guess of df[0] does not speed up.
; 05/02/19 Can treat reference ramp given externally. Code cleaned up.
; 07/06/15 Simpler way to find the ramp with the lowest tilt
; 07/06/18 Negative scaling factor reset to 1.0.
; 07/09/30 HIGHREF option implemented
; 07/09/30 YFITRANGE option implemented

common y_acomx, xr, yr, yr2, xrmin, xrmax, xf, yf, xf0, yf0, nf, c2, BAD

BAD = y_bad_val_def()

r_dim = SIZE(ramp, /DIMENSIONS)
if (N_ELEMENTS(r_dim) NE 3) then begin
   MESSAGE, "Incorrect ramp dimension (expect [nsmpl, nramp, ndet])."+STRING(r_dim,/PRINT)
   return, 0
endif
nsmpl = r_dim[0]
nramp = r_dim[1]
ndet  = r_dim[2]

NAMBMAX = 300        ; maximum Amoeba itteration

dx = DBLARR(nramp, ndet)
fx = DBLARR(nramp, ndet)
sg = DBLARR(nramp, ndet)

r0 = REPLICATE (1D, nsmpl) 

; find out the fitting range
if ( N_ELEMENTS(range) EQ 2 ) then begin
   if (range[0] LT 0) then range[0] = 0
   if (range[1] GT nsmpl-1) then range[1] = nsmpl - 1
   if (range[0] GT range[1]) then range = reverse(range)
endif else begin
   range = [0, fix(nsmpl*0.8)+1]
endelse

; inquire reference ramp
if (N_ELEMENTS(REFRAMP) EQ 1) then begin
   if (reframp LT 0 OR reframp GE nramp) then begin
      MESSAGE, 'Given REFRAMP out of range. ('+STRTRIM(STRING(reframp))+'). Turn to automatic mode.', /CONTINUE
      reframp = -1
   endif
endif else reframp = -1

; inquire external reference ramp
if (N_ELEMENTS(extref) GT 0) then begin
   e_dim = size(extref, /DIMENSIONS);
   if ((e_dim[0] NE nsmpl) OR (e_dim[1] NE 1) OR (e_dim[2] NE ndet)) then begin
      MESSAGE, "Invalid dimension of reference ramp."+STRING(e_dim,/PRINT)
      return, 0
   endif
endif

; main loop for correction
for idet = 0, ndet-1 do begin
;   print, FORMAT='(A, I4)', 'Det #', idet+1

; set the reference ramp
   if (N_ELEMENTS(extrefy) GT 0) then begin
      yr = extrefy[*, 0, idet]
      if (N_ELEMENTS(extrefx) EQ N_ELEMENTS(extrefy)) then begin
         xr = extrefx[*, 0, idet]
      endif else begin
         xr = dindgen(nsmpl)
      endelse
      iref = -1
   endif else begin

      if (reframp GE 0) then begin
         iref = reframp
         xr = xrmp[*,iref,idet]
         yr = ramp[*,iref,idet]
      endif else begin
         iref = 0    ; default

         if (KEYWORD_SET(HIGHREF)) then begin

            yz = ramp[*, *, idet]
            iyz = WHERE (yz NE BAD, cnt)
            if (cnt GT 0) then begin

; 06/10/09 min -> max
;               ymin = min(ramp[iyz], iref)
               ymax = MAX(ramp[iyz], iref)
               iref = iyz[iref] / nsmpl
            endif

         endif else begin
; 07/01/22 take ramp with smallest inclination
;            pf_a = REPLICATE(BAD, 2, nramp)
;            for ir = 0, nramp - 1 do begin
;               xx = xrmp[*, ir, idet]
;               yy = ramp[*, ir, idet]
;               idx = WHERE(xx NE BAD AND yy NE BAD, cnt)
;               if (cnt GT 0) then begin
;                  pf_a[*, ir] = POLY_FIT(yy, xx, 1, /DOUBLE)
;               endif else begin
;                  pf_a[*, ir] = REPLICATE(BAD, 2)
;               endelse
;            endfor
;            idx = WHERE(pf_a[1, *] NE BAD, cnt)
;            if (cnt GT 0) then begin
;               ymin = MIN(pf_a[1, idx], iref)
;               iref = idx[iref]
;            endif else begin
;               iref = 0
;            endelse

; 07/06/15 simpler way
            yymax = MAX(ramp[*, *, idet], xxmax, DIMENSION=1, /NAN)
            idx = WHERE(yymax NE BAD AND xxmax NE 0, cnt)
            if (cnt GT 0) then begin
               yymax[idx] = yymax[idx] / DOUBLE(xxmax[idx])
               yymaxmin = MIN(yymax[idx], iref)
               iref = idx[iref]
            endif
         endelse
         xr = xrmp[*,iref,idet]
         yr = ramp[*,iref,idet]
      endelse
   endelse

   if (N_ELEMENTS(YFITRANGE) EQ 2) then begin
      idx = WHERE (yr NE BAD, cnt)
      if (cnt EQ 0) then continue
      ymin = MIN(yr[idx], MAX=ymax)
      yrange = ymin + (ymax - ymin) * yfitrange
   endif
   if (N_ELEMENTS(yrange) EQ 2) then begin
      idxr = WHERE ((xr NE BAD AND yr NE BAD) AND $
            (yr GE yrange[0] AND yr LE yrange[1]), cntr)
   endif else begin
      idxr = WHERE ((xr NE BAD AND yr NE BAD) AND $
            (xr GE range[0] AND xr LE range[1]), cntr)
   endelse

   if (cntr GE 3) then begin
      idxr2 = SORT(xr[idxr])
      idxr = idxr[idxr2]
      xr = xr[idxr]
      yr = yr[idxr]
   endif else begin
      msg = STRING(FORMAT='(A, I3, A)', 'Det #', idet+1, ' No valid reference data.')
      MESSAGE, msg, /CONTINUE
      dx[*,idet] = BAD
      fx[*,idet] = 0D
      sg[*,idet] = BAD
      continue
   endelse

; prepare for spline interpolation in y_regridx
   xrmin = min(xr, max=xrmax)
;   yr2 = SPL_INIT(xr, yr, /DOUBLE)

   idxp = WHERE (yr NE yr[0], cntp)
   if (N_ELEMENTS(uniq(xr[sort(xr)])) LT 3 $
      OR N_ELEMENTS(uniq(yr[sort(yr)])) LT 3) then begin
      dx[*,idet] = BAD
      fx[*,idet] = 0D
      sg[*,idet] = BAD
      continue
   endif

; rough scaling
;; reference data
   pf_r =  POLY_FIT(yr, xr, 1, /DOUBLE)

   namb_tot = 0L & nnamb = 0
   for iramp = 0, nramp-1 do begin
      namb = 0
      if (iramp EQ iref) then begin
         df = [0D, 1D]
         c2 = 0D
      endif else if (abs(pf_r[1]) GT 1D7) then begin
      ; 07/06/03 30000 -> 100000
      ; 07/06/15 100000 -> 1D6
         df = [0D, 1D]
         C2 = 0D
      endif else begin

         xf0 = xrmp[*,iramp,idet]
         yf0 = ramp[*,iramp,idet]
         idx = where ((xf0 NE BAD AND yf0 NE BAD), cnt)
         if (cnt GE 3) then begin
            idx2 = sort(xf0[idx])
            idx = idx[idx2]
            xf0 = xf0[idx]
            yf0 = yf0[idx]
         endif else begin
            dx[iramp,idet] = BAD
            fx[iramp,idet] = 0D
            sg[iramp,idet] = BAD
            continue
         endelse

         if (N_ELEMENTS(uniq(xf0[sort(xf0)])) LT 3 $
            OR N_ELEMENTS(uniq(yf0[sort(yf0)])) LT 3) then begin
            dx[iramp,idet] = BAD
            fx[iramp,idet] = 0D
            sg[iramp,idet] = BAD
            continue
         endif       

; prepare for y_regridx (allocate array)
         xf = xf0
         yf = yf0

; rough scaling and set the initial parameters
         pf =  POLY_FIT(yf0, xf0, 1, /DOUBLE)
;print, pf[0], pf_r[0], pf[1], pf_r[1]
;         df = [pf_r[0]-pf[0], pf_r[1]/pf[1]]
         df = [0D, pf_r[1]/pf[1]]
         if (df[1] LE 0D) then df[1] = 1D
;        sc = [0.5D, 0.01D]
         ; 07/06/16 changed relative to scaling factor.
         ; no difference between 0.01 and 0.03 
         sc = [0.5D, df[1] * 0.03D]

         if (keyword_set(NOSCALE)) then begin
            df = [0D, 1D]
            sc = [0.5D, 0D]
         endif

         c2 = 0D
         if (NOT Keyword_Set(NOAMOEBA)) then begin
         ; 07/06/16 no diffrence between tolerance = 1D-4 and 1D-3
            df = amoeba(1D-3, FUNCTION_Name="y_afitxf", $
               P0=df, SCALE=sc, NCALLS=namb, NMAX=NAMBMAX)
            namb_tot += namb & nnamb += 1
         endif
      endelse

      if (FINITE(df[0]) AND namb LT NAMBMAX) then begin
         if (Keyword_Set(VERBOSE)) then print, idet, df
         dx[iramp,idet] = df[0]
         fx[iramp,idet] = df[1]
         sg[iramp,idet] = sqrt(c2)
      endif else begin
         if (namb GE NAMBMAX) then $
            MESSAGE, 'Amoeba did not converge at Ramp #'+STRTRIM(STRING(iramp, /PRINT), 2)+'. Namb = '+STRTRIM(STRING(namb, /PRINT), 2), /CONTINUE
         dx[iramp,idet] = BAD
         fx[iramp,idet] = 0D
         sg[iramp,idet] = BAD
      endelse
   endfor
   if (nnamb GT 0) then namb_tot /= double(nnamb)
   msg = STRING(FORMAT='(A, I3, A, I3, A, F5.1, A, I4)', 'Det #', idet+1, '  iref = ', iref, '  namb(avg) = ', namb_tot, '  nnamb = ', nnamb)
   MESSAGE, msg, /CONTINUE

   if (Keyword_Set(PLOTRES)) then begin
      pdx = REFORM(r0 # dx[*,idet], nsmpl, nramp)
      pfx = REFORM(r0 # fx[*,idet], nsmpl, nramp)
      x = xrmp[*,*,idet] * pfx + pdx
      idx = where ( pfx EQ BAD or pdx EQ BAD, cnt)
      if (cnt GT 0) then x[idx] = BAD
      y_plot_ramp, x, ramp[*,*,idet], TITLE="Det. #"+STRING(idet+1, FORMAT='(I4)'), psym=3, _EXTRA=_extra
;if (idet EQ 29) then stop
;wait,1
      pdx = 0B & pfx = 0B & x = 0B
   endif
endfor

r = r0 # REFORM(dx, nramp * ndet)
dx = REFORM(r, nsmpl, nramp, ndet)
r = r0 # REFORM(fx, nramp * ndet)
fx = REFORM(r, nsmpl, nramp, ndet)

if (KEYWORD_SET(XOUT) OR N_ELEMENTS(XOUT) EQ 1) then begin
   xout = xrmp * fx + dx
   idx = WHERE (x EQ BAD OR dx EQ BAD, cnt)
   if (cnt GT 0) then xout[idx] = BAD
endif

sig = REPLICATE (1D, nsmpl) # REFORM(sg, nramp * ndet)
sig = REFORM(sig, nsmpl, nramp, ndet, /OVERWRITE)

r = 0B & r0 = 0B

;return, !SUCCEED
return, 0
end
